@@ -2,9 +2,9 @@
 #
 # Configuration file for the Sphinx documentation builder.
 #
-# This file does only contain a selection of the most common options. For a
-# full list see the documentation:
-# http://www.sphinx-doc.org/en/master/config
+# This file only contains a selection of the most common options. For a full
+# list see the documentation:
+# https://www.sphinx-doc.org/en/master/usage/configuration.html
 
 from glob import glob
 import os.path
@@ -117,17 +117,10 @@ copyright = "fish-shell developers"
 author = "fish-shell developers"
 issue_url = "https://github.com/fish-shell/fish-shell/issues"
 
-# Parsing FISH-BUILD-VERSION-FILE is possible but hard to ensure that it is in the right place
-# fish_indent is guaranteed to be on PATH for the Pygments highlighter anyway
-if "FISH_BUILD_VERSION_FILE" in os.environ:
-    f = open(os.environ["FISH_BUILD_VERSION_FILE"], "r")
-    ret = f.readline().strip()
-elif "FISH_BUILD_VERSION" in os.environ:
-    ret = os.environ["FISH_BUILD_VERSION"]
-else:
-    ret = subprocess.check_output(
-        ("../build_tools/git_version_gen.sh", "--stdout"), stderr=subprocess.STDOUT
-    ).decode("utf-8")
+# From Cargo, or no build system.
+ret = subprocess.check_output(
+    ("../build_tools/git_version_gen.sh"), stderr=subprocess.STDOUT
+).decode("utf-8")
 
 # The full version, including alpha/beta/rc tags
 release = ret.strip().split(" ")[-1]
