@@ -7,7 +7,7 @@ use crate::flogf;
 use crate::io::IoChain;
 use crate::parser::Parser;
 use crate::wutil::{FileId, INVALID_FILE_ID, file_id_for_path};
-use fish_wchar::{L, WExt, WString, wstr};
+use fish_widestring::{L, WExt, WString, wstr};
 use lru::LruCache;
 use rust_embed::RustEmbed;
 use std::collections::{HashMap, HashSet};
@@ -51,11 +51,13 @@ enum AssetDir {
     Completions,
 }
 
+#[derive(Debug)]
 pub enum AutoloadPath {
     Embedded(String),
     Path(WString),
 }
 
+#[derive(Debug)]
 pub enum AutoloadResult {
     Path(AutoloadPath),
     Loaded,
@@ -112,7 +114,7 @@ impl Autoload {
                 );
             }
             AutoloadResult::Loaded | AutoloadResult::Pending | AutoloadResult::None => {}
-        };
+        }
         result
     }
 
@@ -455,6 +457,7 @@ mod tests {
     use super::{Autoload, AutoloadResult};
     use crate::prelude::*;
     use crate::tests::prelude::*;
+    use assert_matches::assert_matches;
 
     #[test]
     #[serial]
@@ -468,7 +471,7 @@ mod tests {
             ( $fmt:expr $(, $arg:expr )* $(,)? ) => {
                  let cmd = wcs2zstring(&sprintf!($fmt $(, $arg)*));
                  let status = unsafe { libc::system(cmd.as_ptr()) };
-                 assert!(status == 0);
+                 assert_eq!(status, 0);
             };
         }
 
@@ -505,64 +508,67 @@ mod tests {
         autoload.invalidate_cache();
 
         assert!(!autoload.autoload_in_progress(L!("file1")));
-        assert!(matches!(
+        assert_matches!(
             autoload.resolve_command_impl(L!("file1"), paths),
             AutoloadResult::Path(_)
-        ));
-        assert!(matches!(
+        );
+        assert_matches!(
             autoload.resolve_command_impl(L!("file1"), paths),
             AutoloadResult::Pending
-        ));
+        );
         assert!(autoload.autoload_in_progress(L!("file1")));
-        assert!(autoload.get_autoloaded_commands() == vec![L!("file1")]);
+        assert_eq!(autoload.get_autoloaded_commands(), vec![L!("file1")]);
         autoload.mark_autoload_finished(L!("file1"));
         assert!(!autoload.autoload_in_progress(L!("file1")));
-        assert!(autoload.get_autoloaded_commands() == vec![L!("file1")]);
+        assert_eq!(autoload.get_autoloaded_commands(), vec![L!("file1")]);
 
-        assert!(matches!(
+        assert_matches!(
             autoload.resolve_command_impl(L!("file1"), paths),
             AutoloadResult::Loaded
-        ));
+        );
         assert!(
             autoload
                 .resolve_command_impl(L!("nothing"), paths)
                 .is_none()
         );
         assert!(autoload.resolve_command_impl(L!("file2"), paths).is_some());
-        assert!(matches!(
+        assert_matches!(
             autoload.resolve_command_impl(L!("file2"), paths),
             AutoloadResult::Pending
-        ));
+        );
         autoload.mark_autoload_finished(L!("file2"));
-        assert!(matches!(
+        assert_matches!(
             autoload.resolve_command_impl(L!("file2"), paths),
             AutoloadResult::Loaded
-        ));
-        assert!((autoload.get_autoloaded_commands() == vec![L!("file1"), L!("file2")]));
+        );
+        assert_eq!(
+            autoload.get_autoloaded_commands(),
+            vec![L!("file1"), L!("file2")]
+        );
 
         autoload.clear();
         assert!(autoload.resolve_command_impl(L!("file1"), paths).is_some());
         autoload.mark_autoload_finished(L!("file1"));
-        assert!(matches!(
+        assert_matches!(
             autoload.resolve_command_impl(L!("file1"), paths),
             AutoloadResult::Loaded
-        ));
+        );
         assert!(
             autoload
                 .resolve_command_impl(L!("nothing"), paths)
                 .is_none()
         );
         assert!(autoload.resolve_command_impl(L!("file2"), paths).is_some());
-        assert!(matches!(
+        assert_matches!(
             autoload.resolve_command_impl(L!("file2"), paths),
             AutoloadResult::Pending
-        ));
+        );
         autoload.mark_autoload_finished(L!("file2"));
 
-        assert!(matches!(
+        assert_matches!(
             autoload.resolve_command_impl(L!("file1"), paths),
             AutoloadResult::Loaded
-        ));
+        );
         touch_file(&sprintf!("%s/file1.fish", p1));
         autoload.invalidate_cache();
         assert!(autoload.resolve_command_impl(L!("file1"), paths).is_some());
