@@ -23,11 +23,11 @@ use fish::{
         fish_indent, fish_key_reader,
         shared::{
             BUILTIN_ERR_MISSING, BUILTIN_ERR_UNEXP_ARG, BUILTIN_ERR_UNKNOWN, STATUS_CMD_ERROR,
-            STATUS_CMD_OK, STATUS_CMD_UNKNOWN,
+            STATUS_CMD_OK, STATUS_CMD_UNKNOWN, VERSION_STRING_TEMPLATE,
         },
     },
     common::{
-        PACKAGE_NAME, PROFILING_ACTIVE, PROGRAM_NAME, bytes2wcstring, escape,
+        PACKAGE_NAME, PROFILING_ACTIVE, PROGRAM_NAME, bytes2wcstring, escape, osstr2wcstring,
         save_term_foreground_process_group, wcs2bytes,
     },
     env::{
@@ -46,7 +46,7 @@ use fish::{
     panic::panic_handler,
     parse_constants::{ParseErrorList, ParseTreeFlags},
     parse_tree::ParsedSource,
-    parse_util::parse_util_detect_errors_in_ast,
+    parse_util::detect_parse_errors_in_ast,
     parser::{BlockType, CancelBehavior, Parser, ParserEnvSetMode},
     path::path_get_config,
     prelude::*,
@@ -62,6 +62,7 @@ use fish::{
     wutil::waccess,
 };
 use libc::STDIN_FILENO;
+use nix::unistd::AccessFlags;
 use std::ffi::{OsStr, OsString};
 use std::fs::File;
 use std::os::unix::prelude::*;
@@ -143,7 +144,7 @@ fn source_config_in_directory(parser: &Parser, dir: &wstr) -> bool {
     // this context so we ignore it.
     let config_pathname = dir.to_owned() + L!("/config.fish");
     let escaped_pathname = escape(dir) + L!("/config.fish");
-    if waccess(&config_pathname, libc::R_OK) != 0 {
+    if waccess(&config_pathname, AccessFlags::R_OK).is_err() {
         flogf!(
             config,
             "not sourcing %s (not readable or does not exist)",
@@ -174,10 +175,7 @@ fn read_init(parser: &Parser, paths: &ConfigPaths) {
         eprintf!("%s", msg);
     }
 
-    source_config_in_directory(
-        parser,
-        &bytes2wcstring(paths.sysconf.as_os_str().as_bytes()),
-    );
+    source_config_in_directory(parser, &osstr2wcstring(&paths.sysconf));
 
     // We need to get the configuration directory before we can source the user configuration file.
     // If path_get_config returns false then we have no configuration directory and no custom config
@@ -190,12 +188,12 @@ fn read_init(parser: &Parser, paths: &ConfigPaths) {
 fn run_command_list(parser: &Parser, cmds: &[OsString]) -> Result<(), libc::c_int> {
     let mut retval = Ok(());
     for cmd in cmds {
-        let cmd_wcs = bytes2wcstring(cmd.as_bytes());
+        let cmd_wcs = osstr2wcstring(cmd);
 
         let mut errors = ParseErrorList::new();
-        let ast = ast::parse(&cmd_wcs, ParseTreeFlags::empty(), Some(&mut errors));
+        let ast = ast::parse(&cmd_wcs, ParseTreeFlags::default(), Some(&mut errors));
         let errored = ast.errored() || {
-            parse_util_detect_errors_in_ast(&ast, &cmd_wcs, Some(&mut errors)).is_err()
+            detect_parse_errors_in_ast(&ast, &cmd_wcs, Some(&mut errors)).is_err()
         };
 
         if !errored {
@@ -215,7 +213,7 @@ fn run_command_list(parser: &Parser, cmds: &[OsString]) -> Result<(), libc::c_in
 }
 
 fn fish_parse_opt(args: &mut [WString], opts: &mut FishCmdOpts) -> ControlFlow<i32, usize> {
-    use fish::wgetopt::{ArgType::*, WGetopter, WOption, wopt};
+    use fish_wgetopt::{ArgType::*, WGetopter, WOption, wopt};
 
     const RUSAGE_ARG: char = 1 as char;
     const PRINT_DEBUG_CATEGORIES_ARG: char = 2 as char;
@@ -304,8 +302,8 @@ fn fish_parse_opt(args: &mut [WString], opts: &mut FishCmdOpts) -> ControlFlow<i
             'P' => opts.enable_private_mode = true,
             'v' => {
                 printf!(
-                    "%s",
-                    wgettext_fmt!("%s, version %s\n", PACKAGE_NAME, fish::BUILD_VERSION)
+                    "%s\n",
+                    wgettext_fmt!(VERSION_STRING_TEMPLATE, PACKAGE_NAME, fish::BUILD_VERSION)
                 );
                 return ControlFlow::Break(0);
             }
@@ -315,21 +313,21 @@ fn fish_parse_opt(args: &mut [WString], opts: &mut FishCmdOpts) -> ControlFlow<i
             }
             '?' => {
                 eprintf!(
-                    "%s\n",
+                    "%s\n\n",
                     wgettext_fmt!(BUILTIN_ERR_UNKNOWN, "fish", args[w.wopt_index - 1])
                 );
                 return ControlFlow::Break(1);
             }
             ':' => {
                 eprintf!(
-                    "%s\n",
+                    "%s\n\n",
                     wgettext_fmt!(BUILTIN_ERR_MISSING, "fish", args[w.wopt_index - 1])
                 );
                 return ControlFlow::Break(1);
             }
             ';' => {
                 eprintf!(
-                    "%s\n",
+                    "%s\n\n",
                     wgettext_fmt!(BUILTIN_ERR_UNEXP_ARG, "fish", args[w.wopt_index - 1])
                 );
                 return ControlFlow::Break(1);
@@ -369,7 +367,7 @@ fn main() {
         // Create a new thread with a decent stack size to be our main thread
         std::thread::scope(|scope| {
             scope.spawn(|| panic_handler(throwing_main));
-        })
+        });
     }
 }
 
@@ -391,13 +389,11 @@ fn throwing_main() -> i32 {
     // Enable debug categories set in FISH_DEBUG.
     // This is in *addition* to the ones given via --debug.
     if let Some(debug_categories) = env::var_os("FISH_DEBUG") {
-        let s = bytes2wcstring(debug_categories.as_bytes());
+        let s = osstr2wcstring(debug_categories);
         activate_flog_categories_by_pattern(&s);
     }
 
-    let mut args: Vec<WString> = env::args_os()
-        .map(|osstr| bytes2wcstring(osstr.as_bytes()))
-        .collect();
+    let mut args: Vec<WString> = env::args_os().map(osstr2wcstring).collect();
     let mut opts = FishCmdOpts::default();
     let mut my_optind = match fish_parse_opt(&mut args, &mut opts) {
         ControlFlow::Continue(optind) => optind,
@@ -430,7 +426,7 @@ fn throwing_main() -> i32 {
                 eprintf!("%s\n", e);
                 return 1;
             }
-        };
+        }
     }
 
     // No-exec is prohibited when in interactive mode.
@@ -515,7 +511,7 @@ fn throwing_main() -> i32 {
     // Stomp the exit status of any initialization commands (issue #635).
     parser.set_last_statuses(Statuses::just(STATUS_CMD_OK));
 
-    // TODO: if-let-chains
+    // TODO(MSRV>=1.88): feature(let_chains)
     if let Some(path) = &opts.profile_startup_output {
         if opts.profile_startup_output != opts.profile_output {
             parser.emit_profiling(path);
@@ -594,10 +590,9 @@ fn throwing_main() -> i32 {
                     });
                 res = reader_read(parser, f.as_raw_fd(), &IoChain::new());
                 if res.is_err() {
-                    flogf!(
+                    flog!(
                         warning,
-                        wgettext!("Error while reading file %s\n"),
-                        path.to_string_lossy()
+                        wgettext_fmt!("Error while reading file %s", path.to_string_lossy())
                     );
                 }
             }
@@ -640,7 +635,7 @@ fn escape_single_quoted_hack_hack_hack_hack(s: &wstr) -> OsString {
         if matches!(c, '\\' | '\'') {
             result.push("\\");
         }
-        result.push(c.to_string())
+        result.push(c.to_string());
     }
     result.push("\'");
     result
