@@ -1,5 +1,5 @@
 use fish::builtins::fish_indent;
 
 fn main() {
-    fish_indent::main()
+    fish_indent::main();
 }
