@@ -9,8 +9,8 @@ use crate::input_common::{CharEvent, ReadlineCmd};
 use crate::operation_context::{OperationContext, no_cancel};
 use crate::parse_constants::{ParseTreeFlags, ParserTestErrorBits};
 use crate::parse_util::{
-    parse_util_detect_errors, parse_util_get_offset_from_line, parse_util_job_extent,
-    parse_util_lineno, parse_util_process_extent, parse_util_token_extent,
+    detect_parse_errors, get_job_extent, get_offset_from_line, get_process_extent,
+    get_token_extent, lineno,
 };
 use crate::prelude::*;
 use crate::proc::is_interactive_session;
@@ -18,9 +18,8 @@ use crate::reader::{
     commandline_get_state, commandline_set_buffer, commandline_set_search_field,
     reader_execute_readline_cmd, reader_showing_suggestion,
 };
-use crate::tokenizer::TOK_ACCEPT_UNFINISHED;
-use crate::tokenizer::{TokenType, Tokenizer};
-use crate::wcstringutil::join_strings;
+use crate::tokenizer::{TOK_ACCEPT_UNFINISHED, TokenType, Tokenizer};
+use fish_wcstringutil::join_strings;
 use std::ops::Range;
 
 /// Which part of the comandbuffer are we operating on.
@@ -103,11 +102,12 @@ fn strip_dollar_prefixes(insert_mode: AppendMode, prefix: &wstr, insert: &wstr)
     }
     insert.find(L!("$ "))?; // Early return.
     let source = prefix.to_owned() + insert;
-    let ast = ast::parse(
-        &source,
-        ParseTreeFlags::ACCEPT_INCOMPLETE_TOKENS | ParseTreeFlags::LEAVE_UNTERMINATED,
-        None,
-    );
+    let flags = ParseTreeFlags {
+        accept_incomplete_tokens: true,
+        leave_unterminated: true,
+        ..Default::default()
+    };
+    let ast = ast::parse(&source, flags, None);
     let mut stripped = WString::new();
     let mut have = prefix.len();
     for node in ast.walk() {
@@ -155,12 +155,12 @@ fn write_part(
     let pos = cursor_pos - range.start;
 
     let Some(token_mode) = token_mode else {
-        if cut_at_cursor {
-            streams.out.append(&buffer[range.start..range.start + pos]);
+        let range = if cut_at_cursor {
+            range.start..range.start + pos
         } else {
-            streams.out.append(&buffer[range]);
-        }
-        streams.out.append_char('\n');
+            range
+        };
+        streams.out.appendln(&buffer[range]);
         return;
     };
 
@@ -229,10 +229,10 @@ fn write_part(
             let token_text = tok.text_of(&token);
             add_token(token_text);
         }
-    };
+    }
 
     for arg in args {
-        streams.out.appendln(arg.completion);
+        streams.out.appendln(&arg.completion);
     }
 }
 
@@ -310,7 +310,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
             'f' => function_mode = true,
             'x' | '\x02' | 'o' => {
                 if token_mode.is_some() {
-                    streams.err.append(&wgettext_fmt!(
+                    streams.err.appendln(&wgettext_fmt!(
                         BUILTIN_ERR_COMBO2,
                         cmd,
                         wgettext!("--tokens options are mutually exclusive")
@@ -323,7 +323,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
                     '\x02' => TokenOutputMode::Raw,
                     'o' => TokenOutputMode::Unescaped,
                     _ => unreachable!(),
-                })
+                });
             }
             'I' => {
                 // A historical, undocumented feature. TODO: consider removing this.
@@ -377,7 +377,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
             || selection_start_mode
             || selection_end_mode
         {
-            streams.err.append(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
+            streams.err.appendln(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -422,7 +422,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
     if (selection_start_mode || selection_end_mode) && positional_args != 0 {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
@@ -432,7 +432,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
     {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
@@ -442,13 +442,13 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
         // Special case - we allow to get/set cursor position relative to the process/job/token.
         && ((buffer_part.is_none() && !search_field_mode) || !cursor_mode)
     {
-        streams.err.append(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
+        streams.err.appendln(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
 
     if (token_mode.is_some() || cut_at_cursor) && positional_args != 0 {
-        streams.err.append(&wgettext_fmt!(
+        streams.err.appendln(&wgettext_fmt!(
             BUILTIN_ERR_COMBO2,
             cmd,
             "--cut-at-cursor and token options can not be used when setting the commandline"
@@ -458,7 +458,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
     }
 
     if search_field_mode && (buffer_part.is_some() || token_mode.is_some()) {
-        streams.err.append(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
+        streams.err.appendln(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
@@ -475,7 +475,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
 
     if append_mode == AppendMode::InsertSmart {
         if search_field_mode {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2_EXCLUSIVE,
                 cmd,
                 "--insert-smart",
@@ -488,7 +488,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
             TextScope::String | TextScope::Job | TextScope::Process => (),
             TextScope::Token => {
                 // To-do: we can support it in command position.
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_COMBO2_EXCLUSIVE,
                     cmd,
                     "--insert-smart",
@@ -507,7 +507,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
                 Err(_) => {
                     streams
                         .err
-                        .append(&wgettext_fmt!(BUILTIN_ERR_NOT_NUMBER, cmd, arg));
+                        .appendln(&wgettext_fmt!(BUILTIN_ERR_NOT_NUMBER, cmd, arg));
                     builtin_print_error_trailer(parser, streams.err, cmd);
                     0
                 }
@@ -522,28 +522,26 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
             };
 
             let new_pos = if line_mode {
-                let Some(offset) = parse_util_get_offset_from_line(
-                    &rstate.text,
-                    i32::try_from(new_coord).unwrap(),
-                ) else {
+                let Some(offset) =
+                    get_offset_from_line(&rstate.text, i32::try_from(new_coord).unwrap())
+                else {
                     streams
                         .err
-                        .append(&wgettext_fmt!("%s: there is no line %s\n", cmd, arg));
+                        .appendln(&wgettext_fmt!("%s: there is no line %s", cmd, arg));
                     builtin_print_error_trailer(parser, streams.err, cmd);
                     return Err(STATUS_INVALID_ARGS);
                 };
                 offset
             } else {
                 let line_index =
-                    i32::try_from(parse_util_lineno(&rstate.text, rstate.cursor_pos)).unwrap() - 1;
+                    i32::try_from(lineno(&rstate.text, rstate.cursor_pos)).unwrap() - 1;
                 let line_offset =
-                    parse_util_get_offset_from_line(&rstate.text, line_index).unwrap_or_default();
+                    get_offset_from_line(&rstate.text, line_index).unwrap_or_default();
                 let next_line_offset =
-                    parse_util_get_offset_from_line(&rstate.text, line_index + 1)
-                        .unwrap_or(rstate.text.len());
+                    get_offset_from_line(&rstate.text, line_index + 1).unwrap_or(rstate.text.len());
                 if line_offset + new_coord > next_line_offset {
-                    streams.err.append(&wgettext_fmt!(
-                        "%s: column %s exceeds line length\n",
+                    streams.err.appendln(&wgettext_fmt!(
+                        "%s: column %s exceeds line length",
                         cmd,
                         arg
                     ));
@@ -557,13 +555,12 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
             streams.out.append(&sprintf!(
                 "%d\n",
                 if line_mode {
-                    parse_util_lineno(&rstate.text, rstate.cursor_pos)
+                    lineno(&rstate.text, rstate.cursor_pos)
                 } else {
                     rstate.cursor_pos + 1
-                        - parse_util_get_offset_from_line(
+                        - get_offset_from_line(
                             &rstate.text,
-                            i32::try_from(parse_util_lineno(&rstate.text, rstate.cursor_pos) - 1)
-                                .unwrap(),
+                            i32::try_from(lineno(&rstate.text, rstate.cursor_pos) - 1).unwrap(),
                         )
                         .unwrap_or_default()
                 }
@@ -658,7 +655,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
         if current_buffer.is_empty() {
             return Err(STATUS_CMD_ERROR);
         }
-        let res = parse_util_detect_errors(current_buffer, None, /*accept_incomplete=*/ true);
+        let res = detect_parse_errors(current_buffer, None, /*accept_incomplete=*/ true);
         return match res {
             Ok(()) => Ok(SUCCESS),
             Err(err) => {
@@ -687,13 +684,13 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
                 range = 0..current_buffer.len();
             }
             TextScope::Job => {
-                range = parse_util_job_extent(current_buffer, current_cursor_pos, None);
+                range = get_job_extent(current_buffer, current_cursor_pos, None);
             }
             TextScope::Process => {
-                range = parse_util_process_extent(current_buffer, current_cursor_pos, None);
+                range = get_process_extent(current_buffer, current_cursor_pos, None);
             }
             TextScope::Token => {
-                (range, _) = parse_util_token_extent(current_buffer, current_cursor_pos);
+                (range, _) = get_token_extent(current_buffer, current_cursor_pos);
             }
         }
     }
@@ -705,7 +702,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
                 Err(_) => {
                     streams
                         .err
-                        .append(&wgettext_fmt!(BUILTIN_ERR_NOT_NUMBER, cmd, arg));
+                        .appendln(&wgettext_fmt!(BUILTIN_ERR_NOT_NUMBER, cmd, arg));
                     builtin_print_error_trailer(parser, streams.err, cmd);
                     0
                 }
