@@ -4,11 +4,10 @@ use crate::complete::{CompletionRequestOptions, complete_add_wrapper, complete_r
 use crate::highlight::highlight_and_colorize;
 use crate::operation_context::OperationContext;
 use crate::parse_constants::ParseErrorList;
-use crate::parse_util::parse_util_detect_errors_in_argument_list;
-use crate::parse_util::{parse_util_detect_errors, parse_util_token_extent};
+use crate::parse_util::detect_errors_in_argument_list;
+use crate::parse_util::{detect_parse_errors, get_token_extent};
 use crate::proc::is_interactive_session;
 use crate::reader::{commandline_get_state, completion_apply_to_command_line};
-use crate::wcstringutil::string_suffixes_string;
 use crate::{
     common::bytes2wcstring,
     complete::{
@@ -16,6 +15,7 @@ use crate::{
         complete_remove, complete_remove_all,
     },
 };
+use fish_wcstringutil::string_suffixes_string;
 
 // builtin_complete_* are a set of rather silly looping functions that make sure that all the proper
 // combinations of complete_add or complete_remove get called. This is needed since complete allows
@@ -243,6 +243,11 @@ const OPT_ESCAPE: char = '\x01';
 /// The complete builtin. Used for specifying programmable tab-completions. Calls the functions in
 /// complete.rs for any heavy lifting.
 pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> BuiltinResult {
+    localizable_consts! {
+        OPTION_REQUIRES_NON_EMPTY_STRING
+        "%s: %s requires a non-empty string"
+    }
+
     let cmd = argv[0];
     let argc = argv.len();
     let mut result_mode = CompletionMode::default();
@@ -321,8 +326,8 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                         cmd_to_complete.push(tmp);
                     }
                 } else {
-                    streams.err.append(&wgettext_fmt!(
-                        "%s: Invalid token '%s'\n",
+                    streams.err.appendln(&wgettext_fmt!(
+                        "%s: Invalid token '%s'",
                         cmd,
                         w.woptarg.unwrap()
                     ));
@@ -342,9 +347,11 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 let arg = w.woptarg.unwrap();
                 short_opt.extend(arg.chars());
                 if arg.is_empty() {
-                    streams
-                        .err
-                        .append(&wgettext_fmt!("%s: -s requires a non-empty string\n", cmd,));
+                    streams.err.appendln(&wgettext_fmt!(
+                        OPTION_REQUIRES_NON_EMPTY_STRING,
+                        cmd,
+                        "-s",
+                    ));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -352,9 +359,11 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 let arg = w.woptarg.unwrap();
                 gnu_opt.push(arg);
                 if arg.is_empty() {
-                    streams
-                        .err
-                        .append(&wgettext_fmt!("%s: -l requires a non-empty string\n", cmd,));
+                    streams.err.appendln(&wgettext_fmt!(
+                        OPTION_REQUIRES_NON_EMPTY_STRING,
+                        cmd,
+                        "-l",
+                    ));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -362,9 +371,11 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 let arg = w.woptarg.unwrap();
                 old_opt.push(arg);
                 if arg.is_empty() {
-                    streams
-                        .err
-                        .append(&wgettext_fmt!("%s: -o requires a non-empty string\n", cmd,));
+                    streams.err.appendln(&wgettext_fmt!(
+                        OPTION_REQUIRES_NON_EMPTY_STRING,
+                        cmd,
+                        "-o",
+                    ));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -413,7 +424,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
 
     if result_mode.no_files && result_mode.force_files {
         if !have_x {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 "complete",
                 "'--no-files' and '--force-files'"
@@ -421,7 +432,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         } else {
             // The reason for us not wanting files is `-x`,
             // which is short for `-rf`.
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 "complete",
                 "'--exclusive' and '--force-files'"
@@ -441,7 +452,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         } else {
             streams
                 .err
-                .append(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
+                .appendln(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -449,16 +460,15 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
 
     for condition_string in &condition {
         let mut errors = ParseErrorList::new();
-        if parse_util_detect_errors(condition_string, Some(&mut errors), false).is_err() {
+        if detect_parse_errors(condition_string, Some(&mut errors), false).is_err() {
             for error in errors {
                 let prefix = cmd.to_owned() + L!(": -n '") + &condition_string[..] + L!("': ");
-                streams.err.append(&error.describe_with_prefix(
+                streams.err.appendln(&error.describe_with_prefix(
                     condition_string,
                     &prefix,
                     parser.is_interactive(),
                     false,
                 ));
-                streams.err.append_char('\n');
             }
             return Err(STATUS_CMD_ERROR);
         }
@@ -469,14 +479,11 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         prefix.push_utfstr(cmd);
         prefix.push_str(": ");
 
-        if let Err(err_text) = parse_util_detect_errors_in_argument_list(&comp, &prefix) {
-            streams.err.append(&wgettext_fmt!(
-                "%s: %s: contains a syntax error\n",
-                cmd,
-                comp
-            ));
-            streams.err.append(&err_text);
-            streams.err.append_char('\n');
+        if let Err(err_text) = detect_errors_in_argument_list(&comp, &prefix) {
+            streams
+                .err
+                .appendln(&wgettext_fmt!("%s: %s: contains a syntax error", cmd, comp));
+            streams.err.appendln(&err_text);
             return Err(STATUS_CMD_ERROR);
         }
     }
@@ -499,7 +506,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
             Some(param) => param,
         };
 
-        let (token, _) = parse_util_token_extent(&do_complete_param, do_complete_param.len());
+        let (token, _) = get_token_extent(&do_complete_param, do_complete_param.len());
 
         // Create a scoped transient command line, so that builtin_commandline will see our
         // argument, not the reader buffer.
