@@ -12,7 +12,7 @@ use super::prelude::*;
 use crate::ast::{self, AsNode, Ast, Kind, Leaf, Node, NodeVisitor, SourceRangeList, Traversal};
 use crate::common::{
     PROGRAM_NAME, ReadExt, UnescapeFlags, UnescapeStringStyle, bytes2wcstring, get_program_name,
-    unescape_string, wcs2bytes,
+    osstr2wcstring, unescape_string, wcs2bytes,
 };
 use crate::env::EnvStack;
 use crate::env::env_init;
@@ -23,15 +23,17 @@ use crate::global_safety::RelaxedAtomicBool;
 use crate::highlight::{HighlightRole, HighlightSpec, colorize, highlight_shell};
 use crate::operation_context::OperationContext;
 use crate::parse_constants::{ParseTokenType, ParseTreeFlags, SourceRange};
-use crate::parse_util::{SPACES_PER_INDENT, apply_indents, parse_util_compute_indents};
+use crate::parse_util::{SPACES_PER_INDENT, apply_indents, compute_indents};
 use crate::prelude::*;
 use crate::print_help::print_help;
 use crate::threads;
 use crate::tokenizer::{TOK_SHOW_BLANK_LINES, TOK_SHOW_COMMENTS, TokenType, Tokenizer};
 use crate::topic_monitor::topic_monitor_init;
-use crate::wcstringutil::count_preceding_backslashes;
-use crate::wgetopt::{ArgType, WGetopter, WOption, wopt};
 use crate::wutil::fish_iswalnum;
+use assert_matches::assert_matches;
+use fish_wcstringutil::count_preceding_backslashes;
+use fish_wgetopt::{ArgType, WGetopter, WOption, wopt};
+use std::fmt::Write as _;
 
 /// Note: this got somewhat more complicated after introducing the new AST, because that AST no
 /// longer encodes detailed lexical information (e.g. every newline). This feels more complex
@@ -161,7 +163,7 @@ impl<'source, 'ast> PrettyPrinter<'source, 'ast> {
                 indents: if do_indent
                 /* Whether to indent, or just insert spaces. */
                 {
-                    parse_util_compute_indents(source)
+                    compute_indents(source)
                 } else {
                     vec![0; source.len()]
                 },
@@ -377,7 +379,7 @@ impl<'source, 'ast> PrettyPrinterState<'source, 'ast> {
         match node.kind() {
             // Allow escaped newlines before leaf nodes that can be part of a long command.
             Kind::Argument(_) | Kind::Redirection(_) | Kind::VariableAssignment(_) => {
-                result.allow_escaped_newlines = true
+                result.allow_escaped_newlines = true;
             }
             Kind::Token(token) => {
                 // Allow escaped newlines before && and ||, and also pipes.
@@ -393,7 +395,7 @@ impl<'source, 'ast> PrettyPrinterState<'source, 'ast> {
                             return result;
                         }
                         let p = self.traversal.parent(p);
-                        assert!(matches!(p.kind(), Kind::Statement(_)));
+                        assert_matches!(p.kind(), Kind::Statement(_));
                         let p = self.traversal.parent(p);
                         if let Kind::JobPipeline(job) = p.kind() {
                             if !job.variables.is_empty() {
@@ -895,10 +897,13 @@ impl<'source, 'ast> PrettyPrinterState<'source, 'ast> {
 
 // The flags we use to parse.
 fn parse_flags() -> ParseTreeFlags {
-    ParseTreeFlags::CONTINUE_AFTER_ERROR
-        | ParseTreeFlags::INCLUDE_COMMENTS
-        | ParseTreeFlags::LEAVE_UNTERMINATED
-        | ParseTreeFlags::SHOW_BLANK_LINES
+    ParseTreeFlags {
+        continue_after_error: true,
+        include_comments: true,
+        leave_unterminated: true,
+        show_blank_lines: true,
+        ..Default::default()
+    }
 }
 
 /// Return whether a character at a given index is escaped.
@@ -941,9 +946,7 @@ fn throwing_main() -> i32 {
         }
     }
 
-    let args: Vec<WString> = std::env::args_os()
-        .map(|osstr| bytes2wcstring(osstr.as_bytes()))
-        .collect();
+    let args: Vec<WString> = std::env::args_os().map(osstr2wcstring).collect();
     do_indent(None, &mut streams, args).builtin_status_code()
 }
 
@@ -1004,8 +1007,8 @@ fn do_indent(
                 return Ok(SUCCESS);
             }
             'v' => {
-                streams.out.appendln(wgettext_fmt!(
-                    "%s, version %s",
+                streams.out.appendln(&wgettext_fmt!(
+                    VERSION_STRING_TEMPLATE,
                     get_program_name(),
                     crate::BUILD_VERSION
                 ));
@@ -1020,7 +1023,7 @@ fn do_indent(
             '\x03' => output_type = OutputType::PygmentsCsv,
             'c' => output_type = OutputType::Check,
             ';' => {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_UNEXP_ARG,
                     "fish_indent",
                     w.argv[w.wopt_index - 1]
@@ -1028,7 +1031,7 @@ fn do_indent(
                 return Err(STATUS_CMD_ERROR);
             }
             '?' => {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_UNKNOWN,
                     "fish_indent",
                     w.argv[w.wopt_index - 1]
@@ -1048,8 +1051,9 @@ fn do_indent(
     while i < args.len() || (args.is_empty() && i == 0) {
         if args.is_empty() && i == 0 {
             if output_type == OutputType::File {
-                streams.err.appendln(wgettext_fmt!(
-                    "Expected file path to read/write for -w:\n\n $ %s -w foo.fish",
+                streams.err.append(&sprintf!(
+                    "%s\n\n $ %s -w foo.fish\n",
+                    wgettext!("Expected file path to read/write for -w:"),
                     get_program_name()
                 ));
                 return Err(STATUS_CMD_ERROR);
@@ -1058,7 +1062,7 @@ fn do_indent(
                 let cmd = "fish_indent";
                 streams
                     .err
-                    .append(&wgettext_fmt!("%s: stdin is closed\n", cmd));
+                    .appendln(&wgettext_fmt!(BUILTIN_ERR_STDIN_CLOSED, cmd));
                 return Err(STATUS_CMD_ERROR);
             };
             let mut buf = vec![];
@@ -1084,7 +1088,7 @@ fn do_indent(
                     output_location = arg;
                 }
                 Err(err) => {
-                    streams.err.appendln(wgettext_fmt!(
+                    streams.err.appendln(&wgettext_fmt!(
                         "Opening \"%s\" failed: %s",
                         arg,
                         err.to_string()
@@ -1102,7 +1106,7 @@ fn do_indent(
         }
 
         let output_wtext = if only_indent || only_unindent {
-            let indents = parse_util_compute_indents(&src);
+            let indents = compute_indents(&src);
             if only_indent {
                 apply_indents(&src, &indents)
             } else {
@@ -1165,7 +1169,7 @@ fn do_indent(
                             let _ = file.write_all(&wcs2bytes(&output_wtext));
                         }
                         Err(err) => {
-                            streams.err.appendln(wgettext_fmt!(
+                            streams.err.appendln(&wgettext_fmt!(
                                 "Opening \"%s\" failed: %s",
                                 output_location,
                                 err.to_string()
@@ -1286,14 +1290,16 @@ fn make_pygments_csv(src: &wstr) -> Vec<u8> {
     }
 
     // Now render these to a string.
-    let mut result = String::new();
+    let mut result = String::with_capacity(token_ranges.len() * 32);
     for range in token_ranges {
-        result += &format!(
-            "{},{},{}\n",
+        writeln!(
+            result,
+            "{},{},{}",
             range.start,
             range.end,
             highlight_role_to_string(range.role)
-        );
+        )
+        .unwrap();
     }
     result.into_bytes()
 }
@@ -1301,20 +1307,20 @@ fn make_pygments_csv(src: &wstr) -> Vec<u8> {
 // Entry point for prettification.
 fn prettify(streams: &mut IoStreams, src: &wstr, do_indent: bool) -> WString {
     if DUMP_PARSE_TREE.load() {
-        let ast = ast::parse(
-            src,
-            ParseTreeFlags::LEAVE_UNTERMINATED
-                | ParseTreeFlags::INCLUDE_COMMENTS
-                | ParseTreeFlags::SHOW_EXTRA_SEMIS,
-            None,
-        );
+        let flags = ParseTreeFlags {
+            leave_unterminated: true,
+            include_comments: true,
+            show_extra_semis: true,
+            ..Default::default()
+        };
+        let ast = ast::parse(src, flags, None);
         let ast_dump = ast.dump(src);
-        streams.err.appendln(ast_dump);
+        streams.err.appendln(&ast_dump);
 
         // Output metrics too.
         let mut metrics = AstSizeMetrics::default();
         metrics.visit(ast.top());
-        streams.err.appendln(format!("{}", metrics));
+        streams.err.appendln(&format!("{}", metrics));
     }
     let ast = ast::parse(src, parse_flags(), None);
     let mut printer = PrettyPrinter::new(src, &ast, do_indent);
