@@ -7,13 +7,13 @@
 //!
 //! Type "exit" or "quit" to terminate the program.
 
-use std::{ops::ControlFlow, os::unix::prelude::OsStrExt};
+use std::ops::ControlFlow;
 
 use libc::{STDIN_FILENO, VEOF, VINTR};
 
 use crate::{
     builtins::shared::BUILTIN_ERR_UNKNOWN,
-    common::{PROGRAM_NAME, bytes2wcstring, get_program_name, shell_modes},
+    common::{PROGRAM_NAME, get_program_name, osstr2wcstring, shell_modes},
     env::{EnvStack, Environment, env_init},
     future_feature_flags,
     input_common::{
@@ -32,8 +32,8 @@ use crate::{
     threads,
     topic_monitor::topic_monitor_init,
     tty_handoff::TtyHandoff,
-    wgetopt::{ArgType, WGetopter, WOption, wopt},
 };
+use fish_wgetopt::{ArgType, WGetopter, WOption, wopt};
 
 use super::prelude::*;
 
@@ -47,7 +47,7 @@ fn should_exit(
 
     for evt in [VINTR, VEOF] {
         let modes = shell_modes();
-        let cc = Key::from_single_byte(modes.c_cc[evt]);
+        let cc = Key::from_single_byte(modes.control_chars[evt]);
 
         if match_key_event_to_key(&key_evt, &cc).is_some() {
             if recent_keys
@@ -59,9 +59,9 @@ fn should_exit(
             {
                 return true;
             }
-            streams.err.append(&wgettext_fmt!(
-                "Press ctrl-%c again to exit\n",
-                char::from(modes.c_cc[evt] + 0x60)
+            streams.err.appendln(&wgettext_fmt!(
+                "Press ctrl-%c again to exit",
+                char::from(modes.control_chars[evt] + 0x60)
             ));
             return false;
         }
@@ -160,10 +160,10 @@ fn setup_and_process_keys(
             .err
             .appendln("To terminate this program type \"exit\" or \"quit\" in this window,");
         let modes = shell_modes();
-        streams.err.appendln(wgettext_fmt!(
+        streams.err.appendln(&wgettext_fmt!(
             "or press ctrl-%c or ctrl-%c twice in a row.",
-            char::from(modes.c_cc[VINTR] + 0x60),
-            char::from(modes.c_cc[VEOF] + 0x60)
+            char::from(modes.control_chars[VINTR] + 0x60),
+            char::from(modes.control_chars[VEOF] + 0x60)
         ));
         streams.err.appendln(L!("\n"));
     }
@@ -201,8 +201,8 @@ fn parse_flags(
                 return ControlFlow::Break(Ok(SUCCESS));
             }
             'v' => {
-                streams.out.appendln(wgettext_fmt!(
-                    "%s, version %s",
+                streams.out.appendln(&wgettext_fmt!(
+                    VERSION_STRING_TEMPLATE,
                     get_program_name(),
                     crate::BUILD_VERSION
                 ));
@@ -212,7 +212,7 @@ fn parse_flags(
                 *verbose = true;
             }
             ';' => {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_UNEXP_ARG,
                     "fish_key_reader",
                     w.argv[w.wopt_index - 1]
@@ -235,7 +235,7 @@ fn parse_flags(
     if argc != 0 {
         streams
             .err
-            .appendln(wgettext_fmt!("Expected no arguments, got %d", argc));
+            .appendln(&wgettext_fmt!("Expected no arguments, got %d", argc));
         return ControlFlow::Break(Err(STATUS_CMD_ERROR));
     }
 
@@ -307,9 +307,7 @@ fn throwing_main() -> i32 {
     let mut continuous_mode = false;
     let mut verbose = false;
 
-    let args: Vec<WString> = std::env::args_os()
-        .map(|osstr| bytes2wcstring(osstr.as_bytes()))
-        .collect();
+    let args: Vec<WString> = std::env::args_os().map(osstr2wcstring).collect();
     if let ControlFlow::Break(s) =
         parse_flags(None, &mut streams, args, &mut continuous_mode, &mut verbose)
     {
