@@ -57,7 +57,7 @@ use crate::wutil::{
     wstr_offset_in,
 };
 use fish_printf::{ToArg, sprintf_locale};
-use fish_wchar::{decode_byte_from_char, encode_byte_to_char};
+use fish_widestring::{decode_byte_from_char, encode_byte_to_char};
 
 /// Return true if `c` is an octal digit.
 fn is_octal_digit(c: char) -> bool {
@@ -208,10 +208,10 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
         if errcode != None && errcode != Some(Error::InvalidChar) && errcode != Some(Error::Empty) {
             match errcode.unwrap() {
                 Error::Overflow => {
-                    self.fatal_error(sprintf!("%s: %s", s, wgettext!("Number out of range")));
+                    self.fatal_error(wgettext_fmt!("%s: Number out of range", s));
                 }
                 Error::Empty => {
-                    self.fatal_error(sprintf!("%s: %s", s, wgettext!("Number was empty")));
+                    self.fatal_error(wgettext_fmt!("%s: Number was empty", s));
                 }
                 Error::InvalidChar => {
                     panic!("Unreachable");
@@ -578,7 +578,7 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
 
         self.streams.err.append(errstr);
         if !errstr.ends_with('\n') {
-            self.streams.err.append_char('\n');
+            self.streams.err.append('\n');
         }
 
         // We set the exit code to error, because one occurred,
@@ -602,7 +602,7 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
 
         self.streams.err.append(errstr);
         if !errstr.ends_with('\n') {
-            self.streams.err.append_char('\n');
+            self.streams.err.append('\n');
         }
 
         self.exit_code = Err(STATUS_CMD_ERROR);
@@ -615,7 +615,7 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
     /// If octal_0 is nonzero, octal escapes are of the form \0ooo, where o
     /// is an octal digit; otherwise they are of the form \ooo.
     fn print_esc(&mut self, escstart: &wstr, octal_0: bool) -> usize {
-        assert!(escstart.char_at(0) == '\\');
+        assert_eq!(escstart.char_at(0), '\\');
         let mut p = &escstart[1..];
         let mut esc_value = 0; /* Value of \nnn escape. */
         let mut esc_length; /* Length of \nnn escape. */
