@@ -40,7 +40,7 @@ pub fn pwd(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Buil
     if w.wopt_index != argc {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_ARG_COUNT1, cmd, 0, argc - 1));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_ARG_COUNT1, cmd, 0, argc - 1));
         return Err(STATUS_INVALID_ARGS);
     }
 
@@ -52,9 +52,10 @@ pub fn pwd(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Buil
         if let Some(real_pwd) = wrealpath(&pwd) {
             pwd = real_pwd;
         } else {
-            streams.err.append(&wgettext_fmt!(
-                "%s: realpath failed: %s\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: %s failed: %s",
                 cmd,
+                "realpath",
                 errno().to_string()
             ));
             return Err(STATUS_CMD_ERROR);
@@ -63,6 +64,6 @@ pub fn pwd(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Buil
     if pwd.is_empty() {
         return Err(STATUS_CMD_ERROR);
     }
-    streams.out.appendln(pwd);
+    streams.out.appendln(&pwd);
     Ok(SUCCESS)
 }
