@@ -24,10 +24,9 @@ use crate::tokenizer::TOK_ACCEPT_UNFINISHED;
 use crate::tokenizer::TOK_ARGUMENT_LIST;
 use crate::tokenizer::Tok;
 use crate::tokenizer::Tokenizer;
-use crate::wcstringutil::split_about;
-use crate::wcstringutil::split_string_tok;
 use crate::wutil;
 use crate::wutil::perror;
+use fish_wcstringutil::{split_about, split_string_tok};
 use libc::SEEK_CUR;
 use std::num::NonZeroUsize;
 use std::os::fd::RawFd;
@@ -123,8 +122,8 @@ fn parse_cmd_opts(
                 opts.delimiter = Some(w.woptarg.unwrap().to_owned());
             }
             'i' => {
-                streams.err.append(&wgettext_fmt!(
-                    "%s: usage of -i for --silent is deprecated. Please use -s or --silent instead.\n",
+                streams.err.appendln(&wgettext_fmt!(
+                    "%s: usage of -i for --silent is deprecated. Please use -s or --silent instead.",
                     cmd
                 ));
                 return Err(STATUS_INVALID_ARGS);
@@ -148,8 +147,8 @@ fn parse_cmd_opts(
                 opts.nchars = match fish_wcstoi(w.woptarg.unwrap()) {
                     Ok(n) if n >= 0 => NonZeroUsize::new(n.try_into().unwrap()),
                     Err(wutil::Error::Overflow) => {
-                        streams.err.append(&wgettext_fmt!(
-                            "%s: Argument '%s' is out of range\n",
+                        streams.err.appendln(&wgettext_fmt!(
+                            "%s: Argument '%s' is out of range",
                             cmd,
                             w.woptarg.unwrap()
                         ));
@@ -157,7 +156,7 @@ fn parse_cmd_opts(
                         return Err(STATUS_INVALID_ARGS);
                     }
                     _ => {
-                        streams.err.append(&wgettext_fmt!(
+                        streams.err.appendln(&wgettext_fmt!(
                             BUILTIN_ERR_NOT_NUMBER,
                             cmd,
                             w.woptarg.unwrap()
@@ -190,7 +189,7 @@ fn parse_cmd_opts(
                 };
                 if let Some(old_mode) = opts.token_mode {
                     if old_mode != new_mode {
-                        streams.err.append(&wgettext_fmt!(
+                        streams.err.appendln(&wgettext_fmt!(
                             BUILTIN_ERR_COMBO2,
                             cmd,
                             wgettext_fmt!(
@@ -248,8 +247,9 @@ fn read_interactive(
     shell: bool,
     silent: bool,
     prompt: &wstr,
+    prompt_str_is_empty: bool,
     right_prompt: &wstr,
-    commandline: &Option<WString>,
+    commandline: Option<&WString>,
     inputfd: RawFd,
 ) -> BuiltinResult {
     let mut exit_res = Ok(SUCCESS);
@@ -265,6 +265,7 @@ fn read_interactive(
         expand_abbrev_ok: false,
 
         exit_on_interrupt: true,
+        read_prompt_str_is_empty: prompt_str_is_empty,
         in_silent_mode: silent,
 
         left_prompt_cmd: prompt.to_owned(),
@@ -448,20 +449,21 @@ fn validate_read_args(
     parser: &Parser,
     streams: &mut IoStreams,
 ) -> BuiltinResult {
+    localizable_consts! {
+        OPTIONS_CANNOT_BE_COMBINED
+        "%s: Options %s and %s cannot be used together"
+    }
     if opts.prompt.is_some() && opts.prompt_str.is_some() {
-        streams.err.append(&wgettext_fmt!(
-            "%s: Options %s and %s cannot be used together\n",
-            cmd,
-            "-p",
-            "-P",
-        ));
+        streams
+            .err
+            .appendln(&wgettext_fmt!(OPTIONS_CANNOT_BE_COMBINED, cmd, "-p", "-P",));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
 
     if opts.delimiter.is_some() && opts.one_line {
-        streams.err.append(&wgettext_fmt!(
-            "%s: Options %s and %s cannot be used together\n",
+        streams.err.appendln(&wgettext_fmt!(
+            OPTIONS_CANNOT_BE_COMBINED,
             cmd,
             "--delimiter",
             "--line"
@@ -469,8 +471,8 @@ fn validate_read_args(
         return Err(STATUS_INVALID_ARGS);
     }
     if opts.one_line && opts.split_null {
-        streams.err.append(&wgettext_fmt!(
-            "%s: Options %s and %s cannot be used together\n",
+        streams.err.appendln(&wgettext_fmt!(
+            OPTIONS_CANNOT_BE_COMBINED,
             cmd,
             "-z",
             "--line"
@@ -487,7 +489,7 @@ fn validate_read_args(
     if opts.place.mode.contains(EnvMode::UNEXPORT) && opts.place.mode.contains(EnvMode::EXPORT) {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_EXPUNEXP, cmd));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_EXPUNEXP, cmd));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
@@ -500,7 +502,9 @@ fn validate_read_args(
         .count()
         > 1
     {
-        streams.err.append(&wgettext_fmt!(BUILTIN_ERR_GLOCAL, cmd));
+        streams
+            .err
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_GLOCAL, cmd));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
@@ -509,21 +513,21 @@ fn validate_read_args(
     if !opts.array && argc < 1 && !opts.to_stdout {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_MIN_ARG_COUNT1, cmd, 1, argc));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_MIN_ARG_COUNT1, cmd, 1, argc));
         return Err(STATUS_INVALID_ARGS);
     }
 
     if opts.array && argc != 1 {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_ARG_COUNT1, cmd, 1, argc));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_ARG_COUNT1, cmd, 1, argc));
         return Err(STATUS_INVALID_ARGS);
     }
 
     if opts.to_stdout && argc > 0 {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_MAX_ARG_COUNT1, cmd, 0, argc));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_MAX_ARG_COUNT1, cmd, 0, argc));
         return Err(STATUS_INVALID_ARGS);
     }
 
@@ -537,7 +541,7 @@ fn validate_read_args(
 
     if let Some(token_mode) = opts.token_mode {
         if opts.delimiter.is_some() {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2_EXCLUSIVE,
                 cmd,
                 "--delimiter",
@@ -547,7 +551,7 @@ fn validate_read_args(
         }
 
         if opts.one_line {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2_EXCLUSIVE,
                 cmd,
                 "--line",
@@ -604,10 +608,10 @@ pub fn read(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
     validate_read_args(cmd, &mut opts, argv, parser, streams)?;
 
     // stdin may have been explicitly closed
-    if streams.stdin_fd < 0 {
+    if streams.is_stdin_closed() {
         streams
             .err
-            .append(&wgettext_fmt!("%s: stdin is closed\n", cmd));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_STDIN_CLOSED, cmd));
         return Err(STATUS_CMD_ERROR);
     }
 
@@ -627,7 +631,7 @@ pub fn read(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
         }
     };
 
-    let stream_stdin_is_a_tty = isatty(streams.stdin_fd);
+    let stream_stdin_is_a_tty = streams.stdin_fd() >= 0 && isatty(streams.stdin_fd());
 
     // Normally, we either consume a line of input or all available input. But if we are reading a
     // line at a time, we need a middle ground where we only consume as many lines as we need to
@@ -644,9 +648,10 @@ pub fn read(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                 opts.shell,
                 opts.silent,
                 opts.prompt.as_ref().unwrap(),
+                opts.prompt_str.as_ref().is_some_and(|ps| ps.is_empty()),
                 &opts.right_prompt,
-                &opts.commandline,
-                streams.stdin_fd,
+                opts.commandline.as_ref(),
+                streams.stdin_fd(),
             );
         } else if opts.nchars.is_none() && !stream_stdin_is_a_tty &&
                    // "one_line" is implemented as reading n-times to a new line,
@@ -655,7 +660,7 @@ pub fn read(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                    !opts.one_line &&
                        (
                            streams.stdin_is_directly_redirected ||
-                               unsafe {libc::lseek(streams.stdin_fd, 0, SEEK_CUR)} != -1)
+                               unsafe {libc::lseek(streams.stdin_fd(), 0, SEEK_CUR)} != -1)
         {
             // We read in chunks when we either can seek (so we put the bytes back),
             // or we have the bytes to ourselves (because it's directly redirected).
@@ -665,14 +670,18 @@ pub fn read(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
             // You don't rewind VHS tapes before throwing them in the trash.
             // TODO: Do this when nchars is set by seeking back.
             exit_res = read_in_chunks(
-                streams.stdin_fd,
+                streams.stdin_fd(),
                 &mut buff,
                 opts.split_null,
                 !streams.stdin_is_directly_redirected,
             );
         } else {
-            exit_res =
-                read_one_char_at_a_time(streams.stdin_fd, &mut buff, opts.nchars, opts.split_null);
+            exit_res = read_one_char_at_a_time(
+                streams.stdin_fd(),
+                &mut buff,
+                opts.nchars,
+                opts.split_null,
+            );
         }
 
         if exit_res.is_err() {
@@ -699,7 +708,7 @@ pub fn read(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                             text = Cow::Owned(unescaped);
                         }
                     }
-                };
+                }
                 text.into_owned()
             };
             if opts.array {
