@@ -1,13 +1,13 @@
 // Implementation of the set_color builtin.
 
 use super::prelude::*;
-use crate::color::Color;
 use crate::common::bytes2wcstring;
 use crate::screen::{is_dumb, only_grayscale};
 use crate::terminal::{Outputter, use_terminfo};
 use crate::text_face::{
     self, PrintColorsArgs, SpecifiedTextFace, TextFace, TextStyling, parse_text_face_and_options,
 };
+use fish_color::Color;
 
 fn print_colors(
     streams: &mut IoStreams,
@@ -99,12 +99,12 @@ pub fn set_color(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -
             Err(UnknownColor(arg)) => {
                 streams
                     .err
-                    .append(&wgettext_fmt!("%s: Unknown color '%s'\n", argv[0], arg));
+                    .appendln(&wgettext_fmt!("%s: Unknown color '%s'", argv[0], arg));
                 return Err(STATUS_INVALID_ARGS);
             }
             Err(UnknownUnderlineStyle(arg)) => {
-                streams.err.append(&wgettext_fmt!(
-                    "%s: invalid underline style: %s\n",
+                streams.err.appendln(&wgettext_fmt!(
+                    "%s: invalid underline style: %s",
                     argv[0],
                     arg
                 ));
