@@ -25,22 +25,22 @@ impl StringSubCommand<'_> for Sub {
                 self.length = Some(
                     fish_wcstol(arg.unwrap())?
                         .try_into()
-                        .map_err(|_| invalid_args!("%s: Invalid length value '%s'\n", name, arg))?,
-                )
+                        .map_err(|_| invalid_args!("%s: Invalid length value '%s'", name, arg))?,
+                );
             }
             's' => {
                 self.start = Some(
                     fish_wcstol(arg.unwrap())?
                         .try_into()
-                        .map_err(|_| invalid_args!("%s: Invalid start value '%s'\n", name, arg))?,
-                )
+                        .map_err(|_| invalid_args!("%s: Invalid start value '%s'", name, arg))?,
+                );
             }
             'e' => {
                 self.end = Some(
                     fish_wcstol(arg.unwrap())?
                         .try_into()
-                        .map_err(|_| invalid_args!("%s: Invalid end value '%s'\n", name, arg))?,
-                )
+                        .map_err(|_| invalid_args!("%s: Invalid end value '%s'", name, arg))?,
+                );
             }
             'q' => self.quiet = true,
             _ => return Err(StringError::UnknownOption),
@@ -57,7 +57,7 @@ impl StringSubCommand<'_> for Sub {
     ) -> Result<(), ErrorCode> {
         let cmd = args[0];
         if self.length.is_some() && self.end.is_some() {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 cmd,
                 wgettext!("--end and --length are mutually exclusive")
@@ -98,7 +98,7 @@ impl StringSubCommand<'_> for Sub {
                     .out
                     .append(&arg[start..usize::min(start + count, arg.len())]);
                 if want_newline {
-                    streams.out.append_char('\n');
+                    streams.out.append('\n');
                 }
             }
             nsub += 1;
