@@ -5,7 +5,9 @@ use super::environment_impl::{
 };
 use crate::abbrs::{Abbreviation, Position, abbrs_get_set};
 use crate::builtins::shared::{BuiltinResult, SUCCESS};
-use crate::common::{UnescapeStringStyle, bytes2wcstring, unescape_string, wcs2zstring};
+use crate::common::{
+    UnescapeStringStyle, cstr2wcstring, osstr2wcstring, str2wcstring, unescape_string,
+};
 use crate::env::config_paths::ConfigPaths;
 use crate::env::{EnvMode, EnvSetMode, EnvVar, Statuses};
 use crate::env_dispatch::{VarChangeMilieu, env_dispatch_init, env_dispatch_var_change};
@@ -14,7 +16,7 @@ use crate::flog::flog;
 use crate::global_safety::RelaxedAtomicBool;
 use crate::input::{FISH_BIND_MODE_VAR, init_input};
 use crate::localization::wgettext;
-use crate::nix::{geteuid, getpid};
+use crate::nix::getpid;
 use crate::null_terminated_array::OwningNullTerminatedArray;
 use crate::path::{
     path_emit_config_directory_messages, path_get_cache, path_get_config, path_get_data,
@@ -24,14 +26,15 @@ use crate::prelude::*;
 use crate::proc::is_interactive_session;
 use crate::termsize;
 use crate::universal_notifier::default_notifier;
-use crate::wcstringutil::join_strings;
 use crate::wutil::{fish_wcstol, wgetcwd};
-
-use libc::{c_int, uid_t};
+use fish_wcstringutil::join_strings;
+use libc::c_int;
+use nix::{
+    NixPath,
+    unistd::{Uid, User, gethostname},
+};
 use std::collections::HashMap;
 use std::ffi::CStr;
-use std::mem::MaybeUninit;
-use std::os::unix::prelude::*;
 use std::path::PathBuf;
 use std::sync::{Arc, LazyLock, OnceLock};
 
@@ -457,91 +460,27 @@ const FISH_CONFIG_DIR: &wstr = L!("__fish_config_dir");
 const FISH_USER_DATA_DIR: &wstr = L!("__fish_user_data_dir");
 const FISH_CACHE_DIR: &wstr = L!("__fish_cache_dir");
 
-/// Maximum length of hostname. Longer hostnames are truncated.
-const HOSTNAME_LEN: usize = 255;
-
-/// Function to get an identifier based on the hostname.
-fn get_hostname_identifier() -> Option<WString> {
-    // The behavior of gethostname if the buffer size is insufficient differs by implementation and
-    // libc version Work around this by using a "guaranteed" sufficient buffer size then truncating
-    // the result.
-    let mut b = [0 as libc::c_char; HOSTNAME_LEN + 1];
-    if unsafe { libc::gethostname(b.as_mut_ptr(), b.len()) } == 0 {
-        let cstr = unsafe { CStr::from_ptr(b.as_ptr()) };
-        let res = bytes2wcstring(cstr.to_bytes());
-
-        if res.is_empty() { None } else { Some(res) }
-    } else {
-        None
-    }
-}
-
-/// Get values for $HOME via getpwuid,
-/// without trusting $USER or $HOME.
-pub fn get_home() -> Option<String> {
-    let uid: uid_t = geteuid();
-
-    let mut userinfo: MaybeUninit<libc::passwd> = MaybeUninit::uninit();
-    let mut result: *mut libc::passwd = std::ptr::null_mut();
-    let mut buf = [0 as libc::c_char; 8192];
-
-    // We need to get the data via the uid and don't trust $USER.
-    let retval = unsafe {
-        libc::getpwuid_r(
-            uid,
-            userinfo.as_mut_ptr(),
-            buf.as_mut_ptr(),
-            buf.len(),
-            &mut result,
-        )
-    };
-    if retval != 0 || result.is_null() {
-        return None;
-    }
-
-    let userinfo = unsafe { userinfo.assume_init() };
-    if !userinfo.pw_dir.is_null() {
-        let home = unsafe { CStr::from_ptr(userinfo.pw_dir) };
-        let home = home.to_str().ok().map(|x| x.to_owned());
-        home
-    } else {
-        None
-    }
-}
-
 /// Set up the USER and HOME variable.
 fn setup_user(global_exported_mode: EnvSetMode, vars: &EnvStack) {
-    let uid: uid_t = geteuid();
+    let uid = Uid::effective();
     let user_var = vars.get_unless_empty(L!("USER"));
 
-    let mut userinfo: MaybeUninit<libc::passwd> = MaybeUninit::uninit();
-    let mut result: *mut libc::passwd = std::ptr::null_mut();
-    let mut buf = [0 as libc::c_char; 8192];
-
     // If we have a $USER, we try to get the passwd entry for the name.
     // If that has the same UID that we use, we assume the data is correct.
     if let Some(user_var) = user_var {
-        let unam_narrow = wcs2zstring(&user_var.as_string());
-        let retval = unsafe {
-            libc::getpwnam_r(
-                unam_narrow.as_ptr(),
-                userinfo.as_mut_ptr(),
-                buf.as_mut_ptr(),
-                buf.len(),
-                &mut result,
-            )
-        };
-        if retval == 0 && !result.is_null() {
-            let userinfo = unsafe { userinfo.assume_init() };
-            if unsafe { *result }.pw_uid == uid {
+        // POSIX.1-2017 requires that user names are a subset of ASCII, so converting to String here
+        // is ok.
+        // https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap03.html#tag_03_437
+        // https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap03.html#tag_03_282
+        if let Ok(Some(userinfo)) = User::from_name(&user_var.as_string().to_string()) {
+            if userinfo.uid == uid {
                 // The uid matches but we still might need to set $HOME.
                 if vars.get_unless_empty(L!("HOME")).is_none() {
-                    if !userinfo.pw_dir.is_null() {
-                        let s = unsafe { CStr::from_ptr(userinfo.pw_dir) };
+                    if !userinfo.dir.is_empty() {
                         vars.set_one(
                             L!("HOME"),
                             global_exported_mode,
-                            bytes2wcstring(s.to_bytes()),
+                            osstr2wcstring(userinfo.dir),
                         );
                     } else {
                         vars.set_empty(L!("HOME"), global_exported_mode);
@@ -552,31 +491,17 @@ fn setup_user(global_exported_mode: EnvSetMode, vars: &EnvStack) {
         }
     }
 
-    // Either we didn't have a $USER or it had a different uid.
-    // We need to get the data *again* via the uid.
-    let retval = unsafe {
-        libc::getpwuid_r(
-            uid,
-            userinfo.as_mut_ptr(),
-            buf.as_mut_ptr(),
-            buf.len(),
-            &mut result,
-        )
-    };
-    if retval == 0 && !result.is_null() {
-        let userinfo = unsafe { userinfo.assume_init() };
-        let s = unsafe { CStr::from_ptr(userinfo.pw_name) };
-        let uname = bytes2wcstring(s.to_bytes());
+    if let Ok(Some(userinfo)) = User::from_uid(uid) {
+        let uname = str2wcstring(userinfo.name);
         vars.set_one(L!("USER"), global_exported_mode, uname);
         // Only change $HOME if it's empty, so we allow e.g. `HOME=(mktemp -d)`.
         // This is okay with common `su` and `sudo` because they set $HOME.
         if vars.get_unless_empty(L!("HOME")).is_none() {
-            if !userinfo.pw_dir.is_null() {
-                let s = unsafe { CStr::from_ptr(userinfo.pw_dir) };
+            if !userinfo.dir.is_empty() {
                 vars.set_one(
                     L!("HOME"),
                     global_exported_mode,
-                    bytes2wcstring(s.to_bytes()),
+                    osstr2wcstring(userinfo.dir),
                 );
             } else {
                 // We cannot get $HOME. This triggers warnings for history and config.fish already,
@@ -601,12 +526,12 @@ pub(crate) static FALLBACK_PATH: LazyLock<&[WString]> = LazyLock::new(|| {
         let buf = buf;
         // safety: buf should contain a null-byte, and is not mutable unless we move ownership
         let cstr = unsafe { CStr::from_ptr(buf.as_ptr()) };
-        colon_split(&[bytes2wcstring(cstr.to_bytes())])
+        colon_split(&[cstr2wcstring(cstr)])
     } else {
         vec![
-            WString::from_str(env!("PREFIX")) + L!("/bin"),
-            WString::from_str("/usr/bin"),
-            WString::from_str("/bin"),
+            str2wcstring(env!("PREFIX")) + L!("/bin"),
+            L!("/usr/bin").to_owned(),
+            L!("/bin").to_owned(),
         ]
     };
     Box::leak(paths.into_boxed_slice())
@@ -632,7 +557,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     let global_exported_mode = EnvSetMode::new_at_early_startup(EnvMode::GLOBAL | EnvMode::EXPORT);
 
     let env_iter: Vec<_> = std::env::vars_os()
-        .map(|(k, v)| (bytes2wcstring(k.as_bytes()), bytes2wcstring(v.as_bytes())))
+        .map(|(k, v)| (osstr2wcstring(k), osstr2wcstring(v)))
         .collect();
 
     let mut inherited_vars = HashMap::new();
@@ -662,7 +587,11 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
 
     // Set $USER, $HOME and $EUID
     // This involves going to passwd and stuff.
-    vars.set_one(L!("EUID"), global_mode, geteuid().to_wstring());
+    vars.set_one(
+        L!("EUID"),
+        global_mode,
+        Uid::effective().as_raw().to_wstring(),
+    );
     setup_user(global_exported_mode, vars);
 
     if let Some(paths) = paths {
@@ -671,7 +600,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
                 key,
                 global_mode,
                 maybe_path
-                    .map(|path| vec![bytes2wcstring(path.as_os_str().as_bytes())])
+                    .map(|path| vec![osstr2wcstring(path)])
                     .unwrap_or_default(),
             );
         };
@@ -720,7 +649,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     );
 
     // Set up the version variable.
-    let version = bytes2wcstring(crate::BUILD_VERSION.as_bytes());
+    let version = str2wcstring(crate::BUILD_VERSION);
     vars.set_one(L!("version"), global_mode, version.clone());
     vars.set_one(L!("FISH_VERSION"), global_mode, version);
 
@@ -728,7 +657,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     vars.set_one(L!("fish_pid"), global_mode, getpid().to_wstring());
 
     // Set the $hostname variable
-    let hostname: WString = get_hostname_identifier().unwrap_or("fish".into());
+    let hostname: WString = gethostname().map_or("fish".into(), osstr2wcstring);
     vars.set_one(L!("hostname"), global_mode, hostname);
 
     // Set up SHLVL variable. Not we can't use vars.get() because SHLVL is read-only, and therefore
@@ -737,7 +666,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
         let nshlvl_str = if let Some(shlvl_var) = std::env::var_os("SHLVL") {
             // TODO: Figure out how to handle invalid numbers better. Shouldn't we issue a
             // diagnostic?
-            match fish_wcstol(&bytes2wcstring(shlvl_var.as_os_str().as_bytes())) {
+            match fish_wcstol(&osstr2wcstring(shlvl_var)) {
                 Ok(shlvl_i) if shlvl_i >= 0 => (shlvl_i + 1).to_wstring(),
                 _ => L!("1").to_owned(),
             }
@@ -748,11 +677,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     } else {
         // If we're not interactive, simply pass the value along.
         if let Some(shlvl_var) = std::env::var_os("SHLVL") {
-            vars.set_one(
-                L!("SHLVL"),
-                global_exported_mode,
-                bytes2wcstring(shlvl_var.as_os_str().as_bytes()),
-            );
+            vars.set_one(L!("SHLVL"), global_exported_mode, osstr2wcstring(shlvl_var));
         }
     }
 
@@ -764,9 +689,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     // Also reject all paths that don't start with "/", this includes windows paths like "F:\foo".
     // (see #7636)
     let incoming_pwd_cstr = std::env::var_os("PWD");
-    let incoming_pwd = incoming_pwd_cstr
-        .map(|s| bytes2wcstring(s.as_os_str().as_bytes()))
-        .unwrap_or_default();
+    let incoming_pwd = incoming_pwd_cstr.map(osstr2wcstring).unwrap_or_default();
     if !incoming_pwd.is_empty()
         && incoming_pwd.char_at(0) == '/'
         && paths_are_same_file(&incoming_pwd, L!("."))
