@@ -1,7 +1,8 @@
 use crate::signal::Signal;
-use crate::wcstringutil::join_strings;
 use bitflags::bitflags;
-use fish_wchar::{L, WString, wstr};
+use fish_common::assert_sorted_by_name;
+use fish_wcstringutil::join_strings;
+use fish_widestring::{L, WString, wstr};
 use libc::c_int;
 use std::collections::HashMap;
 use std::sync::Arc;
@@ -322,6 +323,7 @@ mod tests {
     use crate::env::environment::{EnvStack, Environment};
     use crate::prelude::*;
     use crate::tests::prelude::*;
+    use assert_matches::assert_matches;
     use std::{
         mem::MaybeUninit,
         time::{SystemTime, UNIX_EPOCH},
@@ -360,7 +362,7 @@ mod tests {
         let first_tstamp = return_timezone_hour(tstamp, L!("UTC-1"));
         let second_tstamp = return_timezone_hour(tstamp, L!("UTC-2"));
         let delta = second_tstamp - first_tstamp;
-        assert!(delta == 1 || delta == -23);
+        assert_matches!(delta, 1 | -23);
     }
 
     // Verify that setting special env vars have the expected effect on the current shell process.
