@@ -6,9 +6,9 @@ use crate::flog::{flog, flogf};
 use crate::fs::{PotentialUpdate, lock_and_load, rewrite_via_temporary_file};
 use crate::path::path_get_config;
 use crate::prelude::*;
-use crate::wcstringutil::{LineIterator, join_strings};
 use crate::wutil::{FileId, INVALID_FILE_ID, file_id_for_file, file_id_for_path_narrow, wrealpath};
-use fish_wchar::decode_byte_from_char;
+use fish_wcstringutil::{LineIterator, join_strings};
+use fish_widestring::decode_byte_from_char;
 use std::collections::HashSet;
 use std::collections::hash_map::Entry;
 use std::ffi::CString;
@@ -103,7 +103,7 @@ impl EnvUniversal {
             Entry::Vacant(entry) => {
                 entry.insert(var);
             }
-        };
+        }
         self.modified.insert(key.to_owned());
         if exports {
             self.export_generation += 1;
@@ -248,6 +248,7 @@ impl EnvUniversal {
             let Ok(line) = std::str::from_utf8(line) else {
                 continue;
             };
+            // TODO: investigate whether this should use str2wcstring
             wide_line = WString::from_str(line);
 
             match format {
@@ -807,12 +808,10 @@ fn skip_spaces(mut s: &wstr) -> &wstr {
 
 #[cfg(test)]
 mod tests {
-    use fish_common::ENCODE_DIRECT_BASE;
-    use fish_common::char_offset;
     use fish_tempfile::TempDir;
+    use fish_widestring::{ENCODE_DIRECT_BASE, char_offset};
 
-    use crate::common::bytes2wcstring;
-    use crate::common::wcs2osstring;
+    use crate::common::{osstr2wcstring, wcs2osstring};
     use crate::env::{EnvVar, EnvVarFlags, VarTable};
     use crate::env_universal_common::{EnvUniversal, UvarFormat};
     use crate::prelude::*;
@@ -827,7 +826,7 @@ mod tests {
     fn make_test_uvar_path() -> std::io::Result<(TempDir, WString)> {
         let temp_dir = fish_tempfile::new_dir()?;
         let file_path = temp_dir.path().join("varsfile.txt");
-        let file_path = bytes2wcstring(file_path.as_os_str().as_encoded_bytes());
+        let file_path = osstr2wcstring(file_path);
         Ok((temp_dir, file_path))
     }
 
