@@ -8,9 +8,8 @@ use crate::builtins::shared::{
     STATUS_INVALID_ARGS, STATUS_NOT_EXECUTABLE, STATUS_READ_TOO_MUCH, STATUS_UNMATCHED_WILDCARD,
 };
 use crate::common::{
-    EXPAND_RESERVED_BASE, EXPAND_RESERVED_END, EscapeFlags, EscapeStringStyle, UnescapeFlags,
-    UnescapeStringStyle, charptr2wcstring, escape, escape_string, escape_string_for_double_quotes,
-    unescape_string, valid_var_name_char, wcs2zstring,
+    EscapeFlags, EscapeStringStyle, UnescapeFlags, UnescapeStringStyle, escape, escape_string,
+    escape_string_for_double_quotes, osstr2wcstring, unescape_string, valid_var_name_char,
 };
 use crate::complete::{CompleteFlags, Completion, CompletionList, CompletionReceiver};
 use crate::env::{EnvVar, Environment};
@@ -19,19 +18,18 @@ use crate::future_feature_flags::{FeatureFlag, feature_test};
 use crate::history::{History, history_session_id};
 use crate::operation_context::OperationContext;
 use crate::parse_constants::{ParseError, ParseErrorCode, ParseErrorList, SOURCE_LOCATION_UNKNOWN};
-use crate::parse_util::{
-    MaybeParentheses, parse_util_expand_variable_error, parse_util_locate_cmdsubst_range,
-};
+use crate::parse_util::{MaybeParentheses, expand_variable_error, locate_cmdsubst_range};
 use crate::path::path_apply_working_directory;
 use crate::prelude::*;
-use crate::util::wcsfilecmp_glob;
-use crate::wcstringutil::{join_strings, trim};
 use crate::wildcard::{ANY_CHAR, ANY_STRING, ANY_STRING_RECURSIVE, WildcardResult};
 use crate::wildcard::{wildcard_expand_string, wildcard_has_internal};
 use crate::wutil::{Options, normalize_path, wcstoi_partial};
 use bitflags::bitflags;
-use fish_common::char_offset;
-use std::mem::MaybeUninit;
+use fish_common::{EXPAND_RESERVED_BASE, EXPAND_RESERVED_END};
+use fish_util::wcsfilecmp_glob;
+use fish_wcstringutil::{join_strings, trim};
+use fish_widestring::char_offset;
+use nix::unistd::User;
 
 bitflags! {
     /// Set of flags controlling expansions.
@@ -115,7 +113,7 @@ impl ExpandResult {
     }
     /// Make an error value with the given status.
     pub fn make_error(status: libc::c_int) -> Self {
-        assert!(status != 0, "status cannot be 0 for an error result");
+        assert_ne!(status, 0, "status cannot be 0 for an error result");
         Self {
             result: ExpandResultCode::error,
             status,
@@ -623,7 +621,7 @@ fn expand_variables(
     // It's an error if the name is empty.
     if var_name.is_empty() {
         if let Some(errors) = errors {
-            parse_util_expand_variable_error(
+            expand_variable_error(
                 &instr,
                 0, /* global_token_pos */
                 varexp_char_idx,
@@ -845,7 +843,7 @@ fn expand_braces(
             // that.
             let mut synth = WString::new();
             if let Some(last_sep) = last_sep {
-                synth.push_utfstr(&input[..brace_begin.unwrap() + 1]);
+                synth.push_utfstr(&input[..=brace_begin.unwrap()]);
                 synth.push_utfstr(&input[last_sep + 1..]);
                 synth.push(BRACE_END);
             } else {
@@ -929,7 +927,7 @@ pub fn expand_cmdsubst(
     let mut cursor = 0;
     let mut is_quoted = false;
     let mut has_dollar = false;
-    let parens = match parse_util_locate_cmdsubst_range(
+    let parens = match locate_cmdsubst_range(
         &input,
         &mut cursor,
         false,
@@ -1153,25 +1151,11 @@ fn expand_home_directory(input: &mut WString, vars: &dyn Environment) {
                 home = Some(home_var.as_string());
                 tail_idx = 1;
             }
-        };
+        }
     } else {
-        // Some other user's home directory.
-        let name_cstr = wcs2zstring(username);
-        let mut userinfo = MaybeUninit::uninit();
-        let mut result: *mut libc::passwd = std::ptr::null_mut();
-        let mut buf = [0 as libc::c_char; 8192];
-        let retval = unsafe {
-            libc::getpwnam_r(
-                name_cstr.as_ptr(),
-                userinfo.as_mut_ptr(),
-                &mut buf[0],
-                std::mem::size_of_val(&buf),
-                &mut result,
-            )
-        };
-        if retval == 0 && !result.is_null() {
-            let userinfo = unsafe { userinfo.assume_init() };
-            home = Some(charptr2wcstring(userinfo.pw_dir));
+        // POSIX-conforming usernames are ASCII-only
+        if let Ok(Some(userinfo)) = User::from_name(&username.to_string()) {
+            home = Some(osstr2wcstring(userinfo.dir));
         }
     }
 
@@ -1329,7 +1313,7 @@ impl<'a, 'b, 'c> Expander<'a, 'b, 'c> {
         }
         if self.flags.contains(ExpandFlags::FAIL_ON_CMDSUBST) {
             let mut cursor = 0;
-            match parse_util_locate_cmdsubst_range(&input, &mut cursor, true, None, None) {
+            match locate_cmdsubst_range(&input, &mut cursor, true, None, None) {
                 MaybeParentheses::Error => ExpandResult::make_error(STATUS_EXPAND_ERROR),
                 MaybeParentheses::None => {
                     if !out.add(input) {
@@ -1543,7 +1527,7 @@ impl<'a, 'b, 'c> Expander<'a, 'b, 'c> {
         // Get the username_with_tilde (like ~bert) and expand it into a home directory.
         let mut tail_idx = usize::MAX;
         let username_with_tilde =
-            WString::from_str("~") + get_home_directory_name(input, &mut tail_idx);
+            L!("~").to_owned() + get_home_directory_name(input, &mut tail_idx);
         let mut home = username_with_tilde.clone();
         expand_tilde(&mut home, self.ctx.vars());
 
@@ -1592,6 +1576,7 @@ mod tests {
     use crate::abbrs::Abbreviation;
     use crate::abbrs::{self};
     use crate::abbrs::{with_abbrs, with_abbrs_mut};
+    use crate::common::str2wcstring;
     use crate::complete::{CompletionList, CompletionReceiver};
     use crate::env::{EnvMode, EnvStackSetResult};
     use crate::expand::{ExpandResultCode, expand_to_receiver};
@@ -1954,7 +1939,7 @@ mod tests {
         // Make a list of 64 elements, then expand it cartesian-style 64 times.
         // This is far too large to expand.
         let vals: Vec<WString> = (1..=64).map(|i| i.to_wstring()).collect();
-        let expansion = WString::from_str(&str::repeat("$bigvar", 64));
+        let expansion = str2wcstring(str::repeat("$bigvar", 64));
 
         let parser = TestParser::new();
         parser.vars().push(true);
