@@ -1,10 +1,7 @@
+use crate::portable_atomic::AtomicU64;
 use cfg_if::cfg_if;
-#[cfg(not(target_has_atomic = "64"))]
-use portable_atomic::AtomicU64;
 use std::collections::HashMap;
 use std::os::unix::prelude::*;
-#[cfg(target_has_atomic = "64")]
-use std::sync::atomic::AtomicU64;
 use std::sync::atomic::Ordering;
 use std::sync::{Arc, Mutex};
 use std::time::Duration;
@@ -91,13 +88,8 @@ impl FdEventSignaller {
         );
         let mut ret;
         loop {
-            ret = unsafe {
-                libc::read(
-                    self.read_fd(),
-                    buff.as_mut_ptr().cast(),
-                    std::mem::size_of_val(&buff),
-                )
-            };
+            ret =
+                unsafe { libc::read(self.read_fd(), buff.as_mut_ptr().cast(), size_of_val(&buff)) };
             if ret >= 0 || errno().0 != EINTR {
                 break;
             }
@@ -200,7 +192,7 @@ impl FdMonitorItem {
     /// Invoke this item's callback because the fd is readable.
     /// If the given fd is closed, it will be removed from the [`FdMonitor`] set.
     fn service(&mut self) {
-        (self.callback)(&mut self.fd)
+        (self.callback)(&mut self.fd);
     }
 }
 
@@ -472,18 +464,16 @@ impl Drop for FdMonitor {
 
 #[cfg(test)]
 mod tests {
-    #[cfg(not(target_has_atomic = "64"))]
-    use portable_atomic::AtomicU64;
+    use crate::portable_atomic::AtomicU64;
     use std::fs::File;
     use std::io::Write;
     use std::os::fd::{AsRawFd, OwnedFd};
-    #[cfg(target_has_atomic = "64")]
-    use std::sync::atomic::AtomicU64;
     use std::sync::atomic::{AtomicUsize, Ordering};
     use std::sync::{Arc, Barrier, Mutex};
     use std::thread;
     use std::time::Duration;
 
+    use assert_matches::assert_matches;
     use errno::errno;
 
     use crate::fd_monitor::{FdEventSignaller, FdMonitor};
@@ -705,8 +695,9 @@ mod tests {
             Some(read_fd)
         };
         let result = do_something_bad_during_select(dup2_it);
-        assert!(
-            matches!(result, Err(libc::EBADF) | Ok(0 | 1)),
+        assert_matches!(
+            result,
+            Err(libc::EBADF) | Ok(0 | 1),
             "select/poll should have failed with EBADF or timed out or the fd should be ready"
         );
         // Ensure these stay alive until after thread is joined.
