@@ -13,6 +13,7 @@ use crate::parse_tree::NodeRef;
 use crate::parser::Parser;
 use crate::parser_keywords::parser_keywords_is_reserved;
 use crate::prelude::*;
+use crate::proc::Pid;
 use crate::wutil::dir_iter::DirIter;
 use std::collections::{HashMap, HashSet};
 use std::num::NonZeroU32;
@@ -341,7 +342,7 @@ pub fn get_names(get_hidden: bool, vars: &dyn Environment) -> Vec<WString> {
         };
         if !get_hidden && (bname.is_empty() || bname.starts_with('_')) {
             continue;
-        };
+        }
         let Some(fname) = bname.strip_suffix(".fish") else {
             continue;
         };
@@ -384,7 +385,6 @@ impl FunctionProperties {
     pub fn definition_lineno(&self) -> i32 {
         // Return one plus the number of newlines at offsets less than the start of our function's
         // statement (which includes the header).
-        // TODO: merge with line_offset_of_character_at_offset?
         let Some(source_range) = self.func_node.try_source_range() else {
             panic!("Function has no source range");
         };
@@ -450,11 +450,11 @@ impl FunctionProperties {
                     sprintf!(=> &mut out, " --on-variable %s", name);
                 }
                 EventDescription::ProcessExit { pid } => {
-                    let pid = pid.map(|p| p.get()).unwrap_or(0);
-                    sprintf!(=> &mut out, " --on-process-exit %d", pid)
+                    let pid = pid.as_ref().map_or(0, Pid::get);
+                    sprintf!(=> &mut out, " --on-process-exit %d", pid);
                 }
                 EventDescription::JobExit { pid, .. } => {
-                    let pid = pid.map(|p| p.get()).unwrap_or(0);
+                    let pid = pid.as_ref().map_or(0, Pid::get);
                     sprintf!(=> &mut out, " --on-job-exit %d", pid);
                 }
                 EventDescription::CallerExit { .. } => {
@@ -466,7 +466,7 @@ impl FunctionProperties {
                 EventDescription::Any => {
                     panic!("Unexpected event handler type");
                 }
-            };
+            }
         }
 
         let named = &self.named_arguments;
