@@ -12,15 +12,16 @@ use crate::operation_context::OperationContext;
 use crate::path::path_apply_working_directory;
 use crate::redirection::RedirectionMode;
 use crate::threads::assert_is_background_thread;
-use crate::wcstringutil::{
-    string_prefixes_string, string_prefixes_string_case_insensitive, string_suffixes_string,
-};
 use crate::wildcard::{ANY_CHAR, ANY_STRING, ANY_STRING_RECURSIVE};
 use crate::wutil::{
     dir_iter::DirIter, fish_wcstoi, normalize_path, waccess, wbasename, wdirname, wstat,
 };
-use fish_wchar::{L, WExt, WString, wstr};
+use fish_wcstringutil::{
+    string_prefixes_string, string_prefixes_string_case_insensitive, string_suffixes_string,
+};
+use fish_widestring::{L, WExt, WString, wstr};
 use libc::PATH_MAX;
+use nix::unistd::AccessFlags;
 use std::collections::{HashMap, HashSet};
 use std::os::fd::RawFd;
 
@@ -159,7 +160,7 @@ impl<'s> FileTester<'s> {
                 // Input redirections must have a readable non-directory.
                 // Note we color "try_input" files as errors if they are invalid,
                 // even though it's possible to execute these (replaced via /dev/null).
-                if waccess(&target_path, libc::R_OK) == 0
+                if waccess(&target_path, AccessFlags::R_OK).is_ok()
                     && wstat(&target_path).is_ok_and(|md| !md.file_type().is_dir())
                 {
                     Ok(IsFile(true))
@@ -182,8 +183,8 @@ impl<'s> FileTester<'s> {
                         // No err. We can write to it if it's not a directory and we have
                         // permission.
                         file_exists = true;
-                        file_is_writable =
-                            !md.file_type().is_dir() && waccess(&target_path, libc::W_OK) == 0;
+                        file_is_writable = !md.file_type().is_dir()
+                            && waccess(&target_path, AccessFlags::W_OK).is_ok();
                     }
                     Err(err) => {
                         if err.raw_os_error() == Some(libc::ENOENT) {
@@ -200,7 +201,7 @@ impl<'s> FileTester<'s> {
                             // Now the file is considered writable if the parent directory is
                             // writable.
                             file_exists = false;
-                            file_is_writable = waccess(&parent, libc::W_OK) == 0;
+                            file_is_writable = waccess(&parent, AccessFlags::W_OK).is_ok();
                         } else {
                             // Other errors we treat as not writable. This includes things like
                             // ENOTDIR.
@@ -404,7 +405,7 @@ fn fs_is_case_insensitive(
 ) -> bool {
     if let Some(cached) = case_sensitivity_cache.get(path) {
         return *cached;
-    };
+    }
     // Ask the system. A -1 value means error (so assume case sensitive), a 1 value means case
     // sensitive, and a 0 value means case insensitive.
     let ret = unsafe { libc::fpathconf(fd, libc::_PC_CASE_SENSITIVE) };
@@ -426,6 +427,7 @@ pub fn fs_is_case_insensitive(
 #[cfg(test)]
 mod tests {
     use super::{FileTester, IsErr, IsFile, PathFlags, is_potential_path};
+    use crate::common::osstr2wcstring;
     use crate::env::EnvStack;
     use crate::operation_context::{EXPANSION_LIMIT_DEFAULT, OperationContext};
     use crate::prelude::*;
@@ -454,10 +456,7 @@ mod tests {
         }
 
         fn file_tester(&self) -> FileTester<'_> {
-            FileTester::new(
-                WString::from_str(self.tempdir.path().to_str().unwrap()),
-                &self.ctx,
-            )
+            FileTester::new(osstr2wcstring(self.tempdir.path()), &self.ctx)
         }
     }
 
