@@ -5,10 +5,7 @@ use crate::ast::{
     Keyword, Kind, Node, NodeVisitor, Redirection, Token, VariableAssignment,
 };
 use crate::builtins::shared::builtin_exists;
-use crate::color::Color;
-use crate::common::{
-    ASCII_MAX, EXPAND_RESERVED_BASE, EXPAND_RESERVED_END, valid_var_name, valid_var_name_char,
-};
+use crate::common::{valid_var_name, valid_var_name_char};
 use crate::complete::complete_wrap_map;
 use crate::env::{EnvVar, Environment};
 use crate::expand::{
@@ -17,21 +14,23 @@ use crate::expand::{
 use crate::function;
 use crate::future_feature_flags::{FeatureFlag, feature_test};
 use crate::highlight::file_tester::FileTester;
-use crate::history::{HistoryItem, all_paths_are_valid};
+use crate::history::all_paths_are_valid;
 use crate::operation_context::OperationContext;
 use crate::parse_constants::{
     ParseKeyword, ParseTokenType, ParseTreeFlags, SourceRange, StatementDecoration,
 };
 use crate::parse_util::{
-    MaybeParentheses, parse_util_locate_cmdsubst_range, parse_util_slice_length,
+    MaybeParentheses, get_process_first_token_offset, locate_cmdsubst_range, slice_length,
 };
 use crate::path::{path_as_implicit_cd, path_get_cdpath, path_get_path, paths_are_same_file};
 use crate::terminal::Outputter;
 use crate::text_face::{SpecifiedTextFace, TextFace, UnderlineStyle, parse_text_face};
 use crate::threads::assert_is_background_thread;
 use crate::tokenizer::{PipeOrRedir, variable_assignment_equals_pos};
-use crate::wcstringutil::string_prefixes_string;
-use fish_wchar::{L, WExt, WString, wstr};
+use fish_color::Color;
+use fish_common::{ASCII_MAX, EXPAND_RESERVED_BASE, EXPAND_RESERVED_END};
+use fish_wcstringutil::string_prefixes_string;
+use fish_widestring::{L, WExt, WString, wstr};
 use std::collections::HashMap;
 use std::collections::hash_map::Entry;
 
@@ -62,7 +61,7 @@ impl HighlightSpec {
 /// Given a string and list of colors of the same size, return the string with ANSI escape sequences
 /// representing the colors.
 pub fn colorize(text: &wstr, colors: &[HighlightSpec], vars: &dyn Environment) -> Vec<u8> {
-    assert!(colors.len() == text.len());
+    assert_eq!(colors.len(), text.len());
     let mut rv = HighlightColorResolver::new();
     let mut outp = Outputter::new_buffering();
 
@@ -106,6 +105,22 @@ pub fn highlight_shell(
     *color = highlighter.highlight();
 }
 
+pub fn highlight_and_colorize(
+    text: &wstr,
+    ctx: &OperationContext<'_>,
+    vars: &dyn Environment,
+) -> Vec<u8> {
+    let mut colors = Vec::new();
+    highlight_shell(
+        text,
+        &mut colors,
+        ctx,
+        /*io_ok=*/ false,
+        /*cursor=*/ None,
+    );
+    colorize(text, &colors, vars)
+}
+
 /// highlight_color_resolver_t resolves highlight specs (like "a command") to actual RGB colors.
 /// It maintains a cache with no invalidation mechanism. The lifetime of these should typically be
 /// one screen redraw.
@@ -242,23 +257,23 @@ fn command_is_valid(
 
     // Builtins
     if !is_valid && builtin_ok {
-        is_valid = builtin_exists(cmd)
-    };
+        is_valid = builtin_exists(cmd);
+    }
 
     // Functions
     if !is_valid && function_ok {
-        is_valid = function::exists_no_autoload(cmd)
-    };
+        is_valid = function::exists_no_autoload(cmd);
+    }
 
     // Abbreviations
     if !is_valid && abbreviation_ok {
-        is_valid = with_abbrs(|set| set.has_match(cmd, abbrs::Position::Command, L!("")))
-    };
+        is_valid = with_abbrs(|set| set.has_match(cmd, abbrs::Position::Command, L!("")));
+    }
 
     // Regular commands
     if !is_valid && command_ok {
-        is_valid = path_get_path(cmd, vars).is_some()
-    };
+        is_valid = path_get_path(cmd, vars).is_some();
+    }
 
     // Implicit cd
     if !is_valid && implicit_cd_ok {
@@ -284,11 +299,12 @@ fn autosuggest_parse_command(
     buff: &wstr,
     ctx: &OperationContext<'_>,
 ) -> Option<(WString, WString)> {
-    let ast = ast::parse(
-        buff,
-        ParseTreeFlags::CONTINUE_AFTER_ERROR | ParseTreeFlags::ACCEPT_INCOMPLETE_TOKENS,
-        None,
-    );
+    let flags = ParseTreeFlags {
+        continue_after_error: true,
+        accept_incomplete_tokens: true,
+        ..Default::default()
+    };
+    let ast = ast::parse(buff, flags, None);
 
     // Find the first statement.
     let job_list: &ast::JobList = ast.top();
@@ -318,14 +334,25 @@ pub fn is_veritable_cd(expanded_command: &wstr) -> bool {
 /// autosuggestion is valid. It may not be valid if e.g. it is attempting to cd into a directory
 /// which does not exist.
 pub fn autosuggest_validate_from_history(
-    item: &HistoryItem,
+    item_commandline: &wstr,
+    suggested_range: std::ops::Range<usize>,
+    required_paths: &[WString],
     working_directory: &wstr,
     ctx: &OperationContext<'_>,
 ) -> bool {
     assert_is_background_thread();
 
+    if suggested_range != (0..item_commandline.char_count())
+        && get_process_first_token_offset(item_commandline, suggested_range.start)
+            .is_some_and(|offset| offset != suggested_range.start)
+    {
+        return false;
+    }
+
     // Parse the string.
-    let Some((parsed_command, mut cd_dir)) = autosuggest_parse_command(item.str(), ctx) else {
+    let suggested_command = &item_commandline[suggested_range];
+    let Some((parsed_command, mut cd_dir)) = autosuggest_parse_command(suggested_command, ctx)
+    else {
         // This is for autosuggestions which are not decorated commands, e.g. function declarations.
         return true;
     };
@@ -357,8 +384,7 @@ pub fn autosuggest_validate_from_history(
     }
 
     // Did the historical command have arguments that look like paths, which aren't paths now?
-    let paths = item.get_required_paths();
-    if !all_paths_are_valid(paths.iter().cloned(), ctx) {
+    if !all_paths_are_valid(required_paths, ctx) {
         return false;
     }
 
@@ -368,7 +394,7 @@ pub fn autosuggest_validate_from_history(
 // Highlights the variable starting with 'in', setting colors within the 'colors' array. Returns the
 // number of characters consumed.
 fn color_variable(inp: &wstr, colors: &mut [HighlightSpec]) -> usize {
-    assert!(inp.char_at(0) == '$');
+    assert_eq!(inp.char_at(0), '$');
 
     // Handle an initial run of $s.
     let mut idx = 0;
@@ -407,7 +433,7 @@ fn color_variable(inp: &wstr, colors: &mut [HighlightSpec]) -> usize {
     // Handle a slice, up to dollar_count of them. Note that we currently don't do any validation of
     // the slice's contents, e.g. $foo[blah] will not show an error even though it's invalid.
     for _slice_count in 0..dollar_count {
-        match parse_util_slice_length(&inp[idx..]) {
+        match slice_length(&inp[idx..]) {
             Some(slice_len) if slice_len > 0 => {
                 colors[idx] = HighlightSpec::with_fg(HighlightRole::operat);
                 colors[idx + slice_len - 1] = HighlightSpec::with_fg(HighlightRole::operat);
@@ -422,7 +448,7 @@ fn color_variable(inp: &wstr, colors: &mut [HighlightSpec]) -> usize {
                 // double-quoted string that doesn't happen. As such, color the variable + the slice
                 // start red. Coloring any more than that looks bad, unless we're willing to try and
                 // detect where the double-quoted string ends, and I'd rather not do that.
-                colors[..idx + 1].fill(HighlightSpec::with_fg(HighlightRole::error));
+                colors[..=idx].fill(HighlightSpec::with_fg(HighlightRole::error));
                 break;
             }
         }
@@ -716,11 +742,14 @@ impl<'s> Highlighter<'s> {
             .resize(self.buff.len(), HighlightSpec::default());
 
         // Flags we use for AST parsing.
-        let ast_flags = ParseTreeFlags::CONTINUE_AFTER_ERROR
-            | ParseTreeFlags::INCLUDE_COMMENTS
-            | ParseTreeFlags::ACCEPT_INCOMPLETE_TOKENS
-            | ParseTreeFlags::LEAVE_UNTERMINATED
-            | ParseTreeFlags::SHOW_EXTRA_SEMIS;
+        let ast_flags = ParseTreeFlags {
+            continue_after_error: true,
+            include_comments: true,
+            accept_incomplete_tokens: true,
+            leave_unterminated: true,
+            show_extra_semis: true,
+            ..Default::default()
+        };
         let ast = ast::parse(self.buff, ast_flags, None);
 
         self.visit_children(ast.top());
@@ -798,7 +827,7 @@ impl<'s> Highlighter<'s> {
         // Now do command substitutions.
         let mut cmdsub_cursor = 0;
         let mut is_quoted = false;
-        while let MaybeParentheses::CommandSubstitution(parens) = parse_util_locate_cmdsubst_range(
+        while let MaybeParentheses::CommandSubstitution(parens) = locate_cmdsubst_range(
             arg_str,
             &mut cmdsub_cursor,
             /*accept_incomplete=*/ true,
@@ -828,13 +857,13 @@ impl<'s> Highlighter<'s> {
             let subcolors = cmdsub_highlighter.highlight();
 
             // Copy out the subcolors back into our array.
-            assert!(subcolors.len() == cmdsub_contents.len());
+            assert_eq!(subcolors.len(), cmdsub_contents.len());
             self.color_array[arg_start..][parens.command()].copy_from_slice(&subcolors);
         }
     }
     // Colors the source range of a node with a given color.
     fn color_node(&mut self, node: &dyn ast::Node, color: HighlightSpec) {
-        self.color_range(node.source_range(), color)
+        self.color_range(node.source_range(), color);
     }
     // Colors a range with a given color.
     fn color_range(&mut self, range: SourceRange, color: HighlightSpec) {
@@ -869,14 +898,14 @@ impl<'s> Highlighter<'s> {
             | ParseKeyword::Exclam
             | ParseKeyword::Time => role = HighlightRole::operat,
             ParseKeyword::None => (),
-        };
+        }
         self.color_node(node.as_node(), HighlightSpec::with_fg(role));
     }
     fn visit_token(&mut self, tok: &dyn Token) {
         let mut role = HighlightRole::normal;
         match tok.token_type() {
             ParseTokenType::End | ParseTokenType::Pipe | ParseTokenType::Background => {
-                role = HighlightRole::statement_terminator
+                role = HighlightRole::statement_terminator;
             }
             ParseTokenType::LeftBrace | ParseTokenType::RightBrace => {
                 role = HighlightRole::keyword;
@@ -951,24 +980,29 @@ impl<'s> Highlighter<'s> {
         }
         // No command substitution, so we can highlight the target file or fd. For example,
         // disallow redirections into a non-existent directory.
-        let target_is_valid = if !self.io_still_ok() {
+        let (role, file_exists) = if !self.io_still_ok() {
             // I/O is disallowed, so we don't have much hope of catching anything but gross
             // errors. Assume it's valid.
-            true
+            (HighlightRole::redirection, false)
         } else if contains_pending_variable(&self.pending_variables, &target) {
-            true
+            // Target uses a variable defined by the current commandline. Assume it's valid.
+            (HighlightRole::redirection, false)
         } else {
             // Validate the redirection target..
-            self.file_tester.test_redirection_target(&target, oper.mode)
-        };
-        self.color_node(
-            &redir.target,
-            HighlightSpec::with_fg(if target_is_valid {
-                HighlightRole::redirection
+            if let Ok(IsFile(file_exists)) =
+                self.file_tester.test_redirection_target(&target, oper.mode)
+            {
+                (HighlightRole::redirection, file_exists)
             } else {
-                HighlightRole::error
-            }),
-        );
+                (HighlightRole::error, false)
+            }
+        };
+        self.color_node(&redir.target, HighlightSpec::with_fg(role));
+        if file_exists {
+            for i in redir.target.source_range().as_usize() {
+                self.color_array[i].valid_path = true;
+            }
+        }
     }
 
     fn visit_variable_assignment(&mut self, varas: &VariableAssignment) {
@@ -985,7 +1019,7 @@ impl<'s> Highlighter<'s> {
         self.color_node(
             node,
             HighlightSpec::with_fg(HighlightRole::statement_terminator),
-        )
+        );
     }
     fn visit_decorated_statement(&mut self, stmt: &DecoratedStatement) {
         // Color any decoration.
@@ -1027,7 +1061,7 @@ impl<'s> Highlighter<'s> {
         if is_valid_cmd {
             self.color_command(&stmt.command);
         } else {
-            self.color_node(&stmt.command, HighlightSpec::with_fg(HighlightRole::error))
+            self.color_node(&stmt.command, HighlightSpec::with_fg(HighlightRole::error));
         }
 
         // Color arguments and redirections.
@@ -1082,7 +1116,7 @@ impl<'s> Highlighter<'s> {
 /// Return whether a string contains a command substitution.
 fn has_cmdsub(src: &wstr) -> bool {
     let mut cursor = 0;
-    match parse_util_locate_cmdsubst_range(src, &mut cursor, true, None, None) {
+    match locate_cmdsubst_range(src, &mut cursor, true, None, None) {
         MaybeParentheses::Error => false,
         MaybeParentheses::None => false,
         MaybeParentheses::CommandSubstitution(_) => true,
@@ -1274,7 +1308,7 @@ pub struct HighlightSpec {
 mod tests {
     use super::{HighlightColorResolver, HighlightRole, HighlightSpec, highlight_shell};
     use crate::common::ScopeGuard;
-    use crate::env::{EnvMode, Environment};
+    use crate::env::{EnvMode, EnvSetMode, Environment};
     use crate::future_feature_flags::{self, FeatureFlag};
     use crate::highlight::parse_text_face_for_highlight;
     use crate::operation_context::{EXPANSION_LIMIT_BACKGROUND, OperationContext};
@@ -1376,6 +1410,9 @@ mod tests {
         let mut param_valid_path = HighlightSpec::with_fg(HighlightRole::param);
         param_valid_path.valid_path = true;
 
+        let mut redirection_valid_path = HighlightSpec::with_fg(HighlightRole::redirection);
+        redirection_valid_path.valid_path = true;
+
         let saved_flag = future_feature_flags::test(FeatureFlag::AmpersandNoBgInToken);
         future_feature_flags::set(FeatureFlag::AmpersandNoBgInToken, true);
         let _restore_saved_flag = ScopeGuard::new((), |_| {
@@ -1386,26 +1423,15 @@ mod tests {
 
         // Verify variables and wildcards in commands using /bin/cat.
         let vars = parser.vars();
-        vars.set_one(
-            L!("CDPATH"),
-            EnvMode::LOCAL,
-            L!("./cdpath-entry").to_owned(),
-        );
+        let local_mode = EnvSetMode::new_at_early_startup(EnvMode::LOCAL);
+        vars.set_one(L!("CDPATH"), local_mode, L!("./cdpath-entry").to_owned());
 
-        vars.set_one(
-            L!("VARIABLE_IN_COMMAND"),
-            EnvMode::LOCAL,
-            L!("a").to_owned(),
-        );
-        vars.set_one(
-            L!("VARIABLE_IN_COMMAND2"),
-            EnvMode::LOCAL,
-            L!("at").to_owned(),
-        );
+        vars.set_one(L!("VARIABLE_IN_COMMAND"), local_mode, L!("a").to_owned());
+        vars.set_one(L!("VARIABLE_IN_COMMAND2"), local_mode, L!("at").to_owned());
 
         let _cleanup = ScopeGuard::new((), |_| {
-            vars.remove(L!("VARIABLE_IN_COMMAND"), EnvMode::default());
-            vars.remove(L!("VARIABLE_IN_COMMAND2"), EnvMode::default());
+            vars.remove(L!("VARIABLE_IN_COMMAND"), EnvSetMode::default());
+            vars.remove(L!("VARIABLE_IN_COMMAND2"), EnvSetMode::default());
         });
 
         validate!(
@@ -1524,7 +1550,7 @@ mod tests {
             ("param1", fg(HighlightRole::param)),
             // Input redirection.
             ("<", fg(HighlightRole::redirection)),
-            ("/bin/echo", fg(HighlightRole::redirection)),
+            ("/dev/null", redirection_valid_path),
             // Output redirection to a valid fd.
             ("1>&2", fg(HighlightRole::redirection)),
             // Output redirection to an invalid fd.
@@ -1798,7 +1824,7 @@ mod tests {
         // First, set up fish_color_command to include underline
         vars.set_one(
             L!("fish_color_command"),
-            EnvMode::LOCAL,
+            EnvSetMode::new_at_early_startup(EnvMode::LOCAL),
             L!("--underline").to_owned(),
         );
 
@@ -1850,7 +1876,7 @@ mod tests {
         let vars = parser.vars();
 
         let set = |var: &wstr, value: Vec<WString>| {
-            vars.set(var, EnvMode::LOCAL, value);
+            vars.set(var, EnvSetMode::new(EnvMode::LOCAL, false), value);
         };
         set(L!("fish_color_normal"), vec![L!("normal").into()]);
         set(
