@@ -22,12 +22,13 @@ use crate::{
         rewrite_via_temporary_file,
     },
     threads::ThreadPool,
-    wcstringutil::trim,
 };
+use fish_wcstringutil::{subsequence_in_string, trim};
+use fish_widestring::subslice_position;
 use std::{
     borrow::Cow,
     collections::{BTreeMap, HashMap, HashSet},
-    ffi::CString,
+    ffi::{CStr, CString},
     fs::File,
     io::{BufRead, BufWriter, Read, Write},
     mem::MaybeUninit,
@@ -56,13 +57,11 @@ use crate::{
     localization::wgettext_fmt,
     operation_context::{EXPANSION_LIMIT_BACKGROUND, OperationContext},
     parse_constants::{ParseTreeFlags, StatementDecoration},
-    parse_util::{parse_util_detect_errors, parse_util_unescape_wildcards},
+    parse_util::{detect_parse_errors, unescape_wildcards},
     parser::Parser,
     path::{path_get_config, path_get_data, path_is_valid},
     prelude::*,
     threads::assert_is_background_thread,
-    util::find_subslice,
-    wcstringutil::subsequence_in_string,
     wildcard::{ANY_STRING, wildcard_match},
     wutil::{FileId, INVALID_FILE_ID, file_id_for_file, wrealpath, wstat, wunlink},
 };
@@ -83,8 +82,6 @@ pub enum SearchType {
     PrefixGlob,
     /// Search for commands containing the given string as a subsequence
     ContainsSubsequence,
-    /// Matches everything.
-    MatchEverything,
 }
 
 /// Ways that a history item may be written to disk (or omitted).
@@ -135,9 +132,9 @@ impl Drop for TimeProfiler {
                 self.what,
                 ms as u64, // todo!("remove cast")
                 ns as u32
-            )
+            );
         } else {
-            flogf!(profile_history, "%s: ??? ms", self.what)
+            flogf!(profile_history, "%s: ??? ms", self.what);
         }
     }
 }
@@ -219,7 +216,7 @@ impl HistoryItem {
         match typ {
             SearchType::Exact => term == *content_to_match,
             SearchType::Contains => {
-                find_subslice(term.as_slice(), content_to_match.as_slice()).is_some()
+                subslice_position(content_to_match.as_slice(), term.as_slice()).is_some()
             }
             SearchType::Prefix => content_to_match.as_slice().starts_with(term.as_slice()),
             SearchType::LinePrefix => content_to_match
@@ -227,7 +224,7 @@ impl HistoryItem {
                 .split(|&c| c == '\n')
                 .any(|line| line.starts_with(term.as_char_slice())),
             SearchType::ContainsGlob => {
-                let mut pat = parse_util_unescape_wildcards(term);
+                let mut pat = unescape_wildcards(term);
                 if !pat.starts_with(ANY_STRING) {
                     pat.insert(0, ANY_STRING);
                 }
@@ -237,14 +234,13 @@ impl HistoryItem {
                 wildcard_match(content_to_match.as_ref(), &pat, false)
             }
             SearchType::PrefixGlob => {
-                let mut pat = parse_util_unescape_wildcards(term);
+                let mut pat = unescape_wildcards(term);
                 if !pat.ends_with(ANY_STRING) {
                     pat.push(ANY_STRING);
                 }
                 wildcard_match(content_to_match.as_ref(), &pat, false)
             }
             SearchType::ContainsSubsequence => subsequence_in_string(term, &content_to_match),
-            SearchType::MatchEverything => true,
         }
     }
 
@@ -700,7 +696,7 @@ impl HistoryImpl {
         if !ok {
             // We did not or could not append; rewrite the file ("vacuum" it).
             if let Err(e) = self.save_internal_via_rewrite(&history_path) {
-                flog!(history, "Rewriting history failed:", e)
+                flog!(history, "Rewriting history failed:", e);
             }
         }
     }
@@ -820,7 +816,7 @@ impl HistoryImpl {
     /// Enable / disable automatic saving. Main thread only!
     fn disable_automatic_saving(&mut self) {
         self.disable_automatic_save_counter += 1;
-        assert!(self.disable_automatic_save_counter != 0); // overflow!
+        assert_ne!(self.disable_automatic_save_counter, 0); // overflow!
     }
 
     fn enable_automatic_saving(&mut self) {
@@ -835,7 +831,7 @@ impl HistoryImpl {
         self.first_unwritten_new_item_index = 0;
         self.file_contents = None;
         if let Ok(Some(filename)) = self.history_file_path() {
-            wunlink(&filename);
+            let _ = wunlink(&filename);
         }
         self.clear_file_state();
     }
@@ -969,7 +965,7 @@ impl HistoryImpl {
             }
 
             if seen.insert(item.str().to_owned()) {
-                result.push(item.str().to_owned())
+                result.push(item.str().to_owned());
             }
         }
 
@@ -1107,13 +1103,12 @@ fn do_1_history_search(
 }
 
 /// Formats a single history record, including a trailing newline.
-///
-/// Returns nothing. The only possible failure involves formatting the timestamp. If that happens we
-/// simply omit the timestamp from the output.
 fn format_history_record(
     item: &HistoryItem,
     show_time_format: Option<&str>,
     null_terminate: bool,
+    parser: &Parser,
+    color_enabled: bool,
 ) -> WString {
     let mut result = WString::new();
     let seconds = time_to_seconds(item.timestamp());
@@ -1134,12 +1129,24 @@ fn format_history_record(
                 )
             } != 0
             {
-                result.push_utfstr(&cstr2wcstring(&timestamp_str[..]));
+                // SAFETY: strftime terminates the string with a null byte. If there is insufficient
+                // space, strftime returns 0.
+                let timestamp_cstr = CStr::from_bytes_until_nul(&timestamp_str).unwrap();
+                result.push_utfstr(&cstr2wcstring(timestamp_cstr));
             }
         }
     }
 
-    result.push_utfstr(item.str());
+    let mut command = item.str().to_owned();
+    if color_enabled {
+        command = bytes2wcstring(&highlight_and_colorize(
+            &command,
+            &parser.context(),
+            parser.vars(),
+        ));
+    }
+
+    result.push_utfstr(&command);
     result.push(if null_terminate { '\0' } else { '\n' });
     result
 }
@@ -1167,18 +1174,18 @@ fn should_import_bash_history_line(line: &wstr) -> bool {
     // Skip lines with [[...]] and ((...)) since we don't handle those constructs.
     // "<<" here is a proxy for heredocs (and herestrings).
     for seq in [L!("[["), L!("]]"), L!("(("), L!("))"), L!("<<")] {
-        if find_subslice(seq, line.as_char_slice()).is_some() {
+        if subslice_position(line.as_char_slice(), seq).is_some() {
             return false;
         }
     }
 
-    if ast::parse(line, ParseTreeFlags::empty(), None).errored() {
+    if ast::parse(line, ParseTreeFlags::default(), None).errored() {
         return false;
     }
 
     // In doing this test do not allow incomplete strings. Hence the "false" argument.
     let mut errors = Vec::new();
-    let _ = parse_util_detect_errors(line, Some(&mut errors), false);
+    let _ = detect_parse_errors(line, Some(&mut errors), false);
     errors.is_empty()
 }
 
@@ -1193,14 +1200,14 @@ impl History {
     /// call to resolve_pending. Any trailing ephemeral items are dropped.
     /// Exposed for testing.
     pub fn add(&self, item: HistoryItem, pending: bool) {
-        self.imp().add(item, pending, true)
+        self.imp().add(item, pending, true);
     }
 
     pub fn add_commandline(&self, s: WString) {
         let mut imp = self.imp();
         let when = imp.timestamp_now();
         let item = HistoryItem::new(s, when, PersistenceMode::Disk);
-        imp.add(item, false, true)
+        imp.add(item, false, true);
     }
 
     pub fn new(name: &wstr) -> Arc<Self> {
@@ -1232,12 +1239,12 @@ impl History {
 
     /// Remove a history item.
     pub fn remove(&self, s: &wstr) {
-        self.imp().remove(s)
+        self.imp().remove(s);
     }
 
     /// Remove any trailing ephemeral items.
     pub fn remove_ephemeral_items(&self) {
-        self.imp().remove_ephemeral_items()
+        self.imp().remove_ephemeral_items();
     }
 
     /// Add a new pending history item to the end, and then begin file detection on the items to
@@ -1257,7 +1264,7 @@ impl History {
 
         // Find all arguments that look like they could be file paths.
         let mut needs_sync_write = false;
-        let ast = ast::parse(s, ParseTreeFlags::empty(), None);
+        let ast = ast::parse(s, ParseTreeFlags::default(), None);
 
         let mut potential_paths = Vec::new();
         for node in ast.walk() {
@@ -1332,12 +1339,12 @@ impl History {
 
     /// Resolves any pending history items, so that they may be returned in history searches.
     pub fn resolve_pending(&self) {
-        self.imp().resolve_pending()
+        self.imp().resolve_pending();
     }
 
     /// Saves history.
     pub fn save(&self) {
-        self.imp().save(false)
+        self.imp().save(false);
     }
 
     /// Searches history.
@@ -1366,16 +1373,13 @@ impl History {
                 return ControlFlow::Break(());
             }
             remaining -= 1;
-            let mut formatted_record =
-                format_history_record(item, show_time_format, null_terminate);
-
-            if color_enabled {
-                formatted_record = bytes2wcstring(&highlight_and_colorize(
-                    &formatted_record,
-                    &parser.context(),
-                    parser.vars(),
-                ));
-            }
+            let formatted_record = format_history_record(
+                item,
+                show_time_format,
+                null_terminate,
+                parser,
+                color_enabled,
+            );
 
             if reverse {
                 // We need to collect this for later.
@@ -1395,9 +1399,9 @@ impl History {
             // The user had no search terms; just append everything.
             do_1_history_search(
                 Arc::clone(self),
-                SearchType::MatchEverything,
+                SearchType::Contains,
                 WString::new(),
-                false,
+                true,
                 &mut func,
                 cancel_check,
             );
@@ -1441,27 +1445,27 @@ impl History {
 
     /// Irreversibly clears history.
     pub fn clear(&self) {
-        self.imp().clear()
+        self.imp().clear();
     }
 
     /// Irreversibly clears history for the current session.
     pub fn clear_session(&self) {
-        self.imp().clear_session()
+        self.imp().clear_session();
     }
 
     /// Populates from older location (in config path, rather than data path).
     pub fn populate_from_config_path(&self) {
-        self.imp().populate_from_config_path()
+        self.imp().populate_from_config_path();
     }
 
     /// Populates from a bash history file.
     pub fn populate_from_bash<R: BufRead>(&self, contents: R) {
-        self.imp().populate_from_bash(contents)
+        self.imp().populate_from_bash(contents);
     }
 
     /// Incorporates the history of other shells into this history.
     pub fn incorporate_external_changes(&self) {
-        self.imp().incorporate_external_changes()
+        self.imp().incorporate_external_changes();
     }
 
     /// Gets all the history into a list. This is intended for the $history environment variable.
@@ -1525,13 +1529,16 @@ pub struct HistorySearch {
 }
 
 impl HistorySearch {
-    pub fn new(hist: Arc<History>, s: WString) -> Self {
+    #[cfg(test)]
+    fn new(hist: Arc<History>, s: WString) -> Self {
         Self::new_with(hist, s, SearchType::Contains, SearchFlags::default(), 0)
     }
-    pub fn new_with_type(hist: Arc<History>, s: WString, search_type: SearchType) -> Self {
+    #[cfg(test)]
+    fn new_with_type(hist: Arc<History>, s: WString, search_type: SearchType) -> Self {
         Self::new_with(hist, s, search_type, SearchFlags::default(), 0)
     }
-    pub fn new_with_flags(hist: Arc<History>, s: WString, flags: SearchFlags) -> Self {
+    #[cfg(test)]
+    fn new_with_flags(hist: Arc<History>, s: WString, flags: SearchFlags) -> Self {
         Self::new_with(hist, s, SearchType::Contains, flags, 0)
     }
     /// Constructs a new history search.
@@ -1566,7 +1573,7 @@ impl HistorySearch {
     }
 
     pub fn prepare_to_search_after_deletion(&mut self) {
-        assert!(self.current_index != 0);
+        assert_ne!(self.current_index, 0);
         self.current_index -= 1;
         self.current_item = None;
     }
@@ -1588,7 +1595,7 @@ impl HistorySearch {
             match direction {
                 SearchDirection::Backward => index += 1,
                 SearchDirection::Forward => index -= 1,
-            };
+            }
 
             if self.current_index == invalid_index {
                 return false;
@@ -1636,6 +1643,10 @@ impl HistorySearch {
         self.current_item.as_ref().expect("No current item")
     }
 
+    pub fn canon_term(&self) -> &wstr {
+        &self.canon_term
+    }
+
     /// Returns the current search result item contents.
     ///
     /// # Panics
@@ -1733,13 +1744,12 @@ pub fn expand_and_detect_paths<P: IntoIterator<Item = WString>>(
 /// Given a list of proposed paths and a context, expand each one and see if it refers to a file.
 /// Wildcard expansions are suppressed.
 /// Returns `true` if `paths` is empty or every path is valid.
-pub fn all_paths_are_valid<P: IntoIterator<Item = WString>>(
-    paths: P,
-    ctx: &OperationContext<'_>,
-) -> bool {
+pub fn all_paths_are_valid(paths: &[WString], ctx: &OperationContext<'_>) -> bool {
     assert_is_background_thread();
     let working_directory = ctx.vars().get_pwd_slash();
-    for mut path in paths {
+    let mut path = WString::new();
+    for unexpanded_path in paths {
+        path.clone_from(unexpanded_path);
         if ctx.check_cancel() {
             return false;
         }
@@ -1776,20 +1786,18 @@ mod tests {
         History, HistoryItem, HistorySearch, PathList, PersistenceMode, SearchDirection,
         SearchFlags, SearchType, VACUUM_FREQUENCY,
     };
-    use crate::common::ESCAPE_TEST_CHAR;
-    use crate::common::{ScopeGuard, bytes2wcstring, wcs2bytes, wcs2osstring};
+    use crate::common::{ESCAPE_TEST_CHAR, ScopeGuard, osstr2wcstring, wcs2bytes, wcs2osstring};
     use crate::env::{EnvMode, EnvSetMode, EnvStack};
     use crate::fs::{LockedFile, WriteMethod};
     use crate::path::path_get_data;
     use crate::prelude::*;
     use crate::tests::prelude::*;
-    use crate::wcstringutil::{string_prefixes_string, string_prefixes_string_case_insensitive};
     use fish_build_helper::workspace_root;
+    use fish_wcstringutil::{string_prefixes_string, string_prefixes_string_case_insensitive};
     use rand::Rng;
     use rand::rngs::ThreadRng;
     use std::collections::VecDeque;
     use std::io::BufReader;
-    use std::os::unix::ffi::OsStrExt;
     use std::sync::Arc;
     use std::time::UNIX_EPOCH;
     use std::time::{Duration, SystemTime};
@@ -1933,7 +1941,7 @@ mod tests {
         let mut rng = rand::rng();
         for i in 1..=max {
             // Generate a value.
-            let mut value = WString::from_str("test item ") + &i.to_wstring()[..];
+            let mut value = L!("test item ").to_owned() + &i.to_wstring()[..];
 
             // Maybe add some backslashes.
             if i % 3 == 0 {
@@ -2027,7 +2035,7 @@ mod tests {
         });
         if LockedFile::new(
             crate::fs::LockingMode::Exclusive(WriteMethod::RenameIntoPlace),
-            &bytes2wcstring(tmp_path.as_os_str().as_bytes()),
+            &osstr2wcstring(&tmp_path),
         )
         .is_err()
         {
@@ -2103,7 +2111,7 @@ mod tests {
                 );
                 for list in &expected_lines {
                     if !list.is_empty() {
-                        printf!("\tRemaining: %s\n", list.last().unwrap())
+                        printf!("\tRemaining: %s\n", list.last().unwrap());
                     }
                 }
             }
@@ -2180,7 +2188,7 @@ mod tests {
 
         // Save them.
         for hist in &hists {
-            hist.save()
+            hist.save();
         }
 
         // Make sure each history contains what it ought to, but they have not leaked into each other.
