@@ -4,23 +4,25 @@ use crate::common::{
 };
 use crate::env::{EnvStack, Environment};
 use crate::fd_readable_set::{FdReadableSet, Timeout};
-use crate::flog::{FLOG, FloggableDebug, FloggableDisplay};
+use crate::flog::{FloggableDebug, FloggableDisplay, flog};
+use crate::future_feature_flags::{FeatureFlag, test as feature_test};
 use crate::key::{
     self, Key, Modifiers, ViewportPosition, alt, canonicalize_control_char,
     canonicalize_keyed_control_char, char_to_symbol, function_key, shift,
 };
+use crate::prelude::*;
 use crate::reader::reader_test_and_clear_interrupted;
 use crate::tty_handoff::{
-    SCROLL_CONTENT_UP_TERMINFO_CODE, XTVERSION, maybe_set_kitty_keyboard_capability,
-    maybe_set_scroll_content_up_capability,
+    SCROLL_CONTENT_UP_TERMINFO_CODE, TERMINAL_OS_NAME, XTGETTCAP_QUERY_OS_NAME, XTVERSION,
+    maybe_set_kitty_keyboard_capability, maybe_set_scroll_content_up_capability,
 };
 use crate::universal_notifier::default_notifier;
-use crate::wchar::{encode_byte_to_char, prelude::*};
 use crate::wutil::{fish_is_pua, fish_wcstol};
+use fish_widestring::encode_byte_to_char;
+use nix::sys::{select::FdSet, signal::SigSet, time::TimeSpec};
 use std::cell::{RefCell, RefMut};
 use std::collections::VecDeque;
-use std::mem::MaybeUninit;
-use std::os::fd::RawFd;
+use std::os::fd::{BorrowedFd, RawFd};
 use std::sync::atomic::{AtomicUsize, Ordering};
 use std::time::Duration;
 
@@ -48,13 +50,21 @@ pub enum ReadlineCmd {
     BackwardCharPassive,
     ForwardSingleChar,
     ForwardCharPassive,
-    ForwardWord,
     BackwardWord,
-    ForwardBigword,
+    ForwardWordEmacs,
+    ForwardBigwordEmacs,
     BackwardBigword,
+    ForwardWordEnd,
+    BackwardWordEnd,
+    ForwardBigwordEnd,
+    BackwardBigwordEnd,
+    ForwardWordVi,
+    ForwardBigwordVi,
+    ForwardPathComponent,
     ForwardToken,
+    BackwardPathComponent,
     BackwardToken,
-    NextdOrForwardWord,
+    NextdOrForwardWordEmacs,
     PrevdOrBackwardWord,
     HistoryDelete,
     HistorySearchBackward,
@@ -77,8 +87,15 @@ pub enum ReadlineCmd {
     BackwardKillLine,
     KillWholeLine,
     KillInnerLine,
-    KillWord,
-    KillBigword,
+    KillWordEmacs,
+    KillBigwordEmacs,
+    KillWordVi,
+    KillBigwordVi,
+    KillInnerWord,
+    KillInnerBigWord,
+    KillAWord,
+    KillABigWord,
+    KillPathComponent,
     KillToken,
     BackwardKillWord,
     BackwardKillPathComponent,
@@ -229,7 +246,7 @@ fn apply_shift(mut key: Key, do_ascii: bool, shifted_codepoint: char) -> Option<
         key.codepoint = key.codepoint.to_ascii_uppercase();
     } else {
         return None;
-    };
+    }
     key.modifiers.shift = false;
     Some(key)
 }
@@ -324,13 +341,16 @@ pub enum ImplicitEvent {
     FocusOut,
     /// Mouse left click.
     MouseLeft(ViewportPosition),
+    /// Terminal color theme change (light/dark mode).
+    NewColorTheme,
     /// Window height changed.
-    WindowHeight,
+    NewWindowHeight,
 }
 
 #[derive(Debug, Clone)]
 pub enum QueryResponse {
     PrimaryDeviceAttribute,
+    BackgroundColor(xterm_color::Color),
     CursorPosition(ViewportPosition),
 }
 
@@ -471,7 +491,7 @@ fn readb(in_fd: RawFd) -> Option<u8> {
         return None;
     }
     let c = arr[0];
-    FLOG!(reader, "Read byte", char_to_symbol(char::from(c), true));
+    flog!(reader, "Read byte", char_to_symbol(char::from(c), true));
     // The common path is to return a u8.
     Some(c)
 }
@@ -531,52 +551,34 @@ fn next_input_event(in_fd: RawFd, ioport_fd: RawFd, timeout: Timeout) -> InputEv
     }
 }
 
-pub fn check_fd_readable(in_fd: RawFd, timeout: Duration) -> bool {
-    use std::ptr;
+pub fn check_fd_readable(in_fd: BorrowedFd, timeout: Duration) -> bool {
     // We are not prepared to handle a signal immediately; we only want to know if we get input on
     // our fd before the timeout. Use pselect to block all signals; we will handle signals
     // before the next call to readch().
-    let mut sigs = MaybeUninit::uninit();
-    let mut sigs = unsafe {
-        libc::sigfillset(sigs.as_mut_ptr());
-        sigs.assume_init()
-    };
-
-    // pselect expects timeouts in nanoseconds.
-    const NSEC_PER_MSEC: u64 = 1000 * 1000;
-    const NSEC_PER_SEC: u64 = NSEC_PER_MSEC * 1000;
-    let wait_nsec: u64 = (timeout.as_millis() as u64) * NSEC_PER_MSEC;
-    let timeout = libc::timespec {
-        tv_sec: (wait_nsec / NSEC_PER_SEC).try_into().unwrap(),
-        tv_nsec: (wait_nsec % NSEC_PER_SEC).try_into().unwrap(),
-    };
 
     // We have one fd of interest.
-    let mut fdset = MaybeUninit::uninit();
-    let mut fdset = unsafe {
-        libc::FD_ZERO(fdset.as_mut_ptr());
-        fdset.assume_init()
+    let mut readfds = {
+        let mut set = FdSet::new();
+        set.insert(in_fd);
+        set
     };
-    unsafe {
-        libc::FD_SET(in_fd, &mut fdset);
-    }
 
-    let res = unsafe {
-        libc::pselect(
-            in_fd + 1,
-            &mut fdset,
-            ptr::null_mut(),
-            ptr::null_mut(),
-            &timeout,
-            &sigs,
-        )
-    };
+    let res = nix::sys::select::pselect(
+        None,
+        &mut readfds,
+        None,
+        None,
+        &TimeSpec::from_duration(timeout),
+        &SigSet::all(),
+    )
+    .unwrap();
 
     // Prevent signal starvation on WSL causing the `torn_escapes.py` test to fail
     if is_windows_subsystem_for_linux(WSL::V1) {
         // Merely querying the current thread's sigmask is sufficient to deliver a pending signal
-        let _ = unsafe { libc::pthread_sigmask(0, ptr::null(), &mut sigs) };
+        _ = SigSet::thread_get_mask().expect("Failed to get sigmask!");
     }
+
     res > 0
 }
 
@@ -600,7 +602,7 @@ pub fn update_wait_on_escape_ms(vars: &EnvStack) {
                     "is not an integer or is < 10 or >= 5000 ms\n"
                 ),
                 fish_escape_delay_ms
-            )
+            );
         }
     }
 }
@@ -625,7 +627,7 @@ pub fn update_wait_on_sequence_key_ms(vars: &EnvStack) {
                     "is not an integer or is < 10 or >= 5000 ms\n"
                 ),
                 sequence_key_time_ms
-            )
+            );
         }
     }
 }
@@ -696,13 +698,18 @@ impl InputData {
     }
 }
 
-#[derive(Clone, Eq, PartialEq)]
+#[derive(Clone, Debug, Default, Eq, PartialEq)]
+pub struct BackgroundColorQuery {
+    pub result: Option<xterm_color::Color>,
+}
+
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub enum CursorPositionQueryReason {
     NewPrompt,
     WindowHeightChange,
 }
 
-#[derive(Clone, Eq, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub struct CursorPositionQuery {
     pub reason: CursorPositionQueryReason,
     pub result: Option<ViewportPosition>,
@@ -717,12 +724,20 @@ impl CursorPositionQuery {
     }
 }
 
-#[derive(Eq, PartialEq)]
+#[derive(Clone, Debug, Default, Eq, PartialEq)]
+pub struct RecurrentQuery {
+    pub background_color: Option<BackgroundColorQuery>,
+    pub cursor_position: Option<CursorPositionQuery>,
+}
+
+#[derive(Clone, Eq, PartialEq)]
 pub enum TerminalQuery {
     Initial,
-    CursorPosition(CursorPositionQuery),
+    Recurrent(RecurrentQuery),
 }
 
+pub const LONG_READ_TIMEOUT: Duration = Duration::from_secs(2);
+
 /// A trait which knows how to produce a stream of input events.
 /// Note this is conceptually a "base class" with override points.
 pub trait InputEventQueuer {
@@ -848,7 +863,7 @@ pub trait InputEventQueuer {
                         )
                     };
                     if self.is_blocked_querying() {
-                        FLOG!(
+                        flog!(
                             reader,
                             "Still blocked on response from terminal, deferring key event",
                             key_evt
@@ -859,14 +874,14 @@ pub trait InputEventQueuer {
                                 self.push_back(evt);
                             }
                         });
-                        let vintr = shell_modes().c_cc[libc::VINTR];
+                        let vintr = shell_modes().control_chars[libc::VINTR];
                         if vintr != 0
                             && key.is_some_and(|key| {
                                 match_key_event_to_key(&key, &Key::from_single_byte(vintr))
                                     .is_some()
                             })
                         {
-                            FLOG!(
+                            flog!(
                                 reader,
                                 "Received interrupt key, giving up waiting for response from terminal"
                             );
@@ -887,22 +902,39 @@ pub trait InputEventQueuer {
         }
     }
 
-    fn try_readb(&mut self, buffer: &mut Vec<u8>) -> Option<u8> {
+    fn read_sequence_byte(&mut self, buffer: &mut Vec<u8>) -> Option<u8> {
         let fd = self.get_in_fd();
-        if !check_fd_readable(
-            fd,
-            Duration::from_millis(if self.paste_is_buffering() || self.is_blocked_querying() {
-                300
-            } else if buffer == b"\x1b" {
-                1 // distinguish legacy escape
+        let strict = feature_test(FeatureFlag::OmitTermWorkarounds);
+        let historical_millis = |ms| {
+            if strict {
+                LONG_READ_TIMEOUT
             } else {
-                30
-            }),
+                Duration::from_millis(ms)
+            }
+        };
+        if !check_fd_readable(
+            unsafe { BorrowedFd::borrow_raw(fd) },
+            if self.paste_is_buffering() || self.is_blocked_querying() {
+                historical_millis(300)
+            } else if buffer == b"\x1b" {
+                Duration::from_millis(1) // distinguish legacy escape
+            } else {
+                historical_millis(30)
+            },
         ) {
-            FLOG!(
+            flog!(
                 reader,
                 format!("Incomplete escape sequence: {}", DisplayBytes(buffer))
             );
+            if buffer != b"\x1b" && strict {
+                flog!(
+                    error,
+                    format!(
+                        "Incomplete escape sequence seen (logging because omit-term-workarounds is on): {}",
+                        DisplayBytes(buffer)
+                    )
+                );
+            }
             return None;
         }
         let next = readb(fd)?;
@@ -917,7 +949,7 @@ pub trait InputEventQueuer {
     ) -> Option<KeyEvent> {
         assert!(buffer.len() <= 2);
         let recursive_invocation = buffer.len() == 2;
-        let Some(next) = self.try_readb(buffer) else {
+        let Some(next) = self.read_sequence_byte(buffer) else {
             return Some(KeyEvent::from_raw(key::Escape));
         };
         let invalid = KeyEvent::from_raw(key::Invalid);
@@ -943,9 +975,16 @@ pub trait InputEventQueuer {
             // potential SS3
             return Some(self.parse_ss3(buffer).unwrap_or(invalid));
         }
-        if !recursive_invocation && next == b'P' {
-            // potential DCS
-            return Some(self.parse_dcs(buffer).unwrap_or(invalid));
+        if !recursive_invocation {
+            if next == b']' {
+                // OSC
+                self.parse_osc(buffer);
+                return Some(invalid);
+            }
+            if next == b'P' {
+                // potential DCS
+                return Some(self.parse_dcs(buffer).unwrap_or(invalid));
+            }
         }
         match canonicalize_control_char(next) {
             Some(mut key) => {
@@ -962,10 +1001,10 @@ pub trait InputEventQueuer {
     fn parse_csi(&mut self, buffer: &mut Vec<u8>) -> Option<KeyEvent> {
         // The maximum number of CSI parameters is defined by NPAR, nominally 16.
         let mut params = [[0_u32; 4]; 16];
-        let Some(mut c) = self.try_readb(buffer) else {
+        let Some(mut c) = self.read_sequence_byte(buffer) else {
             return Some(KeyEvent::from(alt('[')));
         };
-        let mut next_char = |zelf: &mut Self| zelf.try_readb(buffer).unwrap_or(0xff);
+        let mut next_char = |zelf: &mut Self| zelf.read_sequence_byte(buffer).unwrap_or(0xff);
         let private_mode;
         if matches!(c, b'?' | b'<' | b'=' | b'>') {
             // private mode
@@ -985,7 +1024,7 @@ pub trait InputEventQueuer {
                 {
                     Some(c) => params[count][subcount] = c,
                     None => return invalid_sequence(buffer),
-                };
+                }
             } else if c == b':' && subcount < 3 {
                 subcount += 1;
             } else if c == b';' {
@@ -1051,7 +1090,7 @@ pub trait InputEventQueuer {
             b'F' => masked_key(key::End),  // PC/xterm style
             b'H' => masked_key(key::Home), // PC/xterm style
             b'M' | b'm' => {
-                FLOG!(reader, "mouse event");
+                flog!(reader, "mouse event");
                 // Generic X10 or modified VT200 sequence, or extended (SGR/1006) mouse
                 // reporting mode, with semicolon-separated parameters for button code, Px,
                 // and Py, ending with 'M' for button press or 'm' for button release.
@@ -1091,14 +1130,14 @@ pub trait InputEventQueuer {
                 return None;
             }
             b't' => {
-                FLOG!(reader, "mouse event");
+                flog!(reader, "mouse event");
                 // VT200 button released in mouse highlighting mode at valid text location. 5 chars.
                 let _ = next_char(self);
                 let _ = next_char(self);
                 return None;
             }
             b'T' => {
-                FLOG!(reader, "mouse event");
+                flog!(reader, "mouse event");
                 // VT200 button released in mouse highlighting mode past end-of-line. 9 characters.
                 for _ in 0..6 {
                     let _ = next_char(self);
@@ -1120,11 +1159,9 @@ pub trait InputEventQueuer {
                 else {
                     return invalid_sequence(buffer);
                 };
-                FLOG!(reader, "Received cursor position report y:", y, "x:", x);
+                flog!(reader, "Received cursor position report y:", y, "x:", x);
                 let cursor_pos = ViewportPosition { x, y };
-                self.push_front(CharEvent::QueryResult(QueryResultEvent::Response(
-                    QueryResponse::CursorPosition(cursor_pos),
-                )));
+                self.push_query_response(QueryResponse::CursorPosition(cursor_pos));
                 return None;
             }
             b'S' => masked_key(function_key(4)),
@@ -1175,10 +1212,17 @@ pub trait InputEventQueuer {
                 _ => return None,
             },
             b'c' if private_mode == Some(b'?') => {
-                FLOG!(reader, "Received Primary Device Attribute response");
-                self.push_front(CharEvent::QueryResult(QueryResultEvent::Response(
-                    QueryResponse::PrimaryDeviceAttribute,
-                )));
+                flog!(reader, "Received Primary Device Attribute response");
+                self.push_query_response(QueryResponse::PrimaryDeviceAttribute);
+                return None;
+            }
+            b'n' if private_mode == Some(b'?') && params[0] == [997, 0, 0, 0] => {
+                match params[1] {
+                    [1, 0, 0, 0] | [2, 0, 0, 0] => (),
+                    _ => return None,
+                }
+                flog!(reader, "Received color theme change");
+                self.push_front(CharEvent::Implicit(ImplicitEvent::NewColorTheme));
                 return None;
             }
             b'u' => {
@@ -1253,12 +1297,12 @@ pub trait InputEventQueuer {
 
     fn parse_ss3(&mut self, buffer: &mut Vec<u8>) -> Option<KeyEvent> {
         let mut raw_mask = 0;
-        let Some(mut code) = self.try_readb(buffer) else {
+        let Some(mut code) = self.read_sequence_byte(buffer) else {
             return Some(KeyEvent::from(alt('O')));
         };
         while code.is_ascii_digit() {
             raw_mask = raw_mask * 10 + u32::from(code - b'0');
-            code = self.try_readb(buffer).unwrap_or(0xff);
+            code = self.read_sequence_byte(buffer).unwrap_or(0xff);
         }
         let (modifiers, _caps_lock) = parse_mask(raw_mask.saturating_sub(1));
         #[rustfmt::skip]
@@ -1298,29 +1342,36 @@ pub trait InputEventQueuer {
         Some(key)
     }
 
-    fn read_until_sequence_terminator(&mut self, buffer: &mut Vec<u8>) -> Option<()> {
+    fn read_until_sequence_terminator(
+        &mut self,
+        buffer: &mut Vec<u8>,
+        allow_bel: bool,
+    ) -> Option<()> {
         let mut escape = false;
         loop {
-            let b = self.try_readb(buffer)?;
+            let b = self.read_sequence_byte(buffer)?;
+            if allow_bel && b == b'\x07' {
+                buffer.pop();
+                return Some(());
+            }
             if escape && b == b'\\' {
-                break;
+                buffer.pop();
+                buffer.pop();
+                return Some(());
             }
             escape = b == b'\x1b';
         }
-        buffer.pop();
-        buffer.pop();
-        Some(())
     }
 
     fn parse_xtversion(&mut self, buffer: &mut Vec<u8>) -> Option<()> {
         assert_eq!(buffer, b"\x1bP>");
-        self.read_until_sequence_terminator(buffer)?;
+        self.read_until_sequence_terminator(buffer, false)?;
         if buffer.get(3)? != &b'|' {
             return None;
         }
         XTVERSION.get_or_init(|| {
             let xtversion = bytes2wcstring(&buffer[4..buffer.len()]);
-            FLOG!(
+            flog!(
                 reader,
                 format!("Received XTVERSION response: {}", xtversion)
             );
@@ -1329,9 +1380,21 @@ pub trait InputEventQueuer {
         None
     }
 
+    fn parse_osc(&mut self, buffer: &mut Vec<u8>) -> Option<()> {
+        let osc_prefix = b"\x1b]";
+        assert_eq!(buffer, osc_prefix);
+        self.read_until_sequence_terminator(buffer, /*allow_bel=*/ true)?;
+        let buffer = &buffer[osc_prefix.len()..];
+        let buffer = buffer.strip_prefix(b"11;")?;
+        let c = xterm_color::Color::parse(buffer).ok()?;
+        flog!(reader, format!("Received background color {c:?}"));
+        self.push_query_response(QueryResponse::BackgroundColor(c));
+        None
+    }
+
     fn parse_dcs(&mut self, buffer: &mut Vec<u8>) -> Option<KeyEvent> {
-        assert!(buffer.len() == 2);
-        let Some(success) = self.try_readb(buffer) else {
+        assert_eq!(buffer, b"\x1bP");
+        let Some(success) = self.read_sequence_byte(buffer) else {
             return Some(KeyEvent::from(alt('P')));
         };
         let success = match success {
@@ -1343,18 +1406,18 @@ pub trait InputEventQueuer {
             }
             _ => return None,
         };
-        if self.try_readb(buffer)? != b'+' {
+        if self.read_sequence_byte(buffer)? != b'+' {
             return None;
         }
-        if self.try_readb(buffer)? != b'r' {
+        if self.read_sequence_byte(buffer)? != b'r' {
             return None;
         }
-        self.read_until_sequence_terminator(buffer)?;
+        self.read_until_sequence_terminator(buffer, false)?;
         // \e P 1 r + Pn ST
         // \e P 0 r + msg ST
         let buffer = &buffer[5..];
         if !success {
-            FLOG!(
+            flog!(
                 reader,
                 format!(
                     "Received XTGETTCAP failure response: {}",
@@ -1366,9 +1429,9 @@ pub trait InputEventQueuer {
         let mut buffer = buffer.splitn(2, |&c| c == b'=');
         let key = buffer.next().unwrap();
         let key = parse_hex(key)?;
-        if let Some(value) = buffer.next() {
+        let value = if let Some(value) = buffer.next() {
             let value = parse_hex(value)?;
-            FLOG!(
+            flog!(
                 reader,
                 format!(
                     "Received XTGETTCAP response: {}={:?}",
@@ -1376,16 +1439,22 @@ pub trait InputEventQueuer {
                     bytes2wcstring(&value)
                 )
             );
+            Some(value)
         } else {
-            FLOG!(
+            flog!(
                 reader,
                 format!("Received XTGETTCAP response: {}", bytes2wcstring(&key))
             );
-        }
+            None
+        };
         if key == SCROLL_CONTENT_UP_TERMINFO_CODE.as_bytes() {
             maybe_set_scroll_content_up_capability();
+        } else if key == XTGETTCAP_QUERY_OS_NAME.as_bytes() {
+            if let Some(value) = value {
+                TERMINAL_OS_NAME.get_or_init(|| Some(bytes2wcstring(&value)));
+            }
         }
-        return None;
+        None
     }
 
     fn readch_timed_esc(&mut self) -> Option<CharEvent> {
@@ -1409,7 +1478,7 @@ pub trait InputEventQueuer {
         }
 
         check_fd_readable(
-            self.get_in_fd(),
+            unsafe { BorrowedFd::borrow_raw(self.get_in_fd()) },
             Duration::from_millis(u64::try_from(wait_time_ms).unwrap()),
         )
         .then(|| self.readch())
@@ -1456,7 +1525,7 @@ pub trait InputEventQueuer {
             .paste_buffer
             .as_mut()
             .unwrap()
-            .push(b)
+            .push(b);
     }
 
     fn paste_commit(&mut self) {
@@ -1475,6 +1544,10 @@ pub trait InputEventQueuer {
         self.get_input_data_mut().queue.push_front(ch);
     }
 
+    fn push_query_response(&mut self, response: QueryResponse) {
+        self.push_front(CharEvent::QueryResult(QueryResultEvent::Response(response)));
+    }
+
     /// Find the first sequence of non-char events, and promote them to the front.
     fn promote_interruptions_to_front(&mut self) {
         // Find the first sequence of non-char events.
@@ -1542,11 +1615,11 @@ pub trait InputEventQueuer {
     fn select_interrupted(&mut self) {}
 
     fn enqueue_interrupt_key(&mut self) {
-        let vintr = shell_modes().c_cc[libc::VINTR];
+        let vintr = shell_modes().control_chars[libc::VINTR];
         if vintr != 0 {
             let interrupt_evt = CharEvent::from_key(KeyEvent::from_single_byte(vintr));
             if stop_query(self.blocking_query()) {
-                FLOG!(
+                flog!(
                     reader,
                     "Received interrupt, giving up on waiting for terminal response"
                 );
@@ -1607,7 +1680,7 @@ pub(crate) fn decode_one_codepoint_utf8(
         Err(e) => match e.error_len() {
             Some(_) => match invalid_policy {
                 InvalidPolicy::Error => {
-                    FLOG!(reader, "Illegal input encoding");
+                    flog!(reader, "Illegal input encoding");
                     Error
                 }
                 InvalidPolicy::Passthrough => {
@@ -1627,7 +1700,7 @@ pub(crate) fn stop_query(mut query: RefMut<'_, Option<TerminalQuery>>) -> bool {
 }
 
 fn invalid_sequence(buffer: &[u8]) -> Option<KeyEvent> {
-    FLOG!(
+    flog!(
         reader,
         "Error: invalid escape sequence: ",
         DisplayBytes(buffer)
@@ -1684,15 +1757,20 @@ fn parse_hex(hex: &[u8]) -> Option<Vec<u8>> {
         return None;
     }
     let mut result = vec![0; hex.len() / 2];
+    parse_hex_into(&mut result, hex)?;
+    Some(result)
+}
+fn parse_hex_into(out: &mut [u8], hex: &[u8]) -> Option<()> {
+    assert_eq!(out.len() * 2, hex.len());
     let mut i = 0;
     while i < hex.len() {
         let d1 = char::from(hex[i]).to_digit(16)?;
         let d2 = char::from(hex[i + 1]).to_digit(16)?;
         let decoded = u8::try_from(16 * d1 + d2).unwrap();
-        result[i / 2] = decoded;
+        out[i / 2] = decoded;
         i += 2;
     }
-    Some(result)
+    Some(())
 }
 
 #[cfg(test)]
