@@ -60,12 +60,12 @@ impl KillRing {
 
 /// Add a string to the top of the killring.
 pub fn kill_add(new_entry: WString) {
-    KILL_RING.lock().unwrap().add(new_entry)
+    KILL_RING.lock().unwrap().add(new_entry);
 }
 
 /// Replace the specified string in the killring.
 pub fn kill_replace(old_entry: &wstr, new_entry: WString) {
-    KILL_RING.lock().unwrap().replace(old_entry, new_entry)
+    KILL_RING.lock().unwrap().replace(old_entry, new_entry);
 }
 
 /// Rotate the killring.
@@ -93,23 +93,23 @@ mod tests {
 
         assert!(kr.is_empty());
 
-        kr.add(WString::from_str("a"));
-        kr.add(WString::from_str("b"));
-        kr.add(WString::from_str("c"));
+        kr.add(L!("a").to_owned());
+        kr.add(L!("b").to_owned());
+        kr.add(L!("c").to_owned());
 
-        assert!(kr.entries() == [L!("c"), L!("b"), L!("a")]);
+        assert_eq!(kr.entries(), [L!("c"), L!("b"), L!("a")]);
 
-        assert!(kr.yank_rotate() == "b");
-        assert!(kr.entries() == [L!("b"), L!("a"), L!("c")]);
+        assert_eq!(kr.yank_rotate(), "b");
+        assert_eq!(kr.entries(), [L!("b"), L!("a"), L!("c")]);
 
-        assert!(kr.yank_rotate() == "a");
-        assert!(kr.entries() == [L!("a"), L!("c"), L!("b")]);
+        assert_eq!(kr.yank_rotate(), "a");
+        assert_eq!(kr.entries(), [L!("a"), L!("c"), L!("b")]);
 
-        kr.add(WString::from_str("d"));
+        kr.add(L!("d").to_owned());
 
-        assert!((kr.entries() == [L!("d"), L!("a"), L!("c"), L!("b")]));
+        assert_eq!(kr.entries(), [L!("d"), L!("a"), L!("c"), L!("b")]);
 
-        assert!(kr.yank_rotate() == "a");
-        assert!((kr.entries() == [L!("a"), L!("c"), L!("b"), L!("d")]));
+        assert_eq!(kr.yank_rotate(), "a");
+        assert_eq!(kr.entries(), [L!("a"), L!("c"), L!("b"), L!("d")]);
     }
 }
