@@ -14,7 +14,7 @@ use crate::operation_context::OperationContext;
 use crate::prelude::*;
 use crate::screen::{CharOffset, Line, ScreenData, wcswidth_rendered, wcwidth_rendered};
 use crate::termsize::Termsize;
-use crate::wcstringutil::string_fuzzy_match_string;
+use fish_wcstringutil::string_fuzzy_match_string;
 
 /// Represents rendering from the pager.
 #[derive(Default)]
@@ -80,7 +80,7 @@ const PAGER_SEARCH_FIELD_WIDTH: usize = 12;
 localizable_consts!(
     /// Text we use for the search field.
     SEARCH_FIELD_PROMPT
-    "search: "
+    "search:"
 );
 
 const PAGER_SELECTION_NONE: usize = usize::MAX;
@@ -327,7 +327,7 @@ impl Pager {
         let mut search_field_remaining = term_width - 1;
         search_field_remaining -= print_max(
             CharOffset::None,
-            wgettext!(SEARCH_FIELD_PROMPT),
+            &sprintf!("%s ", wgettext!(SEARCH_FIELD_PROMPT)),
             HighlightSpec::new(),
             search_field_remaining,
             false,
@@ -689,10 +689,10 @@ impl Pager {
                 // Handle the case of nothing selected yet.
                 match direction {
                     SelectionMotion::South | SelectionMotion::PageSouth | SelectionMotion::Next => {
-                        self.selected_completion_idx = Some(0)
+                        self.selected_completion_idx = Some(0);
                     }
                     SelectionMotion::North | SelectionMotion::Prev => {
-                        self.selected_completion_idx = Some(self.completion_infos.len() - 1)
+                        self.selected_completion_idx = Some(self.completion_infos.len() - 1);
                     }
                     SelectionMotion::East
                     | SelectionMotion::West
@@ -911,7 +911,7 @@ impl Pager {
     pub fn get_selected_row_given_rows(&self, rows: usize) -> Option<usize> {
         if rows == 0 {
             return None;
-        };
+        }
 
         self.selected_completion_idx.map(|idx| idx % rows)
     }
@@ -1043,7 +1043,8 @@ impl Pager {
 
     // Position of the cursor.
     pub fn cursor_position(&self) -> usize {
-        let mut result = wgettext!(SEARCH_FIELD_PROMPT).len() + self.search_field_line.position();
+        let mut result =
+            wgettext!(SEARCH_FIELD_PROMPT).len() + 1 + self.search_field_line.position();
         // Clamp it to the right edge.
         if self.available_term_width > 0 && result + 1 > self.available_term_width {
             result = self.available_term_width - 1;
@@ -1102,7 +1103,7 @@ fn divide_round_up(numer: usize, denom: usize) -> usize {
     if numer == 0 {
         return 0;
     }
-    assert!(denom != 0);
+    assert_ne!(denom, 0);
     let has_rem = (numer % denom) != 0;
     numer / denom + if has_rem { 1 } else { 0 }
 }
@@ -1287,7 +1288,7 @@ mod tests {
     use crate::prelude::*;
     use crate::termsize::Termsize;
     use crate::tests::prelude::*;
-    use crate::wcstringutil::StringFuzzyMatch;
+    use fish_wcstringutil::StringFuzzyMatch;
     use std::borrow::Cow;
     use std::num::NonZeroU16;
 
