@@ -22,23 +22,23 @@ use crate::parse_constants::{
     ParseTokenType, ParseTreeFlags, ParserTestErrorBits, PipelinePosition, SourceRange,
     StatementDecoration, UNKNOWN_BUILTIN_ERR_MSG, parse_error_offset_source_start,
 };
+use crate::prelude::*;
 use crate::tokenizer::{
     TOK_ACCEPT_UNFINISHED, TOK_SHOW_COMMENTS, Tok, TokenType, Tokenizer, comment_end,
     is_token_delimiter, quote_end,
 };
-use crate::wchar::prelude::*;
-use crate::wcstringutil::count_newlines;
-use crate::wcstringutil::truncate;
 use crate::wildcard::{ANY_CHAR, ANY_STRING, ANY_STRING_RECURSIVE};
+use fish_common::help_section;
+use fish_wcstringutil::{count_newlines, truncate};
 use std::ops::Range;
 use std::{iter, ops};
 
 /// Handles slices: the square brackets in an expression like $foo[5..4]
 /// Return the length of the slice starting at `in`, or 0 if there is no slice, or None on error.
 /// This never accepts incomplete slices.
-pub fn parse_util_slice_length(input: &wstr) -> Option<usize> {
-    const openc: char = '[';
-    const closec: char = ']';
+pub fn slice_length(input: &wstr) -> Option<usize> {
+    let openc = '[';
+    let closec = ']';
     let mut escaped = false;
 
     // Check for initial opening [
@@ -129,7 +129,7 @@ pub enum MaybeParentheses {
 /// \param out_has_dollar whether the command substitution has the optional leading $.
 /// Return -1 on syntax error, 0 if no subshells exist and 1 on success
 #[allow(clippy::too_many_arguments)]
-pub fn parse_util_locate_cmdsubst_range(
+pub fn locate_cmdsubst_range(
     s: &wstr,
     inout_cursor_offset: &mut usize,
     accept_incomplete: bool,
@@ -142,7 +142,7 @@ pub fn parse_util_locate_cmdsubst_range(
     }
 
     // Defer to the wonky version.
-    let ret = parse_util_locate_cmdsub(
+    let ret = locate_cmdsub(
         s,
         *inout_cursor_offset,
         accept_incomplete,
@@ -167,12 +167,12 @@ pub fn parse_util_locate_cmdsubst_range(
 /// \param cursor_pos the position of the cursor
 /// \param a the start of the searched string
 /// \param b the end of the searched string
-pub fn parse_util_cmdsubst_extent(buff: &wstr, cursor: usize) -> ops::Range<usize> {
+pub fn get_cmdsubst_extent(buff: &wstr, cursor: usize) -> ops::Range<usize> {
     // The tightest command substitution found so far.
     let mut result = 0..buff.len();
     let mut pos = 0;
     loop {
-        let parens = match parse_util_locate_cmdsub(buff, pos, true, None, None) {
+        let parens = match locate_cmdsub(buff, pos, true, None, None) {
             // No subshell found, all done.
             MaybeParentheses::Error | MaybeParentheses::None => break,
             MaybeParentheses::CommandSubstitution(parens) => parens,
@@ -202,7 +202,7 @@ pub fn parse_util_cmdsubst_extent(buff: &wstr, cursor: usize) -> ops::Range<usiz
     result
 }
 
-fn parse_util_locate_cmdsub(
+fn locate_cmdsub(
     input: &wstr,
     cursor: usize,
     allow_incomplete: bool,
@@ -376,34 +376,36 @@ fn parse_util_locate_cmdsub(
 ///
 /// \param buff the string to search for subshells
 /// \param cursor_pos the position of the cursor
-/// \param a the start of the process
-/// \param b the end of the process
 /// \param tokens the tokens in the process
-pub fn parse_util_process_extent(
+pub fn get_process_extent(
     buff: &wstr,
     cursor_pos: usize,
     out_tokens: Option<&mut Vec<Tok>>,
 ) -> ops::Range<usize> {
-    job_or_process_extent(true, buff, cursor_pos, out_tokens)
+    get_job_or_process_extent(true, buff, cursor_pos, out_tokens)
+}
+
+pub fn get_process_first_token_offset(buff: &wstr, cursor_pos: usize) -> Option<usize> {
+    let mut tokens = vec![];
+    get_process_extent(buff, cursor_pos, Some(&mut tokens));
+    tokens.first().map(|tok| tok.offset())
 }
 
 /// Find the beginning and end of the process definition under the cursor
 ///
 /// \param buff the string to search for subshells
 /// \param cursor_pos the position of the cursor
-/// \param a the start of the process
-/// \param b the end of the process
-/// \param tokens the tokens in the process
-pub fn parse_util_job_extent(
+/// \param out_tokens the tokens in the process
+pub fn get_job_extent(
     buff: &wstr,
     cursor_pos: usize,
     out_tokens: Option<&mut Vec<Tok>>,
 ) -> ops::Range<usize> {
-    job_or_process_extent(false, buff, cursor_pos, out_tokens)
+    get_job_or_process_extent(false, buff, cursor_pos, out_tokens)
 }
 
 /// Get the beginning and end of the job or process definition under the cursor.
-fn job_or_process_extent(
+fn get_job_or_process_extent(
     process: bool,
     buff: &wstr,
     cursor_pos: usize,
@@ -411,7 +413,7 @@ fn job_or_process_extent(
 ) -> ops::Range<usize> {
     let mut finished = false;
 
-    let cmdsub_range = parse_util_cmdsubst_extent(buff, cursor_pos);
+    let cmdsub_range = get_cmdsubst_extent(buff, cursor_pos);
     assert!(cursor_pos >= cmdsub_range.start);
     let pos = cursor_pos - cmdsub_range.start;
 
@@ -425,14 +427,14 @@ fn job_or_process_extent(
             break;
         }
         match token.type_ {
-            TokenType::pipe
-            | TokenType::end
-            | TokenType::background
-            | TokenType::andand
-            | TokenType::oror
-            | TokenType::left_brace
-            | TokenType::right_brace
-                if (token.type_ != TokenType::pipe || process) =>
+            TokenType::Pipe
+            | TokenType::End
+            | TokenType::Background
+            | TokenType::AndAnd
+            | TokenType::OrOr
+            | TokenType::LeftBrace
+            | TokenType::RightBrace
+                if (token.type_ != TokenType::Pipe || process) =>
             {
                 if tok_begin >= pos {
                     finished = true;
@@ -455,8 +457,8 @@ fn job_or_process_extent(
 ///
 /// \param buff the string to search for subshells
 /// \param cursor_pos the position of the cursor
-pub fn parse_util_token_extent(buff: &wstr, cursor_pos: usize) -> (Range<usize>, Range<usize>) {
-    let cmdsubst_range = parse_util_cmdsubst_extent(buff, cursor_pos);
+pub fn get_token_extent(buff: &wstr, cursor_pos: usize) -> (Range<usize>, Range<usize>) {
+    let cmdsubst_range = get_cmdsubst_extent(buff, cursor_pos);
     let cmdsubst_begin = cmdsubst_range.start;
 
     // pos is equivalent to cursor_pos within the range of the command substitution {begin, end}.
@@ -475,7 +477,7 @@ pub fn parse_util_token_extent(buff: &wstr, cursor_pos: usize) -> (Range<usize>,
         let mut tok_end = tok_begin;
 
         // Calculate end of token.
-        if token.type_ == TokenType::string {
+        if token.type_ == TokenType::String {
             tok_end += token.length();
         }
 
@@ -489,14 +491,14 @@ pub fn parse_util_token_extent(buff: &wstr, cursor_pos: usize) -> (Range<usize>,
 
         // If cursor is inside the token, this is the token we are looking for. If so, set
         // cur_begin and cur_end and break.
-        if token.type_ == TokenType::string && tok_end >= offset_within_cmdsubst {
+        if token.type_ == TokenType::String && tok_end >= offset_within_cmdsubst {
             cur_begin = cmdsubst_begin + token.offset();
             cur_end = cur_begin + token.length();
             break;
         }
 
         // Remember previous string token.
-        if token.type_ == TokenType::string {
+        if token.type_ == TokenType::String {
             prev_begin = cmdsubst_begin + token.offset();
             prev_end = prev_begin + token.length();
         }
@@ -509,7 +511,7 @@ pub fn parse_util_token_extent(buff: &wstr, cursor_pos: usize) -> (Range<usize>,
 }
 
 /// Get the line number at the specified character offset.
-pub fn parse_util_lineno(s: &wstr, offset: usize) -> usize {
+pub fn lineno(s: &wstr, offset: usize) -> usize {
     // Return the line number of position offset, starting with 1.
     if s.is_empty() {
         return 1;
@@ -520,7 +522,7 @@ pub fn parse_util_lineno(s: &wstr, offset: usize) -> usize {
 }
 
 /// Calculate the line number of the specified cursor position.
-pub fn parse_util_get_line_from_offset(s: &wstr, pos: usize) -> isize {
+pub fn get_line_from_offset(s: &wstr, pos: usize) -> isize {
     // Return the line pos is on, or -1 if it's after the end.
     if pos > s.len() {
         return -1;
@@ -529,7 +531,7 @@ pub fn parse_util_get_line_from_offset(s: &wstr, pos: usize) -> isize {
 }
 
 /// Get the offset of the first character on the specified line.
-pub fn parse_util_get_offset_from_line(s: &wstr, line: i32) -> Option<usize> {
+pub fn get_offset_from_line(s: &wstr, line: i32) -> Option<usize> {
     // Return the first position on line X, counting from 0.
     if line < 0 {
         return None;
@@ -549,9 +551,9 @@ pub fn parse_util_get_offset_from_line(s: &wstr, line: i32) -> Option<usize> {
 }
 
 /// Return the total offset of the buffer for the cursor position nearest to the specified position.
-pub fn parse_util_get_offset(s: &wstr, line: i32, line_offset: isize) -> Option<usize> {
-    let off = parse_util_get_offset_from_line(s, line)?;
-    let off2 = parse_util_get_offset_from_line(s, line + 1).unwrap_or(s.len() + 1);
+pub fn get_offset(s: &wstr, line: i32, line_offset: isize) -> Option<usize> {
+    let off = get_offset_from_line(s, line)?;
+    let off2 = get_offset_from_line(s, line + 1).unwrap_or(s.len() + 1);
 
     let mut line_offset = line_offset as usize;
     if line_offset >= off2 - off - 1 {
@@ -563,9 +565,9 @@ pub fn parse_util_get_offset(s: &wstr, line: i32, line_offset: isize) -> Option<
 
 /// Return the given string, unescaping wildcard characters but not performing any other character
 /// transformation.
-pub fn parse_util_unescape_wildcards(s: &wstr) -> WString {
+pub fn unescape_wildcards(s: &wstr) -> WString {
     let mut result = WString::with_capacity(s.len());
-    let unesc_qmark = !feature_test(FeatureFlag::qmark_noglob);
+    let unesc_qmark = !feature_test(FeatureFlag::QuestionMarkNoGlob);
 
     let mut i = 0;
     while i < s.len() {
@@ -592,8 +594,8 @@ pub fn parse_util_unescape_wildcards(s: &wstr) -> WString {
 }
 
 /// Return if the given string contains wildcard characters.
-pub fn parse_util_contains_wildcards(s: &wstr) -> bool {
-    let unesc_qmark = !feature_test(FeatureFlag::qmark_noglob);
+pub fn contains_wildcards(s: &wstr) -> bool {
+    let unesc_qmark = !feature_test(FeatureFlag::QuestionMarkNoGlob);
 
     let mut i = 0;
     while i < s.len() {
@@ -620,9 +622,9 @@ pub fn parse_util_contains_wildcards(s: &wstr) -> bool {
 
 /// Escape any wildcard characters in the given string. e.g. convert
 /// "a*b" to "a\*b".
-pub fn parse_util_escape_wildcards(s: &wstr) -> WString {
+pub fn escape_wildcards(s: &wstr) -> WString {
     let mut result = WString::with_capacity(s.len());
-    let unesc_qmark = !feature_test(FeatureFlag::qmark_noglob);
+    let unesc_qmark = !feature_test(FeatureFlag::QuestionMarkNoGlob);
 
     for c in s.chars() {
         if c == '*' {
@@ -639,7 +641,7 @@ pub fn parse_util_escape_wildcards(s: &wstr) -> WString {
 }
 
 /// Checks if the specified string is a help option.
-pub fn parse_util_argument_is_help(s: &wstr) -> bool {
+pub fn argument_is_help(s: &wstr) -> bool {
     [L!("-h"), L!("--help")].contains(&s)
 }
 
@@ -694,7 +696,7 @@ fn error_for_character(c: char) -> WString {
 /// Attempts to escape the string 'cmd' using the given quote type, as determined by the quote
 /// character. The quote can be a single quote or double quote, or L'\0' to indicate no quoting (and
 /// thus escaping should be with backslashes). Optionally do not escape tildes.
-pub fn parse_util_escape_string_with_quote(
+pub fn escape_string_with_quote(
     cmd: &wstr,
     quote: Option<char>,
     escape_flags: EscapeFlags,
@@ -751,11 +753,12 @@ pub fn parse_util_escape_string_with_quote(
 
 /// Given a string, parse it as fish code and then return the indents. The return value has the same
 /// size as the string.
-pub fn parse_util_compute_indents(src: &wstr) -> Vec<i32> {
-    compute_indents(src, 0)
+pub fn compute_indents(src: &wstr) -> Vec<i32> {
+    compute_indents_from(src, 0)
 }
 
-fn compute_indents(src: &wstr, initial_indent: i32) -> Vec<i32> {
+/// Compute the indents for the given source, starting with the specified initial indent.
+fn compute_indents_from(src: &wstr, initial_indent: i32) -> Vec<i32> {
     // Make a vector the same size as the input string, which contains the indents. Initialize them
     // to 0.
     let mut indents = vec![0; src.len()];
@@ -769,14 +772,14 @@ fn compute_indents(src: &wstr, initial_indent: i32) -> Vec<i32> {
     // the last node we visited becomes the input indent of the next. I.e. in the case of 'switch
     // foo ; cas', we get an invalid parse tree (since 'cas' is not valid) but we indent it as if it
     // were a case item list.
-    let ast = ast::parse(
-        src,
-        ParseTreeFlags::CONTINUE_AFTER_ERROR
-            | ParseTreeFlags::INCLUDE_COMMENTS
-            | ParseTreeFlags::ACCEPT_INCOMPLETE_TOKENS
-            | ParseTreeFlags::LEAVE_UNTERMINATED,
-        None,
-    );
+    let flags = ParseTreeFlags {
+        continue_after_error: true,
+        include_comments: true,
+        accept_incomplete_tokens: true,
+        leave_unterminated: true,
+        ..Default::default()
+    };
+    let ast = ast::parse(src, flags, None);
     {
         let mut iv = IndentVisitor::new(src, &mut indents, initial_indent);
         iv.visit(ast.top());
@@ -923,7 +926,7 @@ impl<'a> IndentVisitor<'a> {
         let mut was_double_quoted;
         loop {
             was_double_quoted = is_double_quoted;
-            let parens = match parse_util_locate_cmdsubst_range(
+            let parens = match locate_cmdsubst_range(
                 node_src,
                 &mut cursor,
                 /*accept_incomplete=*/ true,
@@ -940,7 +943,7 @@ impl<'a> IndentVisitor<'a> {
             let command = parens.command();
             self.indent_string_part(done..range.start() + command.start, was_double_quoted);
             let cmdsub_contents = &node_src[command.clone()];
-            let indents = compute_indents(cmdsub_contents, self.indent + 1);
+            let indents = compute_indents_from(cmdsub_contents, self.indent + 1);
             self.indents[range.start() + command.start..range.start() + command.end]
                 .copy_from_slice(&indents);
 
@@ -970,7 +973,7 @@ impl<'a> IndentVisitor<'a> {
             let mut callback = |offset| {
                 if !quoted {
                     // Quote open event. Indent unquoted part, including the opening quote.
-                    self.indents[done..start + offset + 1].fill(self.indent);
+                    self.indents[done..=(start + offset)].fill(self.indent);
                     done = start + offset + 1;
                 } else {
                     // Quote close. Don't indent, in case it's a multiline string.
@@ -1063,7 +1066,7 @@ impl<'a> NodeVisitor<'a> for IndentVisitor<'a> {
             Kind::Token(node) => {
                 let token_type = node.token_type();
                 let parent_kind = self.parent.unwrap().kind();
-                if matches!(parent_kind, Kind::BeginHeader(_)) && token_type == ParseTokenType::end
+                if matches!(parent_kind, Kind::BeginHeader(_)) && token_type == ParseTokenType::End
                 {
                     // The newline after "begin" is optional, so it is part of the header.
                     // The header is not in the indented block, so indent the newline here.
@@ -1118,7 +1121,7 @@ impl<'a> NodeVisitor<'a> for IndentVisitor<'a> {
 /// incomplete (e.g. an unclosed quote), an error is not returned and the ParserTestErrorBits::INCOMPLETE bit
 /// is set in the return value. If allow_incomplete is not set, then incomplete strings result in an
 /// error.
-pub fn parse_util_detect_errors(
+pub fn detect_parse_errors(
     buff_src: &wstr,
     mut out_errors: Option<&mut ParseErrorList>,
     allow_incomplete: bool, /*=false*/
@@ -1127,10 +1130,9 @@ pub fn parse_util_detect_errors(
     // allow_incomplete is set.
     let mut has_unclosed_quote_or_subshell = false;
 
-    let parse_flags = if allow_incomplete {
-        ParseTreeFlags::LEAVE_UNTERMINATED
-    } else {
-        ParseTreeFlags::empty()
+    let parse_flags = ParseTreeFlags {
+        leave_unterminated: allow_incomplete,
+        ..Default::default()
     };
 
     // Parse the input string into an ast. Some errors are detected here.
@@ -1141,8 +1143,8 @@ pub fn parse_util_detect_errors(
         // successfully.
         parse_errors.retain(|parse_error| {
             if [
-                ParseErrorCode::tokenizer_unterminated_quote,
-                ParseErrorCode::tokenizer_unterminated_subshell,
+                ParseErrorCode::TokenizerUnterminatedQuote,
+                ParseErrorCode::TokenizerUnterminatedSubshell,
             ]
             .contains(&parse_error.code)
             {
@@ -1171,12 +1173,12 @@ pub fn parse_util_detect_errors(
     }
 
     // Defer to the tree-walking version.
-    parse_util_detect_errors_in_ast(&ast, buff_src, out_errors)
+    detect_parse_errors_in_ast(&ast, buff_src, out_errors)
 }
 
-/// Like parse_util_detect_errors but accepts an already-parsed ast.
+/// Like detect_parse_errors but accepts an already-parsed ast.
 /// The top of the ast is assumed to be a job list.
-pub fn parse_util_detect_errors_in_ast(
+pub fn detect_parse_errors_in_ast(
     ast: &Ast,
     buff_src: &wstr,
     mut out_errors: Option<&mut ParseErrorList>,
@@ -1225,7 +1227,7 @@ pub fn parse_util_detect_errors_in_ast(
             }
             Kind::Argument(arg) => {
                 let arg_src = arg.source(buff_src);
-                res |= parse_util_detect_errors_in_argument(arg, arg_src, &mut out_errors)
+                res |= detect_errors_in_argument(arg, arg_src, &mut out_errors)
                     .err()
                     .unwrap_or_default();
             }
@@ -1313,10 +1315,7 @@ pub fn parse_util_detect_errors_in_ast(
 
 /// Detect errors in the specified string when parsed as an argument list. Returns the text of an
 /// error, or none if no error occurred.
-pub fn parse_util_detect_errors_in_argument_list(
-    arg_list_src: &wstr,
-    prefix: &wstr,
-) -> Result<(), WString> {
+pub fn detect_errors_in_argument_list(arg_list_src: &wstr, prefix: &wstr) -> Result<(), WString> {
     // Helper to return a description of the first error.
     let get_error_text = |errors: &ParseErrorList| {
         assert!(!errors.is_empty(), "Expected an error");
@@ -1330,7 +1329,7 @@ pub fn parse_util_detect_errors_in_argument_list(
 
     // Parse the string as a freestanding argument list.
     let mut errors = ParseErrorList::new();
-    let ast = ast::parse_argument_list(arg_list_src, ParseTreeFlags::empty(), Some(&mut errors));
+    let ast = ast::parse_argument_list(arg_list_src, ParseTreeFlags::default(), Some(&mut errors));
     if !errors.is_empty() {
         return get_error_text(&errors);
     }
@@ -1341,7 +1340,7 @@ pub fn parse_util_detect_errors_in_argument_list(
     let args = &arg_list.arguments;
     for arg in args.iter() {
         let arg_src = arg.source(arg_list_src);
-        if parse_util_detect_errors_in_argument(arg, arg_src, &mut Some(&mut errors)).is_err() {
+        if detect_errors_in_argument(arg, arg_src, &mut Some(&mut errors)).is_err() {
             return get_error_text(&errors);
         }
     }
@@ -1372,7 +1371,7 @@ macro_rules! append_syntax_error_formatted {
             let mut error = ParseError::default();
             error.source_start = $source_location;
             error.source_length = $source_length;
-            error.code = ParseErrorCode::syntax;
+            error.code = ParseErrorCode::Syntax;
             error.text = $text;
             errors.push(error);
         }
@@ -1383,7 +1382,7 @@ macro_rules! append_syntax_error_formatted {
 /// Test if this argument contains any errors. Detected errors include syntax errors in command
 /// substitutions, improperly escaped characters and improper use of the variable expansion
 /// operator.
-pub fn parse_util_detect_errors_in_argument(
+pub fn detect_errors_in_argument(
     arg: &ast::Argument,
     arg_src: &wstr,
     out_errors: &mut Option<&mut ParseErrorList>,
@@ -1449,12 +1448,7 @@ pub fn parse_util_detect_errors_in_argument(
                         {
                             first_dollar -= 1;
                         }
-                        parse_util_expand_variable_error(
-                            unesc.into(),
-                            source_start,
-                            first_dollar,
-                            out_errors,
-                        );
+                        expand_variable_error(unesc.into(), source_start, first_dollar, out_errors);
                     }
                 }
             }
@@ -1469,7 +1463,7 @@ pub fn parse_util_detect_errors_in_argument(
     let mut is_quoted = false;
     while do_loop {
         let mut has_dollar = false;
-        match parse_util_locate_cmdsubst_range(
+        match locate_cmdsubst_range(
             arg_src,
             &mut cursor,
             false,
@@ -1491,11 +1485,9 @@ pub fn parse_util_detect_errors_in_argument(
                     out_errors,
                 );
                 let mut subst_errors = ParseErrorList::new();
-                if let Err(subst_err) = parse_util_detect_errors(
-                    &arg_src[parens.command()],
-                    Some(&mut subst_errors),
-                    false,
-                ) {
+                if let Err(subst_err) =
+                    detect_parse_errors(&arg_src[parens.command()], Some(&mut subst_errors), false)
+                {
                     err |= subst_err;
                 }
 
@@ -1536,7 +1528,7 @@ fn detect_errors_in_job_conjunction(
                 conjunction.source_range().start(),
                 conjunction.source_range().length(),
                 BOOL_AFTER_BACKGROUND_ERROR_MSG,
-                if conjunction.token_type() == ParseTokenType::andand {
+                if conjunction.token_type() == ParseTokenType::AndAnd {
                     L!("&&")
                 } else {
                     L!("||")
@@ -1629,7 +1621,7 @@ fn detect_errors_in_decorated_statement(
     let mut first_arg_is_help = false;
     if let Some(arg) = get_first_arg(&dst.args_or_redirs) {
         let arg_src = arg.source(buff_src);
-        first_arg_is_help = parse_util_argument_is_help(arg_src);
+        first_arg_is_help = argument_is_help(arg_src);
     }
 
     // Get the statement we are part of.
@@ -1648,16 +1640,16 @@ fn detect_errors_in_decorated_statement(
 
     // Check our pipeline position.
     let pipe_pos = if job.continuation.is_empty() {
-        PipelinePosition::none
+        PipelinePosition::None
     } else if is_same_node(&job.statement, st) {
-        PipelinePosition::first
+        PipelinePosition::First
     } else {
-        PipelinePosition::subsequent
+        PipelinePosition::Subsequent
     };
 
     // Check that we don't try to pipe through exec.
-    let is_in_pipeline = pipe_pos != PipelinePosition::none;
-    if is_in_pipeline && decoration == StatementDecoration::exec {
+    let is_in_pipeline = pipe_pos != PipelinePosition::None;
+    if is_in_pipeline && decoration == StatementDecoration::Exec {
         errored = append_syntax_error!(
             parse_errors,
             source_start,
@@ -1670,13 +1662,13 @@ fn detect_errors_in_decorated_statement(
     // This is a somewhat stale check that 'and' and 'or' are not in pipelines, except at the
     // beginning. We can't disallow them as commands entirely because we need to support 'and
     // --help', etc.
-    if pipe_pos == PipelinePosition::subsequent {
+    if pipe_pos == PipelinePosition::Subsequent {
         // We only reject it if we have no decoration.
         // `echo foo | command time something`
         // is entirely fair and valid.
         // Other current decorations like "exec"
         // are already forbidden.
-        if dst.decoration() == StatementDecoration::none {
+        if dst.decoration() == StatementDecoration::None {
             // check if our command is 'and' or 'or'. This is very clumsy; we don't catch e.g. quoted
             // commands.
             let command = dst.command.source(buff_src);
@@ -1711,7 +1703,8 @@ fn detect_errors_in_decorated_statement(
             parse_errors,
             source_start,
             source_length,
-            "$status is not valid as a command. See `help conditions`"
+            "$status is not valid as a command. See `help %s`",
+            help_section!("language#conditions")
         );
     }
 
@@ -1795,7 +1788,7 @@ fn detect_errors_in_decorated_statement(
         }
 
         // Check that we don't do an invalid builtin (issue #1252).
-        if !errored && decoration == StatementDecoration::builtin {
+        if !errored && decoration == StatementDecoration::Builtin {
             let mut command = unexp_command.to_owned();
             if expand_one(
                 &mut command,
@@ -1850,7 +1843,7 @@ fn detect_errors_in_block_redirection_list(
 /// Given a string containing a variable expansion error, append an appropriate error to the errors
 /// list. The global_token_pos is the offset of the token in the larger source, and the dollar_pos
 /// is the offset of the offending dollar sign within the token.
-pub fn parse_util_expand_variable_error(
+pub fn expand_variable_error(
     token: &wstr,
     global_token_pos: usize,
     dollar_pos: usize,
@@ -1896,7 +1889,7 @@ pub fn parse_util_expand_variable_error(
                         global_after_dollar_pos,
                         1,
                         ERROR_BRACKETED_VARIABLE_QUOTED1,
-                        truncate(var_name, var_err_len, None)
+                        truncate(var_name, VAR_ERR_LEN, None)
                     );
                 } else {
                     append_syntax_error!(
@@ -1904,7 +1897,7 @@ pub fn parse_util_expand_variable_error(
                         global_after_dollar_pos,
                         1,
                         ERROR_BRACKETED_VARIABLE1,
-                        truncate(var_name, var_err_len, None),
+                        truncate(var_name, VAR_ERR_LEN, None),
                     );
                 }
             } else {
@@ -1941,7 +1934,7 @@ pub fn parse_util_expand_variable_error(
     }
 
     // We should have appended exactly one error.
-    assert!(errors.as_ref().unwrap().len() == start_error_count + 1);
+    assert_eq!(errors.as_ref().unwrap().len(), start_error_count + 1);
 }
 
 localizable_consts!(
@@ -1966,14 +1959,13 @@ localizable_consts!(
 );
 
 /// Maximum length of a variable name to show in error reports before truncation
-const var_err_len: usize = 16;
+const VAR_ERR_LEN: usize = 16;
 
 #[cfg(test)]
 mod tests {
     use super::{
-        BOOL_AFTER_BACKGROUND_ERROR_MSG, parse_util_cmdsubst_extent, parse_util_compute_indents,
-        parse_util_detect_errors, parse_util_escape_string_with_quote, parse_util_process_extent,
-        parse_util_slice_length,
+        BOOL_AFTER_BACKGROUND_ERROR_MSG, compute_indents, detect_parse_errors,
+        escape_string_with_quote, get_cmdsubst_extent, get_process_extent, slice_length,
     };
     use crate::common::EscapeFlags;
     use crate::parse_constants::{
@@ -1981,8 +1973,8 @@ mod tests {
         ERROR_NO_VAR_NAME, ERROR_NOT_ARGV_AT, ERROR_NOT_ARGV_COUNT, ERROR_NOT_ARGV_STAR,
         ERROR_NOT_PID, ERROR_NOT_STATUS,
     };
+    use crate::prelude::*;
     use crate::tests::prelude::*;
-    use crate::wchar::prelude::*;
     use pcre2::utf32::Regex;
 
     #[test]
@@ -2029,7 +2021,7 @@ mod tests {
         macro_rules! validate {
             ($src:expr, $error_text_format:expr) => {
                 let mut errors = vec![];
-                let res = parse_util_detect_errors(L!($src), Some(&mut errors), false);
+                let res = detect_parse_errors(L!($src), Some(&mut errors), false);
                 let fmt = wgettext!($error_text_format);
                 assert!(res.is_err());
                 assert!(
@@ -2063,11 +2055,11 @@ mod tests {
     }
 
     #[test]
-    fn test_parse_util_process_extent() {
+    fn test_get_process_extent() {
         macro_rules! validate {
             ($commandline:literal, $cursor:expr, $expected_range:expr) => {
                 assert_eq!(
-                    parse_util_process_extent(L!($commandline), $cursor, None),
+                    get_process_extent(L!($commandline), $cursor, None),
                     $expected_range
                 );
             };
@@ -2079,31 +2071,28 @@ mod tests {
 
     #[test]
     #[serial]
-    fn test_parse_util_cmdsubst_extent() {
+    fn test_get_cmdsubst_extent() {
         let _cleanup = test_init();
-        const a: &wstr = L!("echo (echo (echo hi");
-        assert_eq!(parse_util_cmdsubst_extent(a, 0), 0..a.len());
-        assert_eq!(parse_util_cmdsubst_extent(a, 1), 0..a.len());
-        assert_eq!(parse_util_cmdsubst_extent(a, 2), 0..a.len());
-        assert_eq!(parse_util_cmdsubst_extent(a, 3), 0..a.len());
+        let a = L!("echo (echo (echo hi");
+        assert_eq!(get_cmdsubst_extent(a, 0), 0..a.len());
+        assert_eq!(get_cmdsubst_extent(a, 1), 0..a.len());
+        assert_eq!(get_cmdsubst_extent(a, 2), 0..a.len());
+        assert_eq!(get_cmdsubst_extent(a, 3), 0..a.len());
+        assert_eq!(get_cmdsubst_extent(a, 8), "echo (".chars().count()..a.len());
         assert_eq!(
-            parse_util_cmdsubst_extent(a, 8),
-            "echo (".chars().count()..a.len()
-        );
-        assert_eq!(
-            parse_util_cmdsubst_extent(a, 17),
+            get_cmdsubst_extent(a, 17),
             "echo (echo (".chars().count()..a.len()
         );
     }
 
     #[test]
     #[serial]
-    fn test_parse_util_slice_length() {
+    fn test_slice_length() {
         let _cleanup = test_init();
-        assert_eq!(parse_util_slice_length(L!("[2]")), Some(3));
-        assert_eq!(parse_util_slice_length(L!("[12]")), Some(4));
-        assert_eq!(parse_util_slice_length(L!("[\"foo\"]")), Some(7));
-        assert_eq!(parse_util_slice_length(L!("[\"foo\"")), None);
+        assert_eq!(slice_length(L!("[2]")), Some(3));
+        assert_eq!(slice_length(L!("[12]")), Some(4));
+        assert_eq!(slice_length(L!("[\"foo\"]")), Some(7));
+        assert_eq!(slice_length(L!("[\"foo\"")), None);
     }
 
     #[test]
@@ -2113,7 +2102,7 @@ mod tests {
         macro_rules! validate {
             ($cmd:expr, $quote:expr, $no_tilde:expr, $expected:expr) => {
                 assert_eq!(
-                    parse_util_escape_string_with_quote(
+                    escape_string_with_quote(
                         L!($cmd),
                         $quote,
                         if $no_tilde {
@@ -2129,7 +2118,7 @@ mod tests {
         macro_rules! validate_no_quoted {
             ($cmd:expr, $quote:expr, $no_tilde:expr, $expected:expr) => {
                 assert_eq!(
-                    parse_util_escape_string_with_quote(
+                    escape_string_with_quote(
                         L!($cmd),
                         $quote,
                         EscapeFlags::NO_QUOTED
@@ -2199,7 +2188,7 @@ mod tests {
                     expected_indents.push(segment.indent);
                 }
             }
-            let indents = parse_util_compute_indents(&text);
+            let indents = compute_indents(&text);
             assert_eq!(indents, expected_indents);
         }
         macro_rules! validate {
