@@ -26,19 +26,18 @@ use crate::parse_constants::{
     SOURCE_LOCATION_UNKNOWN,
 };
 use crate::parse_execution::{EndExecutionReason, ExecutionContext};
-use crate::parse_tree::NodeRef;
-use crate::parse_tree::{LineCounter, ParsedSourceRef, parse_source};
+use crate::parse_tree::{NodeRef, ParsedSourceRef, SourceLineCache, parse_source};
+use crate::portable_atomic::AtomicU64;
 use crate::prelude::*;
 use crate::proc::{JobGroupRef, JobList, JobRef, Pid, ProcStatus, job_reap};
 use crate::signal::{Signal, signal_check_cancel, signal_clear_cancel};
-use crate::util::get_time;
 use crate::wait_handle::WaitHandleStore;
 use crate::wutil::perror;
 use crate::{flog, flogf, function};
-use fish_wchar::WExt;
+use assert_matches::assert_matches;
+use fish_util::get_time;
+use fish_widestring::WExt;
 use libc::c_int;
-#[cfg(not(target_has_atomic = "64"))]
-use portable_atomic::AtomicU64;
 use std::cell::{Ref, RefCell, RefMut};
 use std::ffi::OsStr;
 use std::fs::File;
@@ -47,8 +46,6 @@ use std::num::NonZeroU32;
 use std::os::fd::OwnedFd;
 use std::rc::Rc;
 use std::sync::Arc;
-#[cfg(target_has_atomic = "64")]
-use std::sync::atomic::AtomicU64;
 use std::time::Duration;
 
 pub enum BlockData {
@@ -83,8 +80,8 @@ pub struct Block {
     /// Name of the file that created this block
     pub src_filename: Option<Arc<WString>>,
 
-    /// Line number where this block was created, starting from 1.
-    pub src_lineno: Option<NonZeroU32>,
+    /// The node containing this block, for lazy line number computation.
+    src_node: Option<NodeRef<ast::JobPipeline>>,
 }
 
 impl Block {
@@ -97,6 +94,11 @@ impl Block {
     pub fn wants_pop_env(&self) -> bool {
         self.typ() != BlockType::top
     }
+
+    /// Return the 1-based line number of this block, using a cache.
+    pub fn src_lineno(&self, cache: &mut SourceLineCache) -> Option<NonZeroU32> {
+        self.src_node.as_ref()?.lineno_with_cache(cache)
+    }
 }
 
 impl Block {
@@ -126,7 +128,7 @@ impl Block {
         }
         .to_owned();
 
-        if let Some(src_lineno) = self.src_lineno {
+        if let Some(src_lineno) = self.src_lineno(&mut SourceLineCache::default()) {
             result.push_utfstr(&sprintf!(" (line %d)", src_lineno.get()));
         }
         if let Some(src_filename) = &self.src_filename {
@@ -394,9 +396,8 @@ pub enum CancelBehavior {
 pub struct Parser {
     pub interactive_initialized: RelaxedAtomicBool,
 
-    /// A shared line counter. This is handed out to each execution context
-    /// so they can communicate the line number back to this Parser.
-    line_counter: ScopedRefCell<LineCounter<ast::JobPipeline>>,
+    /// The currently executing Node.
+    current_node: ScopedRefCell<Option<NodeRef<ast::JobPipeline>>>,
 
     /// The jobs associated with this parser.
     job_list: RefCell<JobList>,
@@ -458,7 +459,7 @@ impl Parser {
     pub fn new(variables: EnvStack, cancel_behavior: CancelBehavior) -> Parser {
         let result = Self {
             interactive_initialized: RelaxedAtomicBool::new(false),
-            line_counter: ScopedRefCell::new(LineCounter::empty()),
+            current_node: ScopedRefCell::new(None),
             job_list: RefCell::default(),
             wait_handles: RefCell::default(),
             block_list: RefCell::default(),
@@ -531,7 +532,7 @@ impl Parser {
         let mut error_list = ParseErrorList::new();
         if let Some(ps) = parse_source(
             cmd.to_owned(),
-            ParseTreeFlags::empty(),
+            ParseTreeFlags::default(),
             Some(&mut error_list),
         ) {
             return self.eval_parsed_source(
@@ -571,7 +572,7 @@ impl Parser {
         block_type: BlockType,
         test_only_suppress_stderr: bool,
     ) -> EvalRes {
-        assert!(matches!(block_type, BlockType::top | BlockType::subst));
+        assert_matches!(block_type, BlockType::top | BlockType::subst);
         let job_list = ps.top_job_list();
         if !job_list.is_empty() {
             // Execute the top job list.
@@ -601,12 +602,12 @@ impl Parser {
         block_type: BlockType,
     ) -> Result<EvalRes, WString> {
         use crate::parse_tree::ParsedSource;
-        use crate::parse_util::parse_util_detect_errors_in_ast;
+        use crate::parse_util::detect_parse_errors_in_ast;
         let mut errors = vec![];
-        let ast = ast::parse(&src, ParseTreeFlags::empty(), Some(&mut errors));
+        let ast = ast::parse(&src, ParseTreeFlags::default(), Some(&mut errors));
         let mut errored = ast.errored();
         if !errored {
-            errored = parse_util_detect_errors_in_ast(&ast, &src, Some(&mut errors)).is_err();
+            errored = detect_parse_errors_in_ast(&ast, &src, Some(&mut errors)).is_err();
         }
         if errored {
             let sb = self.get_backtrace(&src, &errors);
@@ -650,8 +651,9 @@ impl Parser {
         test_only_suppress_stderr: bool,
     ) -> EvalRes {
         // Only certain blocks are allowed.
-        assert!(
-            matches!(block_type, BlockType::top | BlockType::subst),
+        assert_matches!(
+            block_type,
+            BlockType::top | BlockType::subst,
             "Invalid block type"
         );
 
@@ -701,15 +703,14 @@ impl Parser {
         let cancel_checker: CancelChecker = Box::new(move || check_cancel_signal().is_some());
         op_ctx.cancel_checker = cancel_checker;
 
-        // Restore the line counter.
-        let ps = node.parsed_source_ref();
-        let restore_line_counter = self.line_counter.scoped_replace(ps.line_counter());
+        // Restore the current pipeline node.
+        let restore_current_node = self.current_node.scoped_replace(None);
 
         // Create a new execution context.
         let mut execution_context = ExecutionContext::new(
-            ps,
+            node.parsed_source_ref(),
             block_io.clone(),
-            &self.line_counter,
+            &self.current_node,
             test_only_suppress_stderr,
         );
 
@@ -720,7 +721,7 @@ impl Parser {
         let new_exec_count = self.libdata().exec_count;
         let new_status_count = self.libdata().status_count;
 
-        ScopeGuarding::commit(restore_line_counter);
+        ScopeGuarding::commit(restore_current_node);
         self.pop_block(scope_block);
 
         job_reap(self, false, Some(block_io)); // reap again
@@ -774,7 +775,10 @@ impl Parser {
     ///
     /// init.fish (line 127): ls|grep pancake
     pub fn current_line(&self) -> WString {
-        let Some(source_offset) = self.line_counter.borrow_mut().source_offset_of_node() else {
+        let Some(node_ref) = self.current_node.borrow().clone() else {
+            return WString::new();
+        };
+        let Some(source_offset) = node_ref.source_offset() else {
             return WString::new();
         };
 
@@ -787,15 +791,16 @@ impl Parser {
         if !self.is_interactive() || self.is_function() {
             if let Some(file) = file {
                 prefix.push_utfstr(&wgettext_fmt!(
-                    "%s (line %d): ",
+                    "%s (line %d):",
                     &user_presentable_path(&file, self.vars()),
                     lineno
                 ));
             } else if self.libdata().within_fish_init {
-                prefix.push_utfstr(&wgettext_fmt!("Startup (line %d): ", lineno));
+                prefix.push_utfstr(&wgettext_fmt!("Startup (line %d):", lineno));
             } else {
-                prefix.push_utfstr(&wgettext_fmt!("Standard input (line %d): ", lineno));
+                prefix.push_utfstr(&wgettext_fmt!("Standard input (line %d):", lineno));
             }
+            prefix.push(' ');
         }
 
         let skip_caret = self.is_interactive() && !self.is_function();
@@ -807,7 +812,7 @@ impl Parser {
         };
 
         let mut line_info = empty_error.describe_with_prefix(
-            self.line_counter.borrow().get_source(),
+            node_ref.source_str(),
             &prefix,
             self.is_interactive(),
             skip_caret,
@@ -822,16 +827,18 @@ impl Parser {
 
     /// Returns the current line number, indexed from 1.
     pub fn get_lineno(&self) -> Option<NonZeroU32> {
-        // The offset is 0 based; the number is 1 based.
-        self.line_counter
-            .borrow_mut()
-            .line_offset_of_node()
-            .map(|offset| NonZeroU32::new(offset.saturating_add(1)).unwrap())
+        self.current_node.borrow().as_ref().and_then(|n| n.lineno())
     }
 
     /// Returns the current line number, indexed from 1, or zero if not sourced.
     pub fn get_lineno_for_display(&self) -> u32 {
-        self.get_lineno().map_or(0, |val| val.get())
+        self.get_lineno().map_or(0, |n| n.get())
+    }
+
+    /// Returns a NodeRef to the current node being executed, if any.
+    /// This can be used for lazy line number computation.
+    pub fn current_node_ref(&self) -> Option<NodeRef<ast::JobPipeline>> {
+        self.current_node.borrow().clone()
     }
 
     /// Return whether we are currently evaluating a "block" such as an if statement.
@@ -954,7 +961,7 @@ impl Parser {
         self.vars().get_last_statuses()
     }
     pub fn set_last_statuses(&self, s: Statuses) {
-        self.vars().set_last_statuses(s)
+        self.vars().set_last_statuses(s);
     }
 
     /// Cover of vars().set(), which also fires any returned event handlers.
@@ -1022,8 +1029,8 @@ impl Parser {
 
     /// Pushes a new block. Returns an id (index) of the block, which is stored in the parser.
     pub fn push_block(&self, mut block: Block) -> BlockId {
-        block.src_lineno = self.get_lineno();
         block.src_filename = self.current_filename();
+        block.src_node.clone_from(&self.current_node.borrow());
         if block.typ() != BlockType::top {
             let new_scope = block.typ() == BlockType::function_call { shadows: true };
             self.vars().push(new_scope);
@@ -1038,7 +1045,7 @@ impl Parser {
     pub fn pop_block(&self, expected: BlockId) {
         let block = {
             let mut block_list = self.block_list.borrow_mut();
-            assert!(expected.0 == block_list.len() - 1);
+            assert_eq!(expected.0, block_list.len() - 1);
             block_list.pop().unwrap()
         };
         if block.wants_pop_env() {
@@ -1184,11 +1191,13 @@ impl Parser {
 
         let prefix = if let Some(filename) = self.current_filename() {
             if which_line > 0 {
-                wgettext_fmt!(
-                    "%s (line %u): ",
+                let mut prefix = wgettext_fmt!(
+                    "%s (line %u):",
                     user_presentable_path(&filename, self.vars()),
                     which_line
-                )
+                );
+                prefix.push(' ');
+                prefix
             } else {
                 sprintf!("%s: ", user_presentable_path(&filename, self.vars()))
             }
@@ -1213,7 +1222,7 @@ impl Parser {
                 Some(BlockData::Function { name, .. }) => {
                     function::get_props(name).and_then(|props| props.definition_file.clone())
                 }
-                Some(BlockData::Source { file }) => Some(file.clone()),
+                Some(BlockData::Source { file, .. }) => Some(file.clone()),
                 _ => None,
             })
             .or_else(|| self.libdata().current_filename.clone())
@@ -1227,6 +1236,7 @@ impl Parser {
 
     /// Return a string representing the current stack trace.
     pub fn stack_trace(&self) -> WString {
+        let mut line_cache = SourceLineCache::default();
         self.blocks_iter_rev()
             // Stop at event handler. No reason to believe that any other code is relevant.
             // It might make sense in the future to continue printing the stack trace of the code
@@ -1234,7 +1244,7 @@ impl Parser {
             // detect that.
             .take_while(|b| b.typ() != BlockType::event)
             .fold(WString::new(), |mut trace, b| {
-                append_block_description_to_stack_trace(self, &b, &mut trace);
+                append_block_description_to_stack_trace(self, &b, &mut trace, &mut line_cache);
                 trace
             })
     }
@@ -1286,7 +1296,7 @@ impl Parser {
             .is_some_and(|var| var.as_list() == [color_theme])
         {
             return;
-        };
+        }
         flogf!(
             reader,
             "Setting %s to %s",
@@ -1361,8 +1371,13 @@ fn print_profile(items: &[ProfileItem], out: &mut File) {
 }
 
 /// Append stack trace info for the block `b` to `trace`.
-fn append_block_description_to_stack_trace(parser: &Parser, b: &Block, trace: &mut WString) {
-    let mut print_call_site = false;
+fn append_block_description_to_stack_trace(
+    parser: &Parser,
+    b: &Block,
+    trace: &mut WString,
+    line_cache: &mut SourceLineCache,
+) {
+    let mut print_source_location = false;
     match b.typ() {
         BlockType::function_call { .. } => {
             let Some(BlockData::Function { name, args, .. }) = b.data() else {
@@ -1381,21 +1396,23 @@ fn append_block_description_to_stack_trace(parser: &Parser, b: &Block, trace: &m
                     args_str.push_utfstr(&escape_string(
                         arg,
                         EscapeStringStyle::Script(EscapeFlags::NO_QUOTED),
-                    ))
+                    ));
                 } else {
                     args_str.push_str("\"\"");
                 }
             }
             if !args_str.is_empty() {
+                trace.push(' ');
                 // TODO: Escape these.
-                trace.push_utfstr(&wgettext_fmt!(" with arguments '%s'", args_str));
+                trace.push_utfstr(&wgettext_fmt!("with arguments '%s'", args_str));
             }
             trace.push('\n');
-            print_call_site = true;
+            print_source_location = true;
         }
         BlockType::subst => {
-            trace.push_utfstr(&wgettext!("in command substitution\n"));
-            print_call_site = true;
+            trace.push_utfstr(&wgettext!("in command substitution"));
+            trace.push('\n');
+            print_source_location = true;
         }
         BlockType::source => {
             let Some(BlockData::Source { file, .. }) = b.data() else {
@@ -1403,18 +1420,20 @@ fn append_block_description_to_stack_trace(parser: &Parser, b: &Block, trace: &m
             };
             let source_dest = file;
             trace.push_utfstr(&wgettext_fmt!(
-                "from sourcing file %s\n",
+                "from sourcing file %s",
                 &user_presentable_path(source_dest, parser.vars())
             ));
-            print_call_site = true;
+            trace.push('\n');
+            print_source_location = true;
         }
         BlockType::event => {
             let Some(BlockData::Event(event)) = b.data() else {
                 unreachable!()
             };
             let description = event::get_desc(parser, event);
-            trace.push_utfstr(&wgettext_fmt!("in event handler: %s\n", &description));
-            print_call_site = true;
+            trace.push_utfstr(&wgettext_fmt!("in event handler: %s", &description));
+            trace.push('\n');
+            print_source_location = true;
         }
         BlockType::top
         | BlockType::begin
@@ -1426,12 +1445,11 @@ fn append_block_description_to_stack_trace(parser: &Parser, b: &Block, trace: &m
         | BlockType::variable_assignment => {}
     }
 
-    if print_call_site {
-        // Print where the function is called.
+    if print_source_location {
         if let Some(file) = b.src_filename.as_ref() {
             trace.push_utfstr(&sprintf!(
                 "\tcalled on line %d of file %s\n",
-                b.src_lineno.map_or(0, |n| n.get()),
+                b.src_lineno(line_cache).map_or(0, |n| n.get()),
                 user_presentable_path(file, parser.vars())
             ));
         } else if parser.libdata().within_fish_init {
@@ -1485,22 +1503,20 @@ pub enum LoopStatus {
 #[cfg(test)]
 mod tests {
     use super::{CancelBehavior, Parser};
-    use crate::ast::{
-        self, Ast, Castable, JobList, JobPipeline, Kind, Node, Traversal, is_same_node,
-    };
+    use crate::ast::{self, Ast, JobList, Kind, Node, Traversal, is_same_node};
+    use crate::common::str2wcstring;
     use crate::env::EnvStack;
     use crate::expand::ExpandFlags;
     use crate::io::{IoBufferfill, IoChain};
     use crate::parse_constants::{
         ParseErrorCode, ParseTokenType, ParseTreeFlags, ParserTestErrorBits, StatementDecoration,
     };
-    use crate::parse_tree::{LineCounter, parse_source};
-    use crate::parse_util::{parse_util_detect_errors, parse_util_detect_errors_in_argument};
+    use crate::parse_util::{detect_errors_in_argument, detect_parse_errors};
     use crate::prelude::*;
     use crate::reader::{fake_scoped_reader, reader_reset_interrupted};
     use crate::signal::{signal_clear_cancel, signal_reset_handlers, signal_set_handlers};
     use crate::tests::prelude::*;
-    use crate::wcstringutil::join_strings;
+    use fish_wcstringutil::join_strings;
     use libc::SIGINT;
     use std::time::Duration;
     #[test]
@@ -1509,12 +1525,12 @@ mod tests {
         let _cleanup = test_init();
         macro_rules! detect_errors {
             ($src:literal) => {
-                parse_util_detect_errors(L!($src), None, true /* accept incomplete */)
+                detect_parse_errors(L!($src), None, true /* accept incomplete */)
             };
         }
 
         fn detect_argument_errors(src: &str) -> Result<(), ParserTestErrorBits> {
-            let src = WString::from_str(src);
+            let src = str2wcstring(src);
             let ast = ast::parse_argument_list(&src, ParseTreeFlags::default(), None);
             if ast.errored() {
                 return Err(ParserTestErrorBits::ERROR);
@@ -1522,7 +1538,7 @@ mod tests {
             let args = &ast.top().arguments;
             let first_arg = args.first().expect("Failed to parse an argument");
             let mut errors = None;
-            parse_util_detect_errors_in_argument(first_arg, first_arg.source(&src), &mut errors)
+            detect_errors_in_argument(first_arg, first_arg.source(&src), &mut errors)
         }
 
         // Testing block nesting
@@ -1631,28 +1647,33 @@ mod tests {
             "redirection after 'end' wrongly reported as error"
         );
 
-        assert!(
-            detect_errors!("true | ") == Err(ParserTestErrorBits::INCOMPLETE),
+        assert_eq!(
+            detect_errors!("true | "),
+            Err(ParserTestErrorBits::INCOMPLETE),
             "unterminated pipe not reported properly"
         );
 
-        assert!(
-            detect_errors!("echo (\nfoo\n  bar") == Err(ParserTestErrorBits::INCOMPLETE),
+        assert_eq!(
+            detect_errors!("echo (\nfoo\n  bar"),
+            Err(ParserTestErrorBits::INCOMPLETE),
             "unterminated multiline subshell not reported properly"
         );
 
-        assert!(
-            detect_errors!("begin ; true ; end | ") == Err(ParserTestErrorBits::INCOMPLETE),
+        assert_eq!(
+            detect_errors!("begin ; true ; end | "),
+            Err(ParserTestErrorBits::INCOMPLETE),
             "unterminated pipe not reported properly"
         );
 
-        assert!(
-            detect_errors!(" | true ") == Err(ParserTestErrorBits::ERROR),
+        assert_eq!(
+            detect_errors!(" | true "),
+            Err(ParserTestErrorBits::ERROR),
             "leading pipe not reported properly"
         );
 
-        assert!(
-            detect_errors!("true | # comment") == Err(ParserTestErrorBits::INCOMPLETE),
+        assert_eq!(
+            detect_errors!("true | # comment"),
+            Err(ParserTestErrorBits::INCOMPLETE),
             "comment after pipe not reported as incomplete"
         );
 
@@ -1661,8 +1682,9 @@ mod tests {
             "comment and newline after pipe wrongly reported as error"
         );
 
-        assert!(
-            detect_errors!("true | ; false ") == Err(ParserTestErrorBits::ERROR),
+        assert_eq!(
+            detect_errors!("true | ; false "),
+            Err(ParserTestErrorBits::ERROR),
             "semicolon after pipe not detected as error"
         );
 
@@ -1691,7 +1713,7 @@ mod tests {
         );
 
         // Within command substitutions, we should be able to detect everything that
-        // parse_util_detect_errors! can detect.
+        // detect_errors! can detect.
         assert!(
             detect_argument_errors("foo(cat | or cat)")
                 .unwrap_err()
@@ -1769,8 +1791,9 @@ mod tests {
             "bogus boolean statement error not detected"
         );
 
-        assert!(
-            detect_errors!("true && ") == Err(ParserTestErrorBits::INCOMPLETE),
+        assert_eq!(
+            detect_errors!("true && "),
+            Err(ParserTestErrorBits::INCOMPLETE),
             "unterminated conjunction not reported properly"
         );
 
@@ -1779,13 +1802,15 @@ mod tests {
             "newline after && reported as error"
         );
 
-        assert!(
-            detect_errors!("true || \n") == Err(ParserTestErrorBits::INCOMPLETE),
+        assert_eq!(
+            detect_errors!("true || \n"),
+            Err(ParserTestErrorBits::INCOMPLETE),
             "unterminated conjunction not reported properly"
         );
 
-        assert!(
-            detect_errors!("begin ; echo hi; }") == Err(ParserTestErrorBits::ERROR),
+        assert_eq!(
+            detect_errors!("begin ; echo hi; }"),
+            Err(ParserTestErrorBits::ERROR),
             "closing of unopened brace statement not reported properly"
         );
 
@@ -2045,38 +2070,37 @@ mod tests {
         let ast = ast::parse(L!("a="), ParseTreeFlags::default(), None);
         assert!(ast.errored());
 
+        let flags = ParseTreeFlags {
+            leave_unterminated: true,
+            ..ParseTreeFlags::default()
+        };
+
         // If we are leaving things unterminated, this should not produce an error.
         // i.e. when typing "a=" at the command line, it should be treated as valid
         // because we don't want to color it as an error.
-        let ast = ast::parse(L!("a="), ParseTreeFlags::LEAVE_UNTERMINATED, None);
+        let ast = ast::parse(L!("a="), flags, None);
         assert!(!ast.errored());
 
         let mut errors = vec![];
-        ast::parse(
-            L!("begin; echo ("),
-            ParseTreeFlags::LEAVE_UNTERMINATED,
-            Some(&mut errors),
+        ast::parse(L!("begin; echo ("), flags, Some(&mut errors));
+        assert_eq!(errors.len(), 1);
+        assert_eq!(
+            errors[0].code,
+            ParseErrorCode::TokenizerUnterminatedSubshell
         );
-        assert!(errors.len() == 1);
-        assert!(errors[0].code == ParseErrorCode::TokenizerUnterminatedSubshell);
 
         errors.clear();
-        ast::parse(
-            L!("for x in ("),
-            ParseTreeFlags::LEAVE_UNTERMINATED,
-            Some(&mut errors),
+        ast::parse(L!("for x in ("), flags, Some(&mut errors));
+        assert_eq!(errors.len(), 1);
+        assert_eq!(
+            errors[0].code,
+            ParseErrorCode::TokenizerUnterminatedSubshell
         );
-        assert!(errors.len() == 1);
-        assert!(errors[0].code == ParseErrorCode::TokenizerUnterminatedSubshell);
 
         errors.clear();
-        ast::parse(
-            L!("begin; echo '"),
-            ParseTreeFlags::LEAVE_UNTERMINATED,
-            Some(&mut errors),
-        );
-        assert!(errors.len() == 1);
-        assert!(errors[0].code == ParseErrorCode::TokenizerUnterminatedQuote);
+        ast::parse(L!("begin; echo '"), flags, Some(&mut errors));
+        assert_eq!(errors.len(), 1);
+        assert_eq!(errors[0].code, ParseErrorCode::TokenizerUnterminatedQuote);
     }
 
     #[test]
@@ -2249,64 +2273,6 @@ mod tests {
         signal_clear_cancel();
     }
 
-    #[test]
-    fn test_line_counter() {
-        let src = L!("echo line1; echo still_line_1;\n\necho line3");
-        let ps = parse_source(src.to_owned(), ParseTreeFlags::default(), None)
-            .expect("Failed to parse source");
-        assert!(!ps.ast.errored());
-        let mut line_counter = ps.line_counter();
-
-        // Test line_offset_of_character_at_offset, both forwards and backwards to exercise the cache.
-        let mut expected = 0;
-        for (idx, c) in src.chars().enumerate() {
-            let line_offset = line_counter.line_offset_of_character_at_offset(idx);
-            assert_eq!(line_offset, expected);
-            if c == '\n' {
-                expected += 1;
-            }
-        }
-        for (idx, c) in src.chars().enumerate().rev() {
-            if c == '\n' {
-                expected -= 1;
-            }
-            let line_offset = line_counter.line_offset_of_character_at_offset(idx);
-            assert_eq!(line_offset, expected);
-        }
-
-        let pipelines: Vec<_> = ps.ast.walk().filter_map(ast::JobPipeline::cast).collect();
-        assert_eq!(pipelines.len(), 3);
-        let src_offsets = [0, 0, 2];
-        assert_eq!(line_counter.source_offset_of_node(), None);
-        assert_eq!(line_counter.line_offset_of_node(), None);
-
-        for (idx, &node) in pipelines.iter().enumerate() {
-            line_counter.node = std::ptr::from_ref(node);
-            assert_eq!(
-                line_counter.source_offset_of_node(),
-                Some(node.source_range().start())
-            );
-            assert_eq!(line_counter.line_offset_of_node(), Some(src_offsets[idx]));
-        }
-
-        for (idx, &node) in pipelines.iter().enumerate().rev() {
-            line_counter.node = std::ptr::from_ref(node);
-            assert_eq!(
-                line_counter.source_offset_of_node(),
-                Some(node.source_range().start())
-            );
-            assert_eq!(line_counter.line_offset_of_node(), Some(src_offsets[idx]));
-        }
-    }
-
-    #[test]
-    fn test_line_counter_empty() {
-        let mut line_counter = LineCounter::<JobPipeline>::empty();
-        assert_eq!(line_counter.line_offset_of_character_at_offset(0), 0);
-        assert_eq!(line_counter.line_offset_of_node(), None);
-        assert_eq!(line_counter.source_offset_of_node(), None);
-    }
-
     // Helper for testing a simple ast traversal.
     // The ast is always for the command 'true;'.
     struct TrueSemiAstTester<'a> {
@@ -2383,7 +2349,11 @@ mod tests {
     #[test]
     fn test_ast() {
         // Light testing of the AST and traversals.
-        let ast = ast::parse(TrueSemiAstTester::TRUE_SEMI, ParseTreeFlags::empty(), None);
+        let ast = ast::parse(
+            TrueSemiAstTester::TRUE_SEMI,
+            ParseTreeFlags::default(),
+            None,
+        );
         let tester = TrueSemiAstTester::new(&ast);
 
         // Walk the AST and collect all nodes.
@@ -2444,7 +2414,7 @@ mod tests {
     #[should_panic]
     fn test_traversal_skip_children_panics() {
         // Test that we panic if we try to skip children of a node that is not the current node.
-        let ast = ast::parse(L!("true;"), ParseTreeFlags::empty(), None);
+        let ast = ast::parse(L!("true;"), ParseTreeFlags::default(), None);
         let mut traversal = ast.walk();
         while let Some(node) = traversal.next() {
             if matches!(node.kind(), ast::Kind::DecoratedStatement(_)) {
@@ -2458,7 +2428,7 @@ mod tests {
     #[should_panic]
     fn test_traversal_parent_panics() {
         // Can only get the parent of nodes still on the stack.
-        let ast = ast::parse(L!("true;"), ParseTreeFlags::empty(), None);
+        let ast = ast::parse(L!("true;"), ParseTreeFlags::default(), None);
         let mut traversal = ast.walk();
         let mut decorated_statement = None;
         while let Some(node) = traversal.next() {
