@@ -3,9 +3,7 @@
 //! the exec library will call proc to create representations of the running jobs as needed.
 
 use crate::ast;
-use crate::common::{
-    Timepoint, WSL, charptr2wcstring, escape, is_windows_subsystem_for_linux, timef,
-};
+use crate::common::{Timepoint, WSL, escape, is_windows_subsystem_for_linux, timef};
 use crate::env::Statuses;
 use crate::event::{self, Event};
 use crate::flog::{flog, flogf};
@@ -14,6 +12,7 @@ use crate::io::IoChain;
 use crate::job_group::{JobGroup, MaybeJobId};
 use crate::parse_tree::NodeRef;
 use crate::parser::{Block, Parser};
+use crate::portable_atomic::AtomicU64;
 use crate::prelude::*;
 use crate::reader::{fish_is_unwinding_for_exit, reader_schedule_prompt_repaint};
 use crate::redirection::RedirectionSpecList;
@@ -22,22 +21,25 @@ use crate::topic_monitor::{GenerationsList, Topic, topic_monitor_principal};
 use crate::wait_handle::{InternalJobId, WaitHandle, WaitHandleRef, WaitHandleStore};
 use crate::wutil::{wbasename, wperror};
 use cfg_if::cfg_if;
-use fish_wchar::ToWString;
+use fish_widestring::ToWString;
 use libc::{
-    _SC_CLK_TCK, EXIT_SUCCESS, SIG_DFL, SIG_IGN, SIGABRT, SIGBUS, SIGCONT, SIGFPE, SIGHUP, SIGILL,
-    SIGINT, SIGKILL, SIGPIPE, SIGQUIT, SIGSEGV, SIGSYS, SIGTTOU, STDOUT_FILENO, WCONTINUED,
-    WEXITSTATUS, WIFCONTINUED, WIFEXITED, WIFSIGNALED, WIFSTOPPED, WNOHANG, WTERMSIG, WUNTRACED,
+    _SC_CLK_TCK, EXIT_SUCCESS, SIG_IGN, SIGABRT, SIGBUS, SIGFPE, SIGILL, SIGINT, SIGPIPE, SIGQUIT,
+    SIGSEGV, SIGSYS, SIGTTOU, STDOUT_FILENO, WCONTINUED, WEXITSTATUS, WIFCONTINUED, WIFEXITED,
+    WIFSIGNALED, WIFSTOPPED, WNOHANG, WTERMSIG, WUNTRACED,
+};
+use nix::{
+    sys::{
+        signal::{SaFlags, SigAction, SigHandler, SigSet, Signal as NixSignal, kill, killpg},
+        wait::{WaitPidFlag, WaitStatus, waitpid},
+    },
+    unistd::getpgrp,
 };
-#[cfg(not(target_has_atomic = "64"))]
-use portable_atomic::AtomicU64;
 use std::cell::{Cell, Ref, RefCell, RefMut};
 use std::fs;
 use std::io::{Read, Write};
 use std::num::NonZeroU32;
 use std::os::fd::RawFd;
 use std::rc::Rc;
-#[cfg(target_has_atomic = "64")]
-use std::sync::atomic::AtomicU64;
 use std::sync::atomic::{AtomicU8, Ordering};
 use std::sync::{Arc, LazyLock, Mutex, OnceLock};
 
@@ -151,12 +153,13 @@ impl ProcStatus {
                or invalid builtin exit code!"
         );
 
-        // Some paranoia.
-        const _zerocode: i32 = ProcStatus::w_exitcode(0, 0);
-        const _: () = assert!(
-            WIFEXITED(_zerocode),
-            "Synthetic exit status not reported as exited"
-        );
+        const {
+            let _zerocode = ProcStatus::w_exitcode(0, 0);
+            assert!(
+                WIFEXITED(_zerocode),
+                "Synthetic exit status not reported as exited"
+            );
+        }
 
         assert!(ret < 256);
         ProcStatus::new(Some(Self::w_exitcode(ret, 0 /* sig */)))
@@ -294,6 +297,11 @@ impl Pid {
         #[allow(clippy::useless_conversion)]
         self.get().into()
     }
+
+    #[inline(always)]
+    pub fn as_nix_pid(&self) -> nix::unistd::Pid {
+        nix::unistd::Pid::from_raw(self.as_pid_t())
+    }
 }
 
 impl std::fmt::Display for Pid {
@@ -481,7 +489,7 @@ impl Process {
         // Only stomp a successful status.
         if self.status().is_success() {
             self.status
-                .set(ProcStatus::from_exit_code(libc::EXIT_FAILURE))
+                .set(ProcStatus::from_exit_code(libc::EXIT_FAILURE));
         }
     }
 
@@ -825,7 +833,7 @@ impl Job {
     /// Return true on success, false if we failed to send the signal.
     pub fn resume(&self) -> bool {
         self.mut_flags().notified_of_stop = false;
-        if !self.signal(SIGCONT) {
+        if !self.signal(NixSignal::SIGCONT) {
             flogf!(
                 proc_pgroup,
                 "Failed to send SIGCONT to procs in job %s",
@@ -843,24 +851,17 @@ impl Job {
 
     /// Send the specified signal to all processes in this job.
     /// Return true on success, false on failure.
-    pub fn signal(&self, signal: i32) -> bool {
+    pub fn signal(&self, signal: NixSignal) -> bool {
         if let Some(pgid) = self.group().get_pgid() {
-            if unsafe { libc::killpg(pgid.as_pid_t(), signal) } == -1 {
-                let strsignal = unsafe { libc::strsignal(signal) };
-                let strsignal = if strsignal.is_null() {
-                    L!("(nil)").to_owned()
-                } else {
-                    charptr2wcstring(strsignal)
-                };
+            if killpg(pgid.as_nix_pid(), signal).is_err() {
+                let strsignal = signal.as_str();
                 wperror(&sprintf!("killpg(%d, %s)", pgid, strsignal));
                 return false;
             }
         } else {
             // This job lives in fish's pgroup and we need to signal procs individually.
             for p in self.external_procs() {
-                if !p.is_completed()
-                    && unsafe { libc::kill(p.pid().unwrap().as_pid_t(), signal) } == -1
-                {
+                if !p.is_completed() && kill(p.pid().unwrap().as_nix_pid(), signal).is_err() {
                     return false;
                 }
             }
@@ -909,7 +910,7 @@ pub fn is_interactive_session() -> bool {
     IS_INTERACTIVE_SESSION.load()
 }
 pub fn set_interactive_session(flag: bool) {
-    IS_INTERACTIVE_SESSION.store(flag)
+    IS_INTERACTIVE_SESSION.store(flag);
 }
 static IS_INTERACTIVE_SESSION: RelaxedAtomicBool = RelaxedAtomicBool::new(false);
 
@@ -918,7 +919,7 @@ pub fn get_login() -> bool {
     IS_LOGIN.load()
 }
 pub fn mark_login() {
-    IS_LOGIN.store(true)
+    IS_LOGIN.store(true);
 }
 static IS_LOGIN: RelaxedAtomicBool = RelaxedAtomicBool::new(false);
 
@@ -929,7 +930,7 @@ pub fn no_exec() -> bool {
     IS_NO_EXEC.load()
 }
 pub fn mark_no_exec() {
-    IS_NO_EXEC.store(true)
+    IS_NO_EXEC.store(true);
 }
 static IS_NO_EXEC: RelaxedAtomicBool = RelaxedAtomicBool::new(false);
 
@@ -987,8 +988,8 @@ pub fn jobs_requiring_warning_on_exit(parser: &Parser) -> JobList {
 /// Print the exit warning for the given jobs, which should have been obtained via
 /// jobs_requiring_warning_on_exit().
 pub fn print_exit_warning_for_jobs(jobs: &JobList) {
-    printf!("%s", wgettext!("There are still jobs active:\n"));
-    printf!("%s", wgettext!("\n   PID  Command\n"));
+    printf!("%s\n", wgettext!("There are still jobs active:"));
+    printf!("\n   PID  %s\n", wgettext!("Command"));
     for j in jobs {
         // Unwrap safety: we can't have a background job that doesn't have an external process and
         // external processes always have a pid set.
@@ -998,14 +999,13 @@ pub fn print_exit_warning_for_jobs(jobs: &JobList) {
             j.command()
         );
     }
-    printf!("\n");
     printf!(
-        "%s",
-        wgettext!("A second attempt to exit will terminate them.\n"),
+        "\n%s\n",
+        wgettext!("A second attempt to exit will terminate them."),
     );
     printf!(
-        "%s",
-        wgettext!("Use 'disown PID' to remove jobs from the list without terminating them.\n"),
+        "%s\n",
+        wgettext!("Use 'disown PID' to remove jobs from the list without terminating them."),
     );
     reader_schedule_prompt_repaint();
 }
@@ -1013,7 +1013,7 @@ pub fn print_exit_warning_for_jobs(jobs: &JobList) {
 /// Use the procfs filesystem to look up how many jiffies of cpu time was used by a given pid. This
 /// function is only available on systems with the procfs file entry 'stat', i.e. Linux.
 pub fn proc_get_jiffies(inpid: Pid) -> ClockTicks {
-    if !have_proc_stat() {
+    if !*HAVE_PROC_STAT {
         return 0;
     }
 
@@ -1087,10 +1087,9 @@ fn handle_child_status(job: &Job, proc: &Process, status: ProcStatus) {
                 job.group().cancel_with_signal(Signal::new(sig));
             } else if !event::is_signal_observed(sig) {
                 // Deliver the SIGINT or SIGQUIT signal to ourself since we're not interactive.
-                let mut act: libc::sigaction = unsafe { std::mem::zeroed() };
-                unsafe { libc::sigemptyset(&mut act.sa_mask) };
-                act.sa_flags = 0;
-                act.sa_sigaction = SIG_DFL;
+                let act =
+                    SigAction::new(SigHandler::SigDfl, SaFlags::empty(), SigSet::empty()).into();
+
                 unsafe {
                     libc::sigaction(sig, &act, std::ptr::null_mut());
                     libc::kill(libc::getpid(), sig);
@@ -1109,14 +1108,14 @@ pub fn proc_wait_any(parser: &Parser) {
 
 /// Send SIGHUP to the list `jobs`, excepting those which are in fish's pgroup.
 pub fn hup_jobs(jobs: &JobList) {
-    let fish_pgrp = crate::nix::getpgrp();
+    let fish_pgrp = getpgrp();
     let mut kill_list = Vec::new();
     for j in jobs {
         let Some(pgid) = j.get_pgid() else { continue };
-        if pgid.as_pid_t() != fish_pgrp && !j.is_completed() {
-            j.signal(SIGHUP);
+        if pgid.as_nix_pid() != fish_pgrp && !j.is_completed() {
+            j.signal(NixSignal::SIGHUP);
             if j.is_stopped() {
-                j.signal(SIGCONT);
+                j.signal(NixSignal::SIGCONT);
             }
 
             // For most applications, the above alone is sufficient for the suspended process to
@@ -1140,7 +1139,7 @@ pub fn hup_jobs(jobs: &JobList) {
         // handle SIGHUP+SIGCONT without running into SIGTTOU.
         std::thread::sleep(std::time::Duration::from_millis(50));
         for j in kill_list.drain(..) {
-            j.signal(SIGKILL);
+            j.signal(NixSignal::SIGKILL);
         }
     }
 }
@@ -1157,16 +1156,19 @@ pub fn add_disowned_job(j: &Job) {
 // Reap any pids in our disowned list that have exited. This is used to avoid zombies.
 fn reap_disowned_pids() {
     let mut disowned_pids = DISOWNED_PIDS.lock().unwrap();
-    // waitpid returns 0 iff the PID/PGID in question has not changed state; remove the pid/pgid
-    // if it has changed or an error occurs (presumably ECHILD because the child does not exist)
-    disowned_pids.retain(|pid| {
-        let mut status: libc::c_int = 0;
-        let ret = unsafe { libc::waitpid(pid.as_pid_t(), &mut status, WNOHANG) };
-        if ret > 0 {
-            flogf!(proc_reap_external, "Reaped disowned PID or PGID %d", pid);
-        }
-        ret == 0
-    });
+    // Remove the pid/pgid if it has exited or an error occurs (presumably ECHILD because the child does not exist).
+    disowned_pids.retain(
+        |pid| match waitpid(pid.as_nix_pid(), Some(WaitPidFlag::WNOHANG)) {
+            Ok(wait_status) => match wait_status {
+                WaitStatus::Exited(_, _) | WaitStatus::Signaled(_, _, _) => {
+                    flogf!(proc_reap_external, "Reaped disowned PID or PGID %d", pid);
+                    false
+                }
+                _ => true,
+            },
+            Err(_) => false,
+        },
+    );
 }
 
 /// A list of pids that have been disowned. They are kept around until either they exit or
@@ -1240,7 +1242,7 @@ fn process_mark_finished_children(parser: &Parser, block_ok: bool, block_io: Opt
                 continue;
             }
             let pid = Pid::new(pid);
-            assert!(pid == proc.pid().unwrap(), "Unexpected waitpid() return");
+            assert_eq!(pid, proc.pid().unwrap(), "Unexpected waitpid() return");
 
             // The process has stopped or exited! Update its status.
             let status = ProcStatus::from_waitpid(statusv);
@@ -1612,12 +1614,9 @@ fn process_clean_after_marking(parser: &Parser, interactive: bool) -> bool {
     printed
 }
 
-pub fn have_proc_stat() -> bool {
-    // Check for /proc/self/stat to see if we are running with Linux-style procfs.
-    static HAVE_PROC_STAT_RESULT: LazyLock<bool> =
-        LazyLock::new(|| fs::metadata("/proc/self/stat").is_ok());
-    *HAVE_PROC_STAT_RESULT
-}
+/// Check for /proc/self/stat to see if we are running with Linux-style procfs.
+pub static HAVE_PROC_STAT: LazyLock<bool> =
+    LazyLock::new(|| fs::metadata("/proc/self/stat").is_ok());
 
 /// The signals that signify crashes to us.
 const CRASHSIGNALS: [libc::c_int; 6] = [SIGABRT, SIGBUS, SIGFPE, SIGILL, SIGSEGV, SIGSYS];
