@@ -18,9 +18,11 @@ use crate::terminal::TerminalCommand::{
 use crate::terminal::{Output, Outputter};
 use crate::threads::assert_is_main_thread;
 use crate::wutil::{perror, wcstoi};
-use fish_wchar::ToWString;
+use fish_widestring::ToWString;
 use libc::{EINVAL, ENOTTY, EPERM, STDIN_FILENO, WNOHANG};
-use std::mem::MaybeUninit;
+use nix::sys::termios::tcgetattr;
+use nix::unistd::getpgrp;
+use std::os::fd::BorrowedFd;
 use std::sync::{
     OnceLock,
     atomic::{AtomicBool, AtomicPtr, Ordering},
@@ -305,7 +307,7 @@ fn set_tty_protocols_active(on_write: fn(), enable: bool) {
         ProtocolKind::Other => flog!(reader, mode, "other extended keys"),
         ProtocolKind::WorkAroundWezTerm => flog!(reader, mode, "wezterm; no modifyOtherKeys"),
         ProtocolKind::None => (),
-    };
+    }
     (on_write)();
 }
 
@@ -391,7 +393,7 @@ impl TtyHandoff {
     pub fn disable_tty_protocols(&mut self) {
         if !self.tty_protocols_applied {
             return; // Already disabled.
-        };
+        }
         self.tty_protocols_applied = false;
         set_tty_protocols_active(self.on_write, false);
     }
@@ -407,12 +409,17 @@ impl TtyHandoff {
 
     /// Save the current tty modes into the owning job group, if we are transferred.
     pub fn save_tty_modes(&mut self) {
-        if let Some(ref mut owner) = self.owner {
-            let mut tmodes = MaybeUninit::uninit();
-            if unsafe { libc::tcgetattr(STDIN_FILENO, tmodes.as_mut_ptr()) } == 0 {
-                owner.tmodes.replace(Some(unsafe { tmodes.assume_init() }));
-            } else if errno::errno().0 != ENOTTY {
-                perror("tcgetattr");
+        let Some(ref mut owner) = self.owner else {
+            return;
+        };
+        match tcgetattr(unsafe { BorrowedFd::borrow_raw(STDIN_FILENO) }) {
+            Ok(modes) => {
+                owner.tmodes.replace(Some(modes));
+            }
+            Err(err) => {
+                if err != nix::Error::ENOTTY {
+                    perror("tcgetattr");
+                }
             }
         }
     }
@@ -427,9 +434,10 @@ impl TtyHandoff {
         let pgid = jg.get_pgid().unwrap();
 
         // It should never be fish's pgroup.
-        let fish_pgrp = crate::nix::getpgrp();
-        assert!(
-            pgid.as_pid_t() != fish_pgrp,
+        let fish_pgrp = getpgrp();
+        assert_ne!(
+            pgid.as_nix_pid(),
+            fish_pgrp,
             "Job should not have fish's pgroup"
         );
 
@@ -454,7 +462,7 @@ impl TtyHandoff {
         } else if current_owner == pgid.get() {
             // Case 2.
             return true;
-        } else if current_owner != pgid.get() && current_owner != fish_pgrp {
+        } else if current_owner != pgid.get() && current_owner != fish_pgrp.as_raw() {
             // Case 3.
             return false;
         }
