@@ -1,28 +1,28 @@
 // Enumeration of all wildcard types.
 
-use fish_common::char_offset;
-use libc::X_OK;
+use fish_common::WILDCARD_RESERVED_BASE;
+use fish_widestring::char_offset;
+use nix::unistd::AccessFlags;
+use std::cell::LazyCell;
 use std::cmp::Ordering;
 use std::collections::HashSet;
 use std::os::unix::fs::MetadataExt;
-use std::sync::LazyLock;
 
 use crate::common::{
-    UnescapeFlags, UnescapeStringStyle, WILDCARD_RESERVED_BASE, WSL,
-    is_windows_subsystem_for_linux, unescape_string,
+    UnescapeFlags, UnescapeStringStyle, WSL, is_windows_subsystem_for_linux, unescape_string,
 };
 use crate::complete::{CompleteFlags, Completion, CompletionReceiver, PROG_COMPLETE_SEP};
 use crate::expand::ExpandFlags;
 use crate::future_feature_flags::FeatureFlag;
 use crate::future_feature_flags::feature_test;
 use crate::prelude::*;
-use crate::wcstringutil::{
-    CaseSensitivity, string_fuzzy_match_string, string_suffixes_string_case_insensitive,
-    strip_executable_suffix,
-};
 use crate::wutil::dir_iter::DirEntryType;
 use crate::wutil::{dir_iter::DirEntry, lwstat, waccess};
 use fish_fallback::wcscasecmp;
+use fish_wcstringutil::{
+    CaseSensitivity, string_fuzzy_match_string, string_suffixes_string_case_insensitive,
+    strip_executable_suffix,
+};
 
 localizable_consts!(
     COMPLETE_EXEC_DESC "command"
@@ -305,8 +305,9 @@ fn file_get_desc(
     is_link: bool,
     definitely_executable: bool,
 ) -> &'static wstr {
-    let is_executable =
-        |filename: &wstr| -> bool { definitely_executable || waccess(filename, X_OK) == 0 };
+    let is_executable = |filename: &wstr| -> bool {
+        definitely_executable || waccess(filename, AccessFlags::X_OK).is_ok()
+    };
 
     if is_link {
         if is_dir {
@@ -375,12 +376,13 @@ fn wildcard_test_flags_then_complete(
 
     // regular file *excludes* broken links - we have no use for them as commands.
     let is_regular_file = entry.check_type().is_some_and(|x| x == DirEntryType::Reg);
-    let is_executable = LazyLock::new(|| is_regular_file && waccess(filepath, X_OK) == 0);
+    let is_executable =
+        LazyCell::new(|| is_regular_file && waccess(filepath, AccessFlags::X_OK).is_ok());
     if executables_only && !*is_executable {
         return false;
     }
 
-    let filepath_stat = LazyLock::new(|| lwstat(filepath));
+    let filepath_stat = LazyCell::new(|| lwstat(filepath));
 
     // For executables on Cygwin, prefer the name without the .exe, to match
     // better with Unix names, but only if there isn't also a file without that
@@ -391,7 +393,7 @@ fn wildcard_test_flags_then_complete(
             let stat_stripped = lwstat(filepath_stripped).map(|stat| (stat.dev(), stat.ino()));
             let stat = filepath_stat.as_ref().map(|stat| (stat.dev(), stat.ino()));
 
-            // TODO(MSRV>=1.88) use if-let-chain
+            // TODO(MSRV>=1.88): feature(let_chains)
             //   if let Ok(stat_stripped) = stat_stripped
             //       && let Ok(stat) = stat
             //       && stat_stripped == stat
@@ -448,7 +450,6 @@ fn wildcard_test_flags_then_complete(
 }
 
 mod expander {
-    use libc::F_OK;
 
     use crate::{
         path::append_path_component,
@@ -577,7 +578,7 @@ mod expander {
 
                 if allow_fuzzy
                     && self.resolved_completions.len() == before
-                    && waccess(&intermediate_dirpath, F_OK) != 0
+                    && waccess(&intermediate_dirpath, AccessFlags::F_OK).is_err()
                 {
                     assert!(self.flags.contains(ExpandFlags::FOR_COMPLETIONS));
                     if let Ok(mut base_dir_iter) = self.open_dir(base_dir, false) {
@@ -648,8 +649,8 @@ mod expander {
                 // ANY_STRING_RECURSIVE character is present in both the head and the tail.
                 let head_any = wc_segment.slice_to(asr_idx + 1);
                 let any_tail = wc.slice_from(asr_idx);
-                assert!(head_any.chars().next_back().unwrap() == ANY_STRING_RECURSIVE);
-                assert!(any_tail.chars().next().unwrap() == ANY_STRING_RECURSIVE);
+                assert_eq!(head_any.chars().next_back().unwrap(), ANY_STRING_RECURSIVE);
+                assert_eq!(any_tail.chars().next().unwrap(), ANY_STRING_RECURSIVE);
 
                 dir.rewind();
                 self.expand_intermediate_segment(
@@ -685,7 +686,7 @@ mod expander {
 
             if !self.flags.contains(ExpandFlags::FOR_COMPLETIONS) {
                 // Trailing slash and not accepting incomplete, e.g. `echo /xyz/`. Insert this file after checking it exists.
-                if waccess(base_dir, F_OK) == 0 {
+                if waccess(base_dir, AccessFlags::F_OK).is_ok() {
                     self.add_expansion_result(base_dir.to_owned());
                 }
                 return;
@@ -708,7 +709,7 @@ mod expander {
                 let known_dir = need_dir && entry.is_dir();
                 if need_dir && !known_dir {
                     continue;
-                };
+                }
                 if !entry.name.is_empty() && !entry.name.starts_with('.') {
                     self.try_add_completion_result(
                         &(base_dir.to_owned() + entry.name.as_utfstr()),
